{
 "cells": [
  {
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "I have the Hacker News data in the form of Parquet files. It is parsed with my [hn-data-fetcher](https://github.com/adhikasp/hn-data-fetcher) project.\n",
    "\n",
    "I will be using [DuckDB](https://duckdb.org/why_duckdb) to explore the Hacker News data. I found it much faster than SQLite. Both in term of performance and ease of use as I don't need to do indexing on the table to make it workable."
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 1,
   "metadata": {},
   "outputs": [
    {
     "name": "stdout",
     "output_type": "stream",
     "text": [
      "Total rows: 41636178\n"
     ]
    }
   ],
   "source": [
    "import duckdb\n",
    "conn = duckdb.connect()\n",
    "conn.sql(\"CREATE VIEW hn AS SELECT * FROM read_parquet('~/hn-data-fetcher/data_parquet/*/*/*.parquet')\")\n",
    "count = conn.sql(\"SELECT COUNT(*) FROM hn\").fetchone()[0]\n",
    "print(f\"Total rows: {count}\")"
   ]
  },
  {
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "## Schema of the table"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 2,
   "metadata": {},
   "outputs": [
    {
     "name": "stdout",
     "output_type": "stream",
     "text": [
      "┌─────────────┬──────────────┬─────────┬─────────┬─────────┬─────────┐\n",
      "│ column_name │ column_type  │  null   │   key   │ default │  extra  │\n",
      "│   varchar   │   varchar    │ varchar │ varchar │ varchar │ varchar │\n",
      "├─────────────┼──────────────┼─────────┼─────────┼─────────┼─────────┤\n",
      "│ id          │ BIGINT       │ YES     │ NULL    │ NULL    │ NULL    │\n",
      "│ time        │ TIMESTAMP_NS │ YES     │ NULL    │ NULL    │ NULL    │\n",
      "│ by          │ VARCHAR      │ YES     │ NULL    │ NULL    │ NULL    │\n",
      "│ title       │ VARCHAR      │ YES     │ NULL    │ NULL    │ NULL    │\n",
      "│ url         │ VARCHAR      │ YES     │ NULL    │ NULL    │ NULL    │\n",
      "│ text        │ VARCHAR      │ YES     │ NULL    │ NULL    │ NULL    │\n",
      "│ score       │ DOUBLE       │ YES     │ NULL    │ NULL    │ NULL    │\n",
      "│ type        │ VARCHAR      │ YES     │ NULL    │ NULL    │ NULL    │\n",
      "│ parent      │ DOUBLE       │ YES     │ NULL    │ NULL    │ NULL    │\n",
      "│ poll        │ DOUBLE       │ YES     │ NULL    │ NULL    │ NULL    │\n",
      "│ parts       │ INTEGER      │ YES     │ NULL    │ NULL    │ NULL    │\n",
      "│ descendants │ DOUBLE       │ YES     │ NULL    │ NULL    │ NULL    │\n",
      "│ dead        │ INTEGER      │ YES     │ NULL    │ NULL    │ NULL    │\n",
      "│ deleted     │ INTEGER      │ YES     │ NULL    │ NULL    │ NULL    │\n",
      "│ kids        │ BIGINT[]     │ YES     │ NULL    │ NULL    │ NULL    │\n",
      "│ month       │ BIGINT       │ YES     │ NULL    │ NULL    │ NULL    │\n",
      "│ year        │ BIGINT       │ YES     │ NULL    │ NULL    │ NULL    │\n",
      "├─────────────┴──────────────┴─────────┴─────────┴─────────┴─────────┤\n",
      "│ 17 rows                                                  6 columns │\n",
      "└────────────────────────────────────────────────────────────────────┘\n",
      "\n"
     ]
    }
   ],
   "source": [
    "conn.sql(\"DESCRIBE hn\").show()"
   ]
  },
  {
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "## Number of posts and comments"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 3,
   "metadata": {},
   "outputs": [
    {
     "data": {
      "text/html": [
       "\n",
       "<style>\n",
       "  #altair-viz-1cd54a34176a4d39ba4ba2f392351819.vega-embed {\n",
       "    width: 100%;\n",
       "    display: flex;\n",
       "  }\n",
       "\n",
       "  #altair-viz-1cd54a34176a4d39ba4ba2f392351819.vega-embed details,\n",
       "  #altair-viz-1cd54a34176a4d39ba4ba2f392351819.vega-embed details summary {\n",
       "    position: relative;\n",
       "  }\n",
       "</style>\n",
       "<div id=\"altair-viz-1cd54a34176a4d39ba4ba2f392351819\"></div>\n",
       "<script type=\"text/javascript\">\n",
       "  var VEGA_DEBUG = (typeof VEGA_DEBUG == \"undefined\") ? {} : VEGA_DEBUG;\n",
       "  (function(spec, embedOpt){\n",
       "    let outputDiv = document.currentScript.previousElementSibling;\n",
       "    if (outputDiv.id !== \"altair-viz-1cd54a34176a4d39ba4ba2f392351819\") {\n",
       "      outputDiv = document.getElementById(\"altair-viz-1cd54a34176a4d39ba4ba2f392351819\");\n",
       "    }\n",
       "\n",
       "    const paths = {\n",
       "      \"vega\": \"https://cdn.jsdelivr.net/npm/vega@5?noext\",\n",
       "      \"vega-lib\": \"https://cdn.jsdelivr.net/npm/vega-lib?noext\",\n",
       "      \"vega-lite\": \"https://cdn.jsdelivr.net/npm/vega-lite@5.20.1?noext\",\n",
       "      \"vega-embed\": \"https://cdn.jsdelivr.net/npm/vega-embed@6?noext\",\n",
       "    };\n",
       "\n",
       "    function maybeLoadScript(lib, version) {\n",
       "      var key = `${lib.replace(\"-\", \"\")}_version`;\n",
       "      return (VEGA_DEBUG[key] == version) ?\n",
       "        Promise.resolve(paths[lib]) :\n",
       "        new Promise(function(resolve, reject) {\n",
       "          var s = document.createElement('script');\n",
       "          document.getElementsByTagName(\"head\")[0].appendChild(s);\n",
       "          s.async = true;\n",
       "          s.onload = () => {\n",
       "            VEGA_DEBUG[key] = version;\n",
       "            return resolve(paths[lib]);\n",
       "          };\n",
       "          s.onerror = () => reject(`Error loading script: ${paths[lib]}`);\n",
       "          s.src = paths[lib];\n",
       "        });\n",
       "    }\n",
       "\n",
       "    function showError(err) {\n",
       "      outputDiv.innerHTML = `<div class=\"error\" style=\"color:red;\">${err}</div>`;\n",
       "      throw err;\n",
       "    }\n",
       "\n",
       "    function displayChart(vegaEmbed) {\n",
       "      vegaEmbed(outputDiv, spec, embedOpt)\n",
       "        .catch(err => showError(`Javascript Error: ${err.message}<br>This usually means there's a typo in your chart specification. See the javascript console for the full traceback.`));\n",
       "    }\n",
       "\n",
       "    if(typeof define === \"function\" && define.amd) {\n",
       "      requirejs.config({paths});\n",
       "      let deps = [\"vega-embed\"];\n",
       "      require(deps, displayChart, err => showError(`Error loading script: ${err.message}`));\n",
       "    } else {\n",
       "      maybeLoadScript(\"vega\", \"5\")\n",
       "        .then(() => maybeLoadScript(\"vega-lite\", \"5.20.1\"))\n",
       "        .then(() => maybeLoadScript(\"vega-embed\", \"6\"))\n",
       "        .catch(showError)\n",
       "        .then(() => displayChart(vegaEmbed));\n",
       "    }\n",
       "  })({\"config\": {\"view\": {\"continuousWidth\": 300, \"continuousHeight\": 300}}, \"layer\": [{\"mark\": {\"type\": \"bar\"}, \"encoding\": {\"text\": {\"field\": \"count\", \"format\": \",\", \"type\": \"quantitative\"}, \"x\": {\"field\": \"type\", \"title\": \"Content Type\", \"type\": \"nominal\"}, \"y\": {\"field\": \"count\", \"title\": \"Count\", \"type\": \"quantitative\"}}, \"title\": \"Distribution of Hacker News Content Types\"}, {\"mark\": {\"type\": \"text\", \"align\": \"center\", \"dy\": -10}, \"encoding\": {\"text\": {\"field\": \"count\", \"format\": \",\", \"type\": \"quantitative\"}, \"x\": {\"field\": \"type\", \"title\": \"Content Type\", \"type\": \"nominal\"}, \"y\": {\"field\": \"count\", \"title\": \"Count\", \"type\": \"quantitative\"}}, \"title\": \"Distribution of Hacker News Content Types\"}], \"data\": {\"name\": \"data-465cee192f033dd062d1c42dd05a254c\"}, \"width\": 600, \"$schema\": \"https://vega.github.io/schema/vega-lite/v5.20.1.json\", \"datasets\": {\"data-465cee192f033dd062d1c42dd05a254c\": [{\"type\": \"pollopt\", \"count\": 14879}, {\"type\": \"poll\", \"count\": 2139}, {\"type\": \"story\", \"count\": 5375784}, {\"type\": \"job\", \"count\": 16955}, {\"type\": \"comment\", \"count\": 36226421}]}}, {\"mode\": \"vega-lite\"});\n",
       "</script>"
      ],
      "text/plain": [
       "alt.LayerChart(...)"
      ]
     },
     "execution_count": 3,
     "metadata": {},
     "output_type": "execute_result"
    }
   ],
   "source": [
    "import altair as alt\n",
    "\n",
    "df = conn.sql(\"\"\"\n",
    "    SELECT \n",
    "        type,\n",
    "        COUNT(*) as count\n",
    "    FROM hn \n",
    "    GROUP BY type\n",
    "\"\"\").df()\n",
    "\n",
    "bar = alt.Chart(df).mark_bar().encode(\n",
    "    x=alt.X('type:N', title='Content Type'),\n",
    "    y=alt.Y('count:Q', title='Count'),\n",
    "    text=alt.Text('count:Q', format=',')\n",
    ").properties(\n",
    "    title='Distribution of Hacker News Content Types',\n",
    "    width=600\n",
    ")\n",
    "text = bar.mark_text(\n",
    "    align='center',\n",
    "    dy=-10  # Move text up by 10 pixels\n",
    ").encode(\n",
    "    text=alt.Text('count:Q', format=',')\n",
    ")\n",
    "chart = alt.layer(bar, text)\n",
    "chart\n"
   ]
  },
  {
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "Majority of the content are comments and posts, which is not surprising."
   ]
  },
  {
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "### Number of comments and story over time"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 4,
   "metadata": {},
   "outputs": [
    {
     "data": {
      "text/html": [
       "\n",
       "<style>\n",
       "  #altair-viz-ddb36dab5b51476fb9e321acaa425a61.vega-embed {\n",
       "    width: 100%;\n",
       "    display: flex;\n",
       "  }\n",
       "\n",
       "  #altair-viz-ddb36dab5b51476fb9e321acaa425a61.vega-embed details,\n",
       "  #altair-viz-ddb36dab5b51476fb9e321acaa425a61.vega-embed details summary {\n",
       "    position: relative;\n",
       "  }\n",
       "</style>\n",
       "<div id=\"altair-viz-ddb36dab5b51476fb9e321acaa425a61\"></div>\n",
       "<script type=\"text/javascript\">\n",
       "  var VEGA_DEBUG = (typeof VEGA_DEBUG == \"undefined\") ? {} : VEGA_DEBUG;\n",
       "  (function(spec, embedOpt){\n",
       "    let outputDiv = document.currentScript.previousElementSibling;\n",
       "    if (outputDiv.id !== \"altair-viz-ddb36dab5b51476fb9e321acaa425a61\") {\n",
       "      outputDiv = document.getElementById(\"altair-viz-ddb36dab5b51476fb9e321acaa425a61\");\n",
       "    }\n",
       "\n",
       "    const paths = {\n",
       "      \"vega\": \"https://cdn.jsdelivr.net/npm/vega@5?noext\",\n",
       "      \"vega-lib\": \"https://cdn.jsdelivr.net/npm/vega-lib?noext\",\n",
       "      \"vega-lite\": \"https://cdn.jsdelivr.net/npm/vega-lite@5.20.1?noext\",\n",
       "      \"vega-embed\": \"https://cdn.jsdelivr.net/npm/vega-embed@6?noext\",\n",
       "    };\n",
       "\n",
       "    function maybeLoadScript(lib, version) {\n",
       "      var key = `${lib.replace(\"-\", \"\")}_version`;\n",
       "      return (VEGA_DEBUG[key] == version) ?\n",
       "        Promise.resolve(paths[lib]) :\n",
       "        new Promise(function(resolve, reject) {\n",
       "          var s = document.createElement('script');\n",
       "          document.getElementsByTagName(\"head\")[0].appendChild(s);\n",
       "          s.async = true;\n",
       "          s.onload = () => {\n",
       "            VEGA_DEBUG[key] = version;\n",
       "            return resolve(paths[lib]);\n",
       "          };\n",
       "          s.onerror = () => reject(`Error loading script: ${paths[lib]}`);\n",
       "          s.src = paths[lib];\n",
       "        });\n",
       "    }\n",
       "\n",
       "    function showError(err) {\n",
       "      outputDiv.innerHTML = `<div class=\"error\" style=\"color:red;\">${err}</div>`;\n",
       "      throw err;\n",
       "    }\n",
       "\n",
       "    function displayChart(vegaEmbed) {\n",
       "      vegaEmbed(outputDiv, spec, embedOpt)\n",
       "        .catch(err => showError(`Javascript Error: ${err.message}<br>This usually means there's a typo in your chart specification. See the javascript console for the full traceback.`));\n",
       "    }\n",
       "\n",
       "    if(typeof define === \"function\" && define.amd) {\n",
       "      requirejs.config({paths});\n",
       "      let deps = [\"vega-embed\"];\n",
       "      require(deps, displayChart, err => showError(`Error loading script: ${err.message}`));\n",
       "    } else {\n",
       "      maybeLoadScript(\"vega\", \"5\")\n",
       "        .then(() => maybeLoadScript(\"vega-lite\", \"5.20.1\"))\n",
       "        .then(() => maybeLoadScript(\"vega-embed\", \"6\"))\n",
       "        .catch(showError)\n",
       "        .then(() => displayChart(vegaEmbed));\n",
       "    }\n",
       "  })({\"config\": {\"view\": {\"continuousWidth\": 300, \"continuousHeight\": 300}}, \"data\": {\"name\": \"data-d328acfed56611e19b8eea2f67070bae\"}, \"mark\": {\"type\": \"line\"}, \"encoding\": {\"color\": {\"field\": \"type\", \"title\": \"Content Type\", \"type\": \"nominal\"}, \"tooltip\": [{\"field\": \"date\", \"format\": \"%B %Y\", \"title\": \"Date\", \"type\": \"temporal\"}, {\"field\": \"type\", \"title\": \"Type\", \"type\": \"nominal\"}, {\"field\": \"count\", \"format\": \",\", \"title\": \"Count\", \"type\": \"quantitative\"}], \"x\": {\"field\": \"date\", \"title\": \"Date\", \"type\": \"temporal\"}, \"y\": {\"field\": \"count\", \"title\": \"Count\", \"type\": \"quantitative\"}}, \"height\": 400, \"title\": \"Monthly Comments and Stories on Hacker News\", \"width\": 800, \"$schema\": \"https://vega.github.io/schema/vega-lite/v5.20.1.json\", \"datasets\": {\"data-d328acfed56611e19b8eea2f67070bae\": [{\"year\": 2007, \"month\": 2, \"type\": \"comment\", \"count\": 821, \"date\": \"2007-02-01T00:00:00\"}, {\"year\": 2007, \"month\": 2, \"type\": \"story\", \"count\": 497, \"date\": \"2007-02-01T00:00:00\"}, {\"year\": 2007, \"month\": 3, \"type\": \"comment\", \"count\": 4335, \"date\": \"2007-03-01T00:00:00\"}, {\"year\": 2007, \"month\": 3, \"type\": \"story\", \"count\": 1726, \"date\": \"2007-03-01T00:00:00\"}, {\"year\": 2007, \"month\": 4, \"type\": \"story\", \"count\": 2101, \"date\": \"2007-04-01T00:00:00\"}, {\"year\": 2007, \"month\": 4, \"type\": \"comment\", \"count\": 7966, \"date\": \"2007-04-01T00:00:00\"}, {\"year\": 2007, \"month\": 5, \"type\": \"comment\", \"count\": 5273, \"date\": \"2007-05-01T00:00:00\"}, {\"year\": 2007, \"month\": 5, \"type\": \"story\", \"count\": 2021, \"date\": \"2007-05-01T00:00:00\"}, {\"year\": 2007, \"month\": 6, \"type\": \"comment\", \"count\": 4404, \"date\": \"2007-06-01T00:00:00\"}, {\"year\": 2007, \"month\": 6, \"type\": \"story\", \"count\": 1522, \"date\": \"2007-06-01T00:00:00\"}, {\"year\": 2007, \"month\": 7, \"type\": \"comment\", \"count\": 4313, \"date\": \"2007-07-01T00:00:00\"}, {\"year\": 2007, \"month\": 7, \"type\": \"story\", \"count\": 1640, \"date\": \"2007-07-01T00:00:00\"}, {\"year\": 2007, \"month\": 8, \"type\": \"comment\", \"count\": 7788, \"date\": \"2007-08-01T00:00:00\"}, {\"year\": 2007, \"month\": 8, \"type\": \"story\", \"count\": 2513, \"date\": \"2007-08-01T00:00:00\"}, {\"year\": 2007, \"month\": 9, \"type\": \"story\", \"count\": 2563, \"date\": \"2007-09-01T00:00:00\"}, {\"year\": 2007, \"month\": 9, \"type\": \"comment\", \"count\": 9729, \"date\": \"2007-09-01T00:00:00\"}, {\"year\": 2007, \"month\": 10, \"type\": \"story\", \"count\": 2803, \"date\": \"2007-10-01T00:00:00\"}, {\"year\": 2007, \"month\": 10, \"type\": \"comment\", \"count\": 10147, \"date\": \"2007-10-01T00:00:00\"}, {\"year\": 2007, \"month\": 11, \"type\": \"story\", \"count\": 2413, \"date\": \"2007-11-01T00:00:00\"}, {\"year\": 2007, \"month\": 11, \"type\": \"comment\", \"count\": 7504, \"date\": \"2007-11-01T00:00:00\"}, {\"year\": 2007, \"month\": 12, \"type\": \"story\", \"count\": 2313, \"date\": \"2007-12-01T00:00:00\"}, {\"year\": 2007, \"month\": 12, \"type\": \"comment\", \"count\": 6192, \"date\": \"2007-12-01T00:00:00\"}, {\"year\": 2008, \"month\": 1, \"type\": \"comment\", \"count\": 9501, \"date\": \"2008-01-01T00:00:00\"}, {\"year\": 2008, \"month\": 1, \"type\": \"story\", \"count\": 3270, \"date\": \"2008-01-01T00:00:00\"}, {\"year\": 2008, \"month\": 2, \"type\": \"comment\", \"count\": 13829, \"date\": \"2008-02-01T00:00:00\"}, {\"year\": 2008, \"month\": 2, \"type\": \"story\", \"count\": 4176, \"date\": \"2008-02-01T00:00:00\"}, {\"year\": 2008, \"month\": 3, \"type\": \"comment\", \"count\": 18668, \"date\": \"2008-03-01T00:00:00\"}, {\"year\": 2008, \"month\": 3, \"type\": \"story\", \"count\": 5131, \"date\": \"2008-03-01T00:00:00\"}, {\"year\": 2008, \"month\": 4, \"type\": \"story\", \"count\": 5461, \"date\": \"2008-04-01T00:00:00\"}, {\"year\": 2008, \"month\": 4, \"type\": \"comment\", \"count\": 19834, \"date\": \"2008-04-01T00:00:00\"}, {\"year\": 2008, \"month\": 5, \"type\": \"comment\", \"count\": 20966, \"date\": \"2008-05-01T00:00:00\"}, {\"year\": 2008, \"month\": 5, \"type\": \"story\", \"count\": 5878, \"date\": \"2008-05-01T00:00:00\"}, {\"year\": 2008, \"month\": 6, \"type\": \"story\", \"count\": 5977, \"date\": \"2008-06-01T00:00:00\"}, {\"year\": 2008, \"month\": 6, \"type\": \"comment\", \"count\": 19819, \"date\": \"2008-06-01T00:00:00\"}, {\"year\": 2008, \"month\": 7, \"type\": \"comment\", \"count\": 22893, \"date\": \"2008-07-01T00:00:00\"}, {\"year\": 2008, \"month\": 7, \"type\": \"story\", \"count\": 6858, \"date\": \"2008-07-01T00:00:00\"}, {\"year\": 2008, \"month\": 8, \"type\": \"comment\", \"count\": 21120, \"date\": \"2008-08-01T00:00:00\"}, {\"year\": 2008, \"month\": 8, \"type\": \"story\", \"count\": 6546, \"date\": \"2008-08-01T00:00:00\"}, {\"year\": 2008, \"month\": 9, \"type\": \"comment\", \"count\": 20525, \"date\": \"2008-09-01T00:00:00\"}, {\"year\": 2008, \"month\": 9, \"type\": \"story\", \"count\": 6532, \"date\": \"2008-09-01T00:00:00\"}, {\"year\": 2008, \"month\": 10, \"type\": \"story\", \"count\": 7137, \"date\": \"2008-10-01T00:00:00\"}, {\"year\": 2008, \"month\": 10, \"type\": \"comment\", \"count\": 21557, \"date\": \"2008-10-01T00:00:00\"}, {\"year\": 2008, \"month\": 11, \"type\": \"comment\", \"count\": 23691, \"date\": \"2008-11-01T00:00:00\"}, {\"year\": 2008, \"month\": 11, \"type\": \"story\", \"count\": 6348, \"date\": \"2008-11-01T00:00:00\"}, {\"year\": 2008, \"month\": 12, \"type\": \"comment\", \"count\": 26879, \"date\": \"2008-12-01T00:00:00\"}, {\"year\": 2008, \"month\": 12, \"type\": \"story\", \"count\": 6462, \"date\": \"2008-12-01T00:00:00\"}, {\"year\": 2009, \"month\": 1, \"type\": \"comment\", \"count\": 34802, \"date\": \"2009-01-01T00:00:00\"}, {\"year\": 2009, \"month\": 1, \"type\": \"story\", \"count\": 7794, \"date\": \"2009-01-01T00:00:00\"}, {\"year\": 2009, \"month\": 2, \"type\": \"story\", \"count\": 7545, \"date\": \"2009-02-01T00:00:00\"}, {\"year\": 2009, \"month\": 2, \"type\": \"comment\", \"count\": 29956, \"date\": \"2009-02-01T00:00:00\"}, {\"year\": 2009, \"month\": 3, \"type\": \"comment\", \"count\": 31769, \"date\": \"2009-03-01T00:00:00\"}, {\"year\": 2009, \"month\": 3, \"type\": \"story\", \"count\": 8427, \"date\": \"2009-03-01T00:00:00\"}, {\"year\": 2009, \"month\": 4, \"type\": \"story\", \"count\": 8280, \"date\": \"2009-04-01T00:00:00\"}, {\"year\": 2009, \"month\": 4, \"type\": \"comment\", \"count\": 36371, \"date\": \"2009-04-01T00:00:00\"}, {\"year\": 2009, \"month\": 5, \"type\": \"story\", \"count\": 8806, \"date\": \"2009-05-01T00:00:00\"}, {\"year\": 2009, \"month\": 5, \"type\": \"comment\", \"count\": 37794, \"date\": \"2009-05-01T00:00:00\"}, {\"year\": 2009, \"month\": 6, \"type\": \"comment\", \"count\": 35789, \"date\": \"2009-06-01T00:00:00\"}, {\"year\": 2009, \"month\": 6, \"type\": \"story\", \"count\": 8413, \"date\": \"2009-06-01T00:00:00\"}, {\"year\": 2009, \"month\": 7, \"type\": \"comment\", \"count\": 41643, \"date\": \"2009-07-01T00:00:00\"}, {\"year\": 2009, \"month\": 7, \"type\": \"story\", \"count\": 9613, \"date\": \"2009-07-01T00:00:00\"}, {\"year\": 2009, \"month\": 8, \"type\": \"comment\", \"count\": 49169, \"date\": \"2009-08-01T00:00:00\"}, {\"year\": 2009, \"month\": 8, \"type\": \"story\", \"count\": 10511, \"date\": \"2009-08-01T00:00:00\"}, {\"year\": 2009, \"month\": 9, \"type\": \"comment\", \"count\": 44150, \"date\": \"2009-09-01T00:00:00\"}, {\"year\": 2009, \"month\": 9, \"type\": \"story\", \"count\": 10203, \"date\": \"2009-09-01T00:00:00\"}, {\"year\": 2009, \"month\": 10, \"type\": \"comment\", \"count\": 48160, \"date\": \"2009-10-01T00:00:00\"}, {\"year\": 2009, \"month\": 10, \"type\": \"story\", \"count\": 10883, \"date\": \"2009-10-01T00:00:00\"}, {\"year\": 2009, \"month\": 11, \"type\": \"comment\", \"count\": 41864, \"date\": \"2009-11-01T00:00:00\"}, {\"year\": 2009, \"month\": 11, \"type\": \"story\", \"count\": 10393, \"date\": \"2009-11-01T00:00:00\"}, {\"year\": 2009, \"month\": 12, \"type\": \"story\", \"count\": 10661, \"date\": \"2009-12-01T00:00:00\"}, {\"year\": 2009, \"month\": 12, \"type\": \"comment\", \"count\": 43730, \"date\": \"2009-12-01T00:00:00\"}, {\"year\": 2010, \"month\": 1, \"type\": \"comment\", \"count\": 51772, \"date\": \"2010-01-01T00:00:00\"}, {\"year\": 2010, \"month\": 1, \"type\": \"story\", \"count\": 12415, \"date\": \"2010-01-01T00:00:00\"}, {\"year\": 2010, \"month\": 2, \"type\": \"comment\", \"count\": 51926, \"date\": \"2010-02-01T00:00:00\"}, {\"year\": 2010, \"month\": 2, \"type\": \"story\", \"count\": 12831, \"date\": \"2010-02-01T00:00:00\"}, {\"year\": 2010, \"month\": 3, \"type\": \"story\", \"count\": 13416, \"date\": \"2010-03-01T00:00:00\"}, {\"year\": 2010, \"month\": 3, \"type\": \"comment\", \"count\": 58696, \"date\": \"2010-03-01T00:00:00\"}, {\"year\": 2010, \"month\": 4, \"type\": \"comment\", \"count\": 59185, \"date\": \"2010-04-01T00:00:00\"}, {\"year\": 2010, \"month\": 4, \"type\": \"story\", \"count\": 13475, \"date\": \"2010-04-01T00:00:00\"}, {\"year\": 2010, \"month\": 5, \"type\": \"story\", \"count\": 14629, \"date\": \"2010-05-01T00:00:00\"}, {\"year\": 2010, \"month\": 5, \"type\": \"comment\", \"count\": 67246, \"date\": \"2010-05-01T00:00:00\"}, {\"year\": 2010, \"month\": 6, \"type\": \"story\", \"count\": 13650, \"date\": \"2010-06-01T00:00:00\"}, {\"year\": 2010, \"month\": 6, \"type\": \"comment\", \"count\": 65304, \"date\": \"2010-06-01T00:00:00\"}, {\"year\": 2010, \"month\": 7, \"type\": \"story\", \"count\": 14636, \"date\": \"2010-07-01T00:00:00\"}, {\"year\": 2010, \"month\": 7, \"type\": \"comment\", \"count\": 71527, \"date\": \"2010-07-01T00:00:00\"}, {\"year\": 2010, \"month\": 8, \"type\": \"comment\", \"count\": 67699, \"date\": \"2010-08-01T00:00:00\"}, {\"year\": 2010, \"month\": 8, \"type\": \"story\", \"count\": 14984, \"date\": \"2010-08-01T00:00:00\"}, {\"year\": 2010, \"month\": 9, \"type\": \"comment\", \"count\": 74139, \"date\": \"2010-09-01T00:00:00\"}, {\"year\": 2010, \"month\": 9, \"type\": \"story\", \"count\": 15805, \"date\": \"2010-09-01T00:00:00\"}, {\"year\": 2010, \"month\": 10, \"type\": \"story\", \"count\": 18250, \"date\": \"2010-10-01T00:00:00\"}, {\"year\": 2010, \"month\": 10, \"type\": \"comment\", \"count\": 88648, \"date\": \"2010-10-01T00:00:00\"}, {\"year\": 2010, \"month\": 11, \"type\": \"comment\", \"count\": 79215, \"date\": \"2010-11-01T00:00:00\"}, {\"year\": 2010, \"month\": 11, \"type\": \"story\", \"count\": 17855, \"date\": \"2010-11-01T00:00:00\"}, {\"year\": 2010, \"month\": 12, \"type\": \"story\", \"count\": 18946, \"date\": \"2010-12-01T00:00:00\"}, {\"year\": 2010, \"month\": 12, \"type\": \"comment\", \"count\": 78838, \"date\": \"2010-12-01T00:00:00\"}, {\"year\": 2011, \"month\": 1, \"type\": \"story\", \"count\": 19642, \"date\": \"2011-01-01T00:00:00\"}, {\"year\": 2011, \"month\": 1, \"type\": \"comment\", \"count\": 82468, \"date\": \"2011-01-01T00:00:00\"}, {\"year\": 2011, \"month\": 2, \"type\": \"story\", \"count\": 19497, \"date\": \"2011-02-01T00:00:00\"}, {\"year\": 2011, \"month\": 2, \"type\": \"comment\", \"count\": 85795, \"date\": \"2011-02-01T00:00:00\"}, {\"year\": 2011, \"month\": 3, \"type\": \"comment\", \"count\": 92311, \"date\": \"2011-03-01T00:00:00\"}, {\"year\": 2011, \"month\": 3, \"type\": \"story\", \"count\": 23234, \"date\": \"2011-03-01T00:00:00\"}, {\"year\": 2011, \"month\": 4, \"type\": \"comment\", \"count\": 83499, \"date\": \"2011-04-01T00:00:00\"}, {\"year\": 2011, \"month\": 4, \"type\": \"story\", \"count\": 21594, \"date\": \"2011-04-01T00:00:00\"}, {\"year\": 2011, \"month\": 5, \"type\": \"story\", \"count\": 21228, \"date\": \"2011-05-01T00:00:00\"}, {\"year\": 2011, \"month\": 5, \"type\": \"comment\", \"count\": 77326, \"date\": \"2011-05-01T00:00:00\"}, {\"year\": 2011, \"month\": 6, \"type\": \"comment\", \"count\": 83849, \"date\": \"2011-06-01T00:00:00\"}, {\"year\": 2011, \"month\": 6, \"type\": \"story\", \"count\": 22399, \"date\": \"2011-06-01T00:00:00\"}, {\"year\": 2011, \"month\": 7, \"type\": \"story\", \"count\": 23778, \"date\": \"2011-07-01T00:00:00\"}, {\"year\": 2011, \"month\": 7, \"type\": \"comment\", \"count\": 87024, \"date\": \"2011-07-01T00:00:00\"}, {\"year\": 2011, \"month\": 8, \"type\": \"story\", \"count\": 26596, \"date\": \"2011-08-01T00:00:00\"}, {\"year\": 2011, \"month\": 8, \"type\": \"comment\", \"count\": 86406, \"date\": \"2011-08-01T00:00:00\"}, {\"year\": 2011, \"month\": 9, \"type\": \"comment\", \"count\": 77784, \"date\": \"2011-09-01T00:00:00\"}, {\"year\": 2011, \"month\": 9, \"type\": \"story\", \"count\": 28808, \"date\": \"2011-09-01T00:00:00\"}, {\"year\": 2011, \"month\": 10, \"type\": \"comment\", \"count\": 87215, \"date\": \"2011-10-01T00:00:00\"}, {\"year\": 2011, \"month\": 10, \"type\": \"story\", \"count\": 29344, \"date\": \"2011-10-01T00:00:00\"}, {\"year\": 2011, \"month\": 11, \"type\": \"comment\", \"count\": 81998, \"date\": \"2011-11-01T00:00:00\"}, {\"year\": 2011, \"month\": 11, \"type\": \"story\", \"count\": 30796, \"date\": \"2011-11-01T00:00:00\"}, {\"year\": 2011, \"month\": 12, \"type\": \"story\", \"count\": 27417, \"date\": \"2011-12-01T00:00:00\"}, {\"year\": 2011, \"month\": 12, \"type\": \"comment\", \"count\": 84600, \"date\": \"2011-12-01T00:00:00\"}, {\"year\": 2012, \"month\": 1, \"type\": \"comment\", \"count\": 89494, \"date\": \"2012-01-01T00:00:00\"}, {\"year\": 2012, \"month\": 1, \"type\": \"story\", \"count\": 27973, \"date\": \"2012-01-01T00:00:00\"}, {\"year\": 2012, \"month\": 2, \"type\": \"story\", \"count\": 25914, \"date\": \"2012-02-01T00:00:00\"}, {\"year\": 2012, \"month\": 2, \"type\": \"comment\", \"count\": 83406, \"date\": \"2012-02-01T00:00:00\"}, {\"year\": 2012, \"month\": 3, \"type\": \"story\", \"count\": 30742, \"date\": \"2012-03-01T00:00:00\"}, {\"year\": 2012, \"month\": 3, \"type\": \"comment\", \"count\": 96827, \"date\": \"2012-03-01T00:00:00\"}, {\"year\": 2012, \"month\": 4, \"type\": \"story\", \"count\": 29088, \"date\": \"2012-04-01T00:00:00\"}, {\"year\": 2012, \"month\": 4, \"type\": \"comment\", \"count\": 96053, \"date\": \"2012-04-01T00:00:00\"}, {\"year\": 2012, \"month\": 5, \"type\": \"story\", \"count\": 29974, \"date\": \"2012-05-01T00:00:00\"}, {\"year\": 2012, \"month\": 5, \"type\": \"comment\", \"count\": 103303, \"date\": \"2012-05-01T00:00:00\"}, {\"year\": 2012, \"month\": 6, \"type\": \"comment\", \"count\": 101691, \"date\": \"2012-06-01T00:00:00\"}, {\"year\": 2012, \"month\": 6, \"type\": \"story\", \"count\": 26531, \"date\": \"2012-06-01T00:00:00\"}, {\"year\": 2012, \"month\": 7, \"type\": \"story\", \"count\": 27409, \"date\": \"2012-07-01T00:00:00\"}, {\"year\": 2012, \"month\": 7, \"type\": \"comment\", \"count\": 103428, \"date\": \"2012-07-01T00:00:00\"}, {\"year\": 2012, \"month\": 8, \"type\": \"comment\", \"count\": 111468, \"date\": \"2012-08-01T00:00:00\"}, {\"year\": 2012, \"month\": 8, \"type\": \"story\", \"count\": 25467, \"date\": \"2012-08-01T00:00:00\"}, {\"year\": 2012, \"month\": 9, \"type\": \"comment\", \"count\": 106837, \"date\": \"2012-09-01T00:00:00\"}, {\"year\": 2012, \"month\": 9, \"type\": \"story\", \"count\": 22636, \"date\": \"2012-09-01T00:00:00\"}, {\"year\": 2012, \"month\": 10, \"type\": \"story\", \"count\": 23541, \"date\": \"2012-10-01T00:00:00\"}, {\"year\": 2012, \"month\": 10, \"type\": \"comment\", \"count\": 101278, \"date\": \"2012-10-01T00:00:00\"}, {\"year\": 2012, \"month\": 11, \"type\": \"comment\", \"count\": 100983, \"date\": \"2012-11-01T00:00:00\"}, {\"year\": 2012, \"month\": 11, \"type\": \"story\", \"count\": 23429, \"date\": \"2012-11-01T00:00:00\"}, {\"year\": 2012, \"month\": 12, \"type\": \"story\", \"count\": 23411, \"date\": \"2012-12-01T00:00:00\"}, {\"year\": 2012, \"month\": 12, \"type\": \"comment\", \"count\": 108229, \"date\": \"2012-12-01T00:00:00\"}, {\"year\": 2013, \"month\": 1, \"type\": \"story\", \"count\": 26454, \"date\": \"2013-01-01T00:00:00\"}, {\"year\": 2013, \"month\": 1, \"type\": \"comment\", \"count\": 123646, \"date\": \"2013-01-01T00:00:00\"}, {\"year\": 2013, \"month\": 2, \"type\": \"story\", \"count\": 25605, \"date\": \"2013-02-01T00:00:00\"}, {\"year\": 2013, \"month\": 2, \"type\": \"comment\", \"count\": 121305, \"date\": \"2013-02-01T00:00:00\"}, {\"year\": 2013, \"month\": 3, \"type\": \"story\", \"count\": 28215, \"date\": \"2013-03-01T00:00:00\"}, {\"year\": 2013, \"month\": 3, \"type\": \"comment\", \"count\": 137716, \"date\": \"2013-03-01T00:00:00\"}, {\"year\": 2013, \"month\": 4, \"type\": \"story\", \"count\": 26703, \"date\": \"2013-04-01T00:00:00\"}, {\"year\": 2013, \"month\": 4, \"type\": \"comment\", \"count\": 130637, \"date\": \"2013-04-01T00:00:00\"}, {\"year\": 2013, \"month\": 5, \"type\": \"comment\", \"count\": 132635, \"date\": \"2013-05-01T00:00:00\"}, {\"year\": 2013, \"month\": 5, \"type\": \"story\", \"count\": 26916, \"date\": \"2013-05-01T00:00:00\"}, {\"year\": 2013, \"month\": 6, \"type\": \"comment\", \"count\": 134983, \"date\": \"2013-06-01T00:00:00\"}, {\"year\": 2013, \"month\": 6, \"type\": \"story\", \"count\": 27234, \"date\": \"2013-06-01T00:00:00\"}, {\"year\": 2013, \"month\": 7, \"type\": \"story\", \"count\": 26081, \"date\": \"2013-07-01T00:00:00\"}, {\"year\": 2013, \"month\": 7, \"type\": \"comment\", \"count\": 135287, \"date\": \"2013-07-01T00:00:00\"}, {\"year\": 2013, \"month\": 8, \"type\": \"story\", \"count\": 26623, \"date\": \"2013-08-01T00:00:00\"}, {\"year\": 2013, \"month\": 8, \"type\": \"comment\", \"count\": 140671, \"date\": \"2013-08-01T00:00:00\"}, {\"year\": 2013, \"month\": 9, \"type\": \"comment\", \"count\": 132971, \"date\": \"2013-09-01T00:00:00\"}, {\"year\": 2013, \"month\": 9, \"type\": \"story\", \"count\": 25173, \"date\": \"2013-09-01T00:00:00\"}, {\"year\": 2013, \"month\": 10, \"type\": \"comment\", \"count\": 142757, \"date\": \"2013-10-01T00:00:00\"}, {\"year\": 2013, \"month\": 10, \"type\": \"story\", \"count\": 27237, \"date\": \"2013-10-01T00:00:00\"}, {\"year\": 2013, \"month\": 11, \"type\": \"comment\", \"count\": 143788, \"date\": \"2013-11-01T00:00:00\"}, {\"year\": 2013, \"month\": 11, \"type\": \"story\", \"count\": 27651, \"date\": \"2013-11-01T00:00:00\"}, {\"year\": 2013, \"month\": 12, \"type\": \"story\", \"count\": 25055, \"date\": \"2013-12-01T00:00:00\"}, {\"year\": 2013, \"month\": 12, \"type\": \"comment\", \"count\": 136040, \"date\": \"2013-12-01T00:00:00\"}, {\"year\": 2014, \"month\": 1, \"type\": \"comment\", \"count\": 133186, \"date\": \"2014-01-01T00:00:00\"}, {\"year\": 2014, \"month\": 1, \"type\": \"story\", \"count\": 26345, \"date\": \"2014-01-01T00:00:00\"}, {\"year\": 2014, \"month\": 2, \"type\": \"comment\", \"count\": 129694, \"date\": \"2014-02-01T00:00:00\"}, {\"year\": 2014, \"month\": 2, \"type\": \"story\", \"count\": 25365, \"date\": \"2014-02-01T00:00:00\"}, {\"year\": 2014, \"month\": 3, \"type\": \"story\", \"count\": 28439, \"date\": \"2014-03-01T00:00:00\"}, {\"year\": 2014, \"month\": 3, \"type\": \"comment\", \"count\": 147907, \"date\": \"2014-03-01T00:00:00\"}, {\"year\": 2014, \"month\": 4, \"type\": \"comment\", \"count\": 138379, \"date\": \"2014-04-01T00:00:00\"}, {\"year\": 2014, \"month\": 4, \"type\": \"story\", \"count\": 26465, \"date\": \"2014-04-01T00:00:00\"}, {\"year\": 2014, \"month\": 5, \"type\": \"comment\", \"count\": 122030, \"date\": \"2014-05-01T00:00:00\"}, {\"year\": 2014, \"month\": 5, \"type\": \"story\", \"count\": 25369, \"date\": \"2014-05-01T00:00:00\"}, {\"year\": 2014, \"month\": 6, \"type\": \"comment\", \"count\": 111038, \"date\": \"2014-06-01T00:00:00\"}, {\"year\": 2014, \"month\": 6, \"type\": \"story\", \"count\": 23270, \"date\": \"2014-06-01T00:00:00\"}, {\"year\": 2014, \"month\": 7, \"type\": \"comment\", \"count\": 117978, \"date\": \"2014-07-01T00:00:00\"}, {\"year\": 2014, \"month\": 7, \"type\": \"story\", \"count\": 24475, \"date\": \"2014-07-01T00:00:00\"}, {\"year\": 2014, \"month\": 8, \"type\": \"comment\", \"count\": 107732, \"date\": \"2014-08-01T00:00:00\"}, {\"year\": 2014, \"month\": 8, \"type\": \"story\", \"count\": 23372, \"date\": \"2014-08-01T00:00:00\"}, {\"year\": 2014, \"month\": 9, \"type\": \"story\", \"count\": 23737, \"date\": \"2014-09-01T00:00:00\"}, {\"year\": 2014, \"month\": 9, \"type\": \"comment\", \"count\": 109854, \"date\": \"2014-09-01T00:00:00\"}, {\"year\": 2014, \"month\": 10, \"type\": \"story\", \"count\": 25321, \"date\": \"2014-10-01T00:00:00\"}, {\"year\": 2014, \"month\": 10, \"type\": \"comment\", \"count\": 119203, \"date\": \"2014-10-01T00:00:00\"}, {\"year\": 2014, \"month\": 11, \"type\": \"story\", \"count\": 23733, \"date\": \"2014-11-01T00:00:00\"}, {\"year\": 2014, \"month\": 11, \"type\": \"comment\", \"count\": 110309, \"date\": \"2014-11-01T00:00:00\"}, {\"year\": 2014, \"month\": 12, \"type\": \"comment\", \"count\": 114036, \"date\": \"2014-12-01T00:00:00\"}, {\"year\": 2014, \"month\": 12, \"type\": \"story\", \"count\": 23570, \"date\": \"2014-12-01T00:00:00\"}, {\"year\": 2015, \"month\": 1, \"type\": \"comment\", \"count\": 124473, \"date\": \"2015-01-01T00:00:00\"}, {\"year\": 2015, \"month\": 1, \"type\": \"story\", \"count\": 26458, \"date\": \"2015-01-01T00:00:00\"}, {\"year\": 2015, \"month\": 2, \"type\": \"comment\", \"count\": 116208, \"date\": \"2015-02-01T00:00:00\"}, {\"year\": 2015, \"month\": 2, \"type\": \"story\", \"count\": 25055, \"date\": \"2015-02-01T00:00:00\"}, {\"year\": 2015, \"month\": 3, \"type\": \"comment\", \"count\": 136931, \"date\": \"2015-03-01T00:00:00\"}, {\"year\": 2015, \"month\": 3, \"type\": \"story\", \"count\": 28305, \"date\": \"2015-03-01T00:00:00\"}, {\"year\": 2015, \"month\": 4, \"type\": \"comment\", \"count\": 132873, \"date\": \"2015-04-01T00:00:00\"}, {\"year\": 2015, \"month\": 4, \"type\": \"story\", \"count\": 27995, \"date\": \"2015-04-01T00:00:00\"}, {\"year\": 2015, \"month\": 5, \"type\": \"story\", \"count\": 27632, \"date\": \"2015-05-01T00:00:00\"}, {\"year\": 2015, \"month\": 5, \"type\": \"comment\", \"count\": 135589, \"date\": \"2015-05-01T00:00:00\"}, {\"year\": 2015, \"month\": 6, \"type\": \"story\", \"count\": 26608, \"date\": \"2015-06-01T00:00:00\"}, {\"year\": 2015, \"month\": 6, \"type\": \"comment\", \"count\": 136410, \"date\": \"2015-06-01T00:00:00\"}, {\"year\": 2015, \"month\": 7, \"type\": \"comment\", \"count\": 140262, \"date\": \"2015-07-01T00:00:00\"}, {\"year\": 2015, \"month\": 7, \"type\": \"story\", \"count\": 28304, \"date\": \"2015-07-01T00:00:00\"}, {\"year\": 2015, \"month\": 8, \"type\": \"comment\", \"count\": 133602, \"date\": \"2015-08-01T00:00:00\"}, {\"year\": 2015, \"month\": 8, \"type\": \"story\", \"count\": 26696, \"date\": \"2015-08-01T00:00:00\"}, {\"year\": 2015, \"month\": 9, \"type\": \"story\", \"count\": 27946, \"date\": \"2015-09-01T00:00:00\"}, {\"year\": 2015, \"month\": 9, \"type\": \"comment\", \"count\": 121502, \"date\": \"2015-09-01T00:00:00\"}, {\"year\": 2015, \"month\": 10, \"type\": \"comment\", \"count\": 141559, \"date\": \"2015-10-01T00:00:00\"}, {\"year\": 2015, \"month\": 10, \"type\": \"story\", \"count\": 31547, \"date\": \"2015-10-01T00:00:00\"}, {\"year\": 2015, \"month\": 11, \"type\": \"comment\", \"count\": 133499, \"date\": \"2015-11-01T00:00:00\"}, {\"year\": 2015, \"month\": 11, \"type\": \"story\", \"count\": 28074, \"date\": \"2015-11-01T00:00:00\"}, {\"year\": 2015, \"month\": 12, \"type\": \"comment\", \"count\": 134505, \"date\": \"2015-12-01T00:00:00\"}, {\"year\": 2015, \"month\": 12, \"type\": \"story\", \"count\": 27594, \"date\": \"2015-12-01T00:00:00\"}, {\"year\": 2016, \"month\": 1, \"type\": \"comment\", \"count\": 153000, \"date\": \"2016-01-01T00:00:00\"}, {\"year\": 2016, \"month\": 1, \"type\": \"story\", \"count\": 30414, \"date\": \"2016-01-01T00:00:00\"}, {\"year\": 2016, \"month\": 2, \"type\": \"comment\", \"count\": 153206, \"date\": \"2016-02-01T00:00:00\"}, {\"year\": 2016, \"month\": 2, \"type\": \"story\", \"count\": 31023, \"date\": \"2016-02-01T00:00:00\"}, {\"year\": 2016, \"month\": 3, \"type\": \"comment\", \"count\": 162686, \"date\": \"2016-03-01T00:00:00\"}, {\"year\": 2016, \"month\": 3, \"type\": \"story\", \"count\": 31413, \"date\": \"2016-03-01T00:00:00\"}, {\"year\": 2016, \"month\": 4, \"type\": \"comment\", \"count\": 167345, \"date\": \"2016-04-01T00:00:00\"}, {\"year\": 2016, \"month\": 4, \"type\": \"story\", \"count\": 30868, \"date\": \"2016-04-01T00:00:00\"}, {\"year\": 2016, \"month\": 5, \"type\": \"story\", \"count\": 30191, \"date\": \"2016-05-01T00:00:00\"}, {\"year\": 2016, \"month\": 5, \"type\": \"comment\", \"count\": 169269, \"date\": \"2016-05-01T00:00:00\"}, {\"year\": 2016, \"month\": 6, \"type\": \"comment\", \"count\": 165886, \"date\": \"2016-06-01T00:00:00\"}, {\"year\": 2016, \"month\": 6, \"type\": \"story\", \"count\": 29070, \"date\": \"2016-06-01T00:00:00\"}, {\"year\": 2016, \"month\": 7, \"type\": \"story\", \"count\": 28783, \"date\": \"2016-07-01T00:00:00\"}, {\"year\": 2016, \"month\": 7, \"type\": \"comment\", \"count\": 154845, \"date\": \"2016-07-01T00:00:00\"}, {\"year\": 2016, \"month\": 8, \"type\": \"comment\", \"count\": 164011, \"date\": \"2016-08-01T00:00:00\"}, {\"year\": 2016, \"month\": 8, \"type\": \"story\", \"count\": 29932, \"date\": \"2016-08-01T00:00:00\"}, {\"year\": 2016, \"month\": 9, \"type\": \"story\", \"count\": 29375, \"date\": \"2016-09-01T00:00:00\"}, {\"year\": 2016, \"month\": 9, \"type\": \"comment\", \"count\": 177627, \"date\": \"2016-09-01T00:00:00\"}, {\"year\": 2016, \"month\": 10, \"type\": \"comment\", \"count\": 185867, \"date\": \"2016-10-01T00:00:00\"}, {\"year\": 2016, \"month\": 10, \"type\": \"story\", \"count\": 31158, \"date\": \"2016-10-01T00:00:00\"}, {\"year\": 2016, \"month\": 11, \"type\": \"comment\", \"count\": 194089, \"date\": \"2016-11-01T00:00:00\"}, {\"year\": 2016, \"month\": 11, \"type\": \"story\", \"count\": 31375, \"date\": \"2016-11-01T00:00:00\"}, {\"year\": 2016, \"month\": 12, \"type\": \"story\", \"count\": 31575, \"date\": \"2016-12-01T00:00:00\"}, {\"year\": 2016, \"month\": 12, \"type\": \"comment\", \"count\": 181336, \"date\": \"2016-12-01T00:00:00\"}, {\"year\": 2017, \"month\": 1, \"type\": \"story\", \"count\": 32400, \"date\": \"2017-01-01T00:00:00\"}, {\"year\": 2017, \"month\": 1, \"type\": \"comment\", \"count\": 198424, \"date\": \"2017-01-01T00:00:00\"}, {\"year\": 2017, \"month\": 2, \"type\": \"comment\", \"count\": 182734, \"date\": \"2017-02-01T00:00:00\"}, {\"year\": 2017, \"month\": 2, \"type\": \"story\", \"count\": 30348, \"date\": \"2017-02-01T00:00:00\"}, {\"year\": 2017, \"month\": 3, \"type\": \"story\", \"count\": 35698, \"date\": \"2017-03-01T00:00:00\"}, {\"year\": 2017, \"month\": 3, \"type\": \"comment\", \"count\": 206001, \"date\": \"2017-03-01T00:00:00\"}, {\"year\": 2017, \"month\": 4, \"type\": \"story\", \"count\": 31295, \"date\": \"2017-04-01T00:00:00\"}, {\"year\": 2017, \"month\": 4, \"type\": \"comment\", \"count\": 190319, \"date\": \"2017-04-01T00:00:00\"}, {\"year\": 2017, \"month\": 5, \"type\": \"story\", \"count\": 31487, \"date\": \"2017-05-01T00:00:00\"}, {\"year\": 2017, \"month\": 5, \"type\": \"comment\", \"count\": 182549, \"date\": \"2017-05-01T00:00:00\"}, {\"year\": 2017, \"month\": 6, \"type\": \"comment\", \"count\": 180042, \"date\": \"2017-06-01T00:00:00\"}, {\"year\": 2017, \"month\": 6, \"type\": \"story\", \"count\": 30601, \"date\": \"2017-06-01T00:00:00\"}, {\"year\": 2017, \"month\": 7, \"type\": \"story\", \"count\": 30322, \"date\": \"2017-07-01T00:00:00\"}, {\"year\": 2017, \"month\": 7, \"type\": \"comment\", \"count\": 184317, \"date\": \"2017-07-01T00:00:00\"}, {\"year\": 2017, \"month\": 8, \"type\": \"comment\", \"count\": 208728, \"date\": \"2017-08-01T00:00:00\"}, {\"year\": 2017, \"month\": 8, \"type\": \"story\", \"count\": 31180, \"date\": \"2017-08-01T00:00:00\"}, {\"year\": 2017, \"month\": 9, \"type\": \"comment\", \"count\": 194081, \"date\": \"2017-09-01T00:00:00\"}, {\"year\": 2017, \"month\": 9, \"type\": \"story\", \"count\": 30967, \"date\": \"2017-09-01T00:00:00\"}, {\"year\": 2017, \"month\": 10, \"type\": \"comment\", \"count\": 181701, \"date\": \"2017-10-01T00:00:00\"}, {\"year\": 2017, \"month\": 10, \"type\": \"story\", \"count\": 31040, \"date\": \"2017-10-01T00:00:00\"}, {\"year\": 2017, \"month\": 11, \"type\": \"comment\", \"count\": 184153, \"date\": \"2017-11-01T00:00:00\"}, {\"year\": 2017, \"month\": 11, \"type\": \"story\", \"count\": 29419, \"date\": \"2017-11-01T00:00:00\"}, {\"year\": 2017, \"month\": 12, \"type\": \"comment\", \"count\": 190369, \"date\": \"2017-12-01T00:00:00\"}, {\"year\": 2017, \"month\": 12, \"type\": \"story\", \"count\": 28314, \"date\": \"2017-12-01T00:00:00\"}, {\"year\": 2018, \"month\": 1, \"type\": \"story\", \"count\": 31116, \"date\": \"2018-01-01T00:00:00\"}, {\"year\": 2018, \"month\": 1, \"type\": \"comment\", \"count\": 194386, \"date\": \"2018-01-01T00:00:00\"}, {\"year\": 2018, \"month\": 2, \"type\": \"comment\", \"count\": 172081, \"date\": \"2018-02-01T00:00:00\"}, {\"year\": 2018, \"month\": 2, \"type\": \"story\", \"count\": 27811, \"date\": \"2018-02-01T00:00:00\"}, {\"year\": 2018, \"month\": 3, \"type\": \"comment\", \"count\": 201030, \"date\": \"2018-03-01T00:00:00\"}, {\"year\": 2018, \"month\": 3, \"type\": \"story\", \"count\": 32170, \"date\": \"2018-03-01T00:00:00\"}, {\"year\": 2018, \"month\": 4, \"type\": \"comment\", \"count\": 198191, \"date\": \"2018-04-01T00:00:00\"}, {\"year\": 2018, \"month\": 4, \"type\": \"story\", \"count\": 30377, \"date\": \"2018-04-01T00:00:00\"}, {\"year\": 2018, \"month\": 5, \"type\": \"story\", \"count\": 31279, \"date\": \"2018-05-01T00:00:00\"}, {\"year\": 2018, \"month\": 5, \"type\": \"comment\", \"count\": 197667, \"date\": \"2018-05-01T00:00:00\"}, {\"year\": 2018, \"month\": 6, \"type\": \"comment\", \"count\": 198029, \"date\": \"2018-06-01T00:00:00\"}, {\"year\": 2018, \"month\": 6, \"type\": \"story\", \"count\": 29635, \"date\": \"2018-06-01T00:00:00\"}, {\"year\": 2018, \"month\": 7, \"type\": \"story\", \"count\": 28717, \"date\": \"2018-07-01T00:00:00\"}, {\"year\": 2018, \"month\": 7, \"type\": \"comment\", \"count\": 185541, \"date\": \"2018-07-01T00:00:00\"}, {\"year\": 2018, \"month\": 8, \"type\": \"comment\", \"count\": 193035, \"date\": \"2018-08-01T00:00:00\"}, {\"year\": 2018, \"month\": 8, \"type\": \"story\", \"count\": 30308, \"date\": \"2018-08-01T00:00:00\"}, {\"year\": 2018, \"month\": 9, \"type\": \"comment\", \"count\": 186302, \"date\": \"2018-09-01T00:00:00\"}, {\"year\": 2018, \"month\": 9, \"type\": \"story\", \"count\": 27885, \"date\": \"2018-09-01T00:00:00\"}, {\"year\": 2018, \"month\": 10, \"type\": \"story\", \"count\": 30642, \"date\": \"2018-10-01T00:00:00\"}, {\"year\": 2018, \"month\": 10, \"type\": \"comment\", \"count\": 201041, \"date\": \"2018-10-01T00:00:00\"}, {\"year\": 2018, \"month\": 11, \"type\": \"comment\", \"count\": 185866, \"date\": \"2018-11-01T00:00:00\"}, {\"year\": 2018, \"month\": 11, \"type\": \"story\", \"count\": 28327, \"date\": \"2018-11-01T00:00:00\"}, {\"year\": 2018, \"month\": 12, \"type\": \"comment\", \"count\": 191720, \"date\": \"2018-12-01T00:00:00\"}, {\"year\": 2018, \"month\": 12, \"type\": \"story\", \"count\": 27787, \"date\": \"2018-12-01T00:00:00\"}, {\"year\": 2019, \"month\": 1, \"type\": \"comment\", \"count\": 211570, \"date\": \"2019-01-01T00:00:00\"}, {\"year\": 2019, \"month\": 1, \"type\": \"story\", \"count\": 29718, \"date\": \"2019-01-01T00:00:00\"}, {\"year\": 2019, \"month\": 2, \"type\": \"story\", \"count\": 27684, \"date\": \"2019-02-01T00:00:00\"}, {\"year\": 2019, \"month\": 2, \"type\": \"comment\", \"count\": 190556, \"date\": \"2019-02-01T00:00:00\"}, {\"year\": 2019, \"month\": 3, \"type\": \"story\", \"count\": 30668, \"date\": \"2019-03-01T00:00:00\"}, {\"year\": 2019, \"month\": 3, \"type\": \"comment\", \"count\": 226890, \"date\": \"2019-03-01T00:00:00\"}, {\"year\": 2019, \"month\": 4, \"type\": \"story\", \"count\": 29450, \"date\": \"2019-04-01T00:00:00\"}, {\"year\": 2019, \"month\": 4, \"type\": \"comment\", \"count\": 213958, \"date\": \"2019-04-01T00:00:00\"}, {\"year\": 2019, \"month\": 5, \"type\": \"story\", \"count\": 30384, \"date\": \"2019-05-01T00:00:00\"}, {\"year\": 2019, \"month\": 5, \"type\": \"comment\", \"count\": 233120, \"date\": \"2019-05-01T00:00:00\"}, {\"year\": 2019, \"month\": 6, \"type\": \"comment\", \"count\": 221711, \"date\": \"2019-06-01T00:00:00\"}, {\"year\": 2019, \"month\": 6, \"type\": \"story\", \"count\": 29068, \"date\": \"2019-06-01T00:00:00\"}, {\"year\": 2019, \"month\": 7, \"type\": \"story\", \"count\": 29596, \"date\": \"2019-07-01T00:00:00\"}, {\"year\": 2019, \"month\": 7, \"type\": \"comment\", \"count\": 217941, \"date\": \"2019-07-01T00:00:00\"}, {\"year\": 2019, \"month\": 8, \"type\": \"story\", \"count\": 30840, \"date\": \"2019-08-01T00:00:00\"}, {\"year\": 2019, \"month\": 8, \"type\": \"comment\", \"count\": 233339, \"date\": \"2019-08-01T00:00:00\"}, {\"year\": 2019, \"month\": 9, \"type\": \"story\", \"count\": 30109, \"date\": \"2019-09-01T00:00:00\"}, {\"year\": 2019, \"month\": 9, \"type\": \"comment\", \"count\": 230499, \"date\": \"2019-09-01T00:00:00\"}, {\"year\": 2019, \"month\": 10, \"type\": \"comment\", \"count\": 250523, \"date\": \"2019-10-01T00:00:00\"}, {\"year\": 2019, \"month\": 10, \"type\": \"story\", \"count\": 31877, \"date\": \"2019-10-01T00:00:00\"}, {\"year\": 2019, \"month\": 11, \"type\": \"comment\", \"count\": 224256, \"date\": \"2019-11-01T00:00:00\"}, {\"year\": 2019, \"month\": 11, \"type\": \"story\", \"count\": 30138, \"date\": \"2019-11-01T00:00:00\"}, {\"year\": 2019, \"month\": 12, \"type\": \"story\", \"count\": 29961, \"date\": \"2019-12-01T00:00:00\"}, {\"year\": 2019, \"month\": 12, \"type\": \"comment\", \"count\": 213877, \"date\": \"2019-12-01T00:00:00\"}, {\"year\": 2020, \"month\": 1, \"type\": \"story\", \"count\": 33411, \"date\": \"2020-01-01T00:00:00\"}, {\"year\": 2020, \"month\": 1, \"type\": \"comment\", \"count\": 236920, \"date\": \"2020-01-01T00:00:00\"}, {\"year\": 2020, \"month\": 2, \"type\": \"story\", \"count\": 31441, \"date\": \"2020-02-01T00:00:00\"}, {\"year\": 2020, \"month\": 2, \"type\": \"comment\", \"count\": 209454, \"date\": \"2020-02-01T00:00:00\"}, {\"year\": 2020, \"month\": 3, \"type\": \"story\", \"count\": 36109, \"date\": \"2020-03-01T00:00:00\"}, {\"year\": 2020, \"month\": 3, \"type\": \"comment\", \"count\": 242059, \"date\": \"2020-03-01T00:00:00\"}, {\"year\": 2020, \"month\": 4, \"type\": \"comment\", \"count\": 244755, \"date\": \"2020-04-01T00:00:00\"}, {\"year\": 2020, \"month\": 4, \"type\": \"story\", \"count\": 36999, \"date\": \"2020-04-01T00:00:00\"}, {\"year\": 2020, \"month\": 5, \"type\": \"comment\", \"count\": 289843, \"date\": \"2020-05-01T00:00:00\"}, {\"year\": 2020, \"month\": 5, \"type\": \"story\", \"count\": 39527, \"date\": \"2020-05-01T00:00:00\"}, {\"year\": 2020, \"month\": 6, \"type\": \"comment\", \"count\": 272649, \"date\": \"2020-06-01T00:00:00\"}, {\"year\": 2020, \"month\": 6, \"type\": \"story\", \"count\": 35850, \"date\": \"2020-06-01T00:00:00\"}, {\"year\": 2020, \"month\": 7, \"type\": \"comment\", \"count\": 272454, \"date\": \"2020-07-01T00:00:00\"}, {\"year\": 2020, \"month\": 7, \"type\": \"story\", \"count\": 35788, \"date\": \"2020-07-01T00:00:00\"}, {\"year\": 2020, \"month\": 8, \"type\": \"comment\", \"count\": 279371, \"date\": \"2020-08-01T00:00:00\"}, {\"year\": 2020, \"month\": 8, \"type\": \"story\", \"count\": 33913, \"date\": \"2020-08-01T00:00:00\"}, {\"year\": 2020, \"month\": 9, \"type\": \"comment\", \"count\": 264554, \"date\": \"2020-09-01T00:00:00\"}, {\"year\": 2020, \"month\": 9, \"type\": \"story\", \"count\": 32107, \"date\": \"2020-09-01T00:00:00\"}, {\"year\": 2020, \"month\": 10, \"type\": \"story\", \"count\": 33428, \"date\": \"2020-10-01T00:00:00\"}, {\"year\": 2020, \"month\": 10, \"type\": \"comment\", \"count\": 269866, \"date\": \"2020-10-01T00:00:00\"}, {\"year\": 2020, \"month\": 11, \"type\": \"story\", \"count\": 31396, \"date\": \"2020-11-01T00:00:00\"}, {\"year\": 2020, \"month\": 11, \"type\": \"comment\", \"count\": 261755, \"date\": \"2020-11-01T00:00:00\"}, {\"year\": 2020, \"month\": 12, \"type\": \"comment\", \"count\": 292605, \"date\": \"2020-12-01T00:00:00\"}, {\"year\": 2020, \"month\": 12, \"type\": \"story\", \"count\": 34637, \"date\": \"2020-12-01T00:00:00\"}, {\"year\": 2021, \"month\": 1, \"type\": \"story\", \"count\": 35700, \"date\": \"2021-01-01T00:00:00\"}, {\"year\": 2021, \"month\": 1, \"type\": \"comment\", \"count\": 340392, \"date\": \"2021-01-01T00:00:00\"}, {\"year\": 2021, \"month\": 2, \"type\": \"story\", \"count\": 32333, \"date\": \"2021-02-01T00:00:00\"}, {\"year\": 2021, \"month\": 2, \"type\": \"comment\", \"count\": 273346, \"date\": \"2021-02-01T00:00:00\"}, {\"year\": 2021, \"month\": 3, \"type\": \"comment\", \"count\": 307808, \"date\": \"2021-03-01T00:00:00\"}, {\"year\": 2021, \"month\": 3, \"type\": \"story\", \"count\": 34416, \"date\": \"2021-03-01T00:00:00\"}, {\"year\": 2021, \"month\": 4, \"type\": \"story\", \"count\": 32476, \"date\": \"2021-04-01T00:00:00\"}, {\"year\": 2021, \"month\": 4, \"type\": \"comment\", \"count\": 301246, \"date\": \"2021-04-01T00:00:00\"}, {\"year\": 2021, \"month\": 5, \"type\": \"comment\", \"count\": 306053, \"date\": \"2021-05-01T00:00:00\"}, {\"year\": 2021, \"month\": 5, \"type\": \"story\", \"count\": 32383, \"date\": \"2021-05-01T00:00:00\"}, {\"year\": 2021, \"month\": 6, \"type\": \"story\", \"count\": 30907, \"date\": \"2021-06-01T00:00:00\"}, {\"year\": 2021, \"month\": 6, \"type\": \"comment\", \"count\": 301577, \"date\": \"2021-06-01T00:00:00\"}, {\"year\": 2021, \"month\": 7, \"type\": \"comment\", \"count\": 289461, \"date\": \"2021-07-01T00:00:00\"}, {\"year\": 2021, \"month\": 7, \"type\": \"story\", \"count\": 29510, \"date\": \"2021-07-01T00:00:00\"}, {\"year\": 2021, \"month\": 8, \"type\": \"comment\", \"count\": 308363, \"date\": \"2021-08-01T00:00:00\"}, {\"year\": 2021, \"month\": 8, \"type\": \"story\", \"count\": 29558, \"date\": \"2021-08-01T00:00:00\"}, {\"year\": 2021, \"month\": 9, \"type\": \"story\", \"count\": 29231, \"date\": \"2021-09-01T00:00:00\"}, {\"year\": 2021, \"month\": 9, \"type\": \"comment\", \"count\": 296585, \"date\": \"2021-09-01T00:00:00\"}, {\"year\": 2021, \"month\": 10, \"type\": \"comment\", \"count\": 311107, \"date\": \"2021-10-01T00:00:00\"}, {\"year\": 2021, \"month\": 10, \"type\": \"story\", \"count\": 30536, \"date\": \"2021-10-01T00:00:00\"}, {\"year\": 2021, \"month\": 11, \"type\": \"story\", \"count\": 28662, \"date\": \"2021-11-01T00:00:00\"}, {\"year\": 2021, \"month\": 11, \"type\": \"comment\", \"count\": 294970, \"date\": \"2021-11-01T00:00:00\"}, {\"year\": 2021, \"month\": 12, \"type\": \"comment\", \"count\": 314557, \"date\": \"2021-12-01T00:00:00\"}, {\"year\": 2021, \"month\": 12, \"type\": \"story\", \"count\": 29396, \"date\": \"2021-12-01T00:00:00\"}, {\"year\": 2022, \"month\": 1, \"type\": \"comment\", \"count\": 355587, \"date\": \"2022-01-01T00:00:00\"}, {\"year\": 2022, \"month\": 1, \"type\": \"story\", \"count\": 33041, \"date\": \"2022-01-01T00:00:00\"}, {\"year\": 2022, \"month\": 2, \"type\": \"story\", \"count\": 29009, \"date\": \"2022-02-01T00:00:00\"}, {\"year\": 2022, \"month\": 2, \"type\": \"comment\", \"count\": 310443, \"date\": \"2022-02-01T00:00:00\"}, {\"year\": 2022, \"month\": 3, \"type\": \"story\", \"count\": 32460, \"date\": \"2022-03-01T00:00:00\"}, {\"year\": 2022, \"month\": 3, \"type\": \"comment\", \"count\": 323112, \"date\": \"2022-03-01T00:00:00\"}, {\"year\": 2022, \"month\": 4, \"type\": \"comment\", \"count\": 308746, \"date\": \"2022-04-01T00:00:00\"}, {\"year\": 2022, \"month\": 4, \"type\": \"story\", \"count\": 29013, \"date\": \"2022-04-01T00:00:00\"}, {\"year\": 2022, \"month\": 5, \"type\": \"story\", \"count\": 30341, \"date\": \"2022-05-01T00:00:00\"}, {\"year\": 2022, \"month\": 5, \"type\": \"comment\", \"count\": 313146, \"date\": \"2022-05-01T00:00:00\"}, {\"year\": 2022, \"month\": 6, \"type\": \"comment\", \"count\": 320479, \"date\": \"2022-06-01T00:00:00\"}, {\"year\": 2022, \"month\": 6, \"type\": \"story\", \"count\": 30302, \"date\": \"2022-06-01T00:00:00\"}, {\"year\": 2022, \"month\": 7, \"type\": \"comment\", \"count\": 322543, \"date\": \"2022-07-01T00:00:00\"}, {\"year\": 2022, \"month\": 7, \"type\": \"story\", \"count\": 30283, \"date\": \"2022-07-01T00:00:00\"}, {\"year\": 2022, \"month\": 8, \"type\": \"comment\", \"count\": 324896, \"date\": \"2022-08-01T00:00:00\"}, {\"year\": 2022, \"month\": 8, \"type\": \"story\", \"count\": 30694, \"date\": \"2022-08-01T00:00:00\"}, {\"year\": 2022, \"month\": 9, \"type\": \"story\", \"count\": 30233, \"date\": \"2022-09-01T00:00:00\"}, {\"year\": 2022, \"month\": 9, \"type\": \"comment\", \"count\": 328207, \"date\": \"2022-09-01T00:00:00\"}, {\"year\": 2022, \"month\": 10, \"type\": \"story\", \"count\": 30929, \"date\": \"2022-10-01T00:00:00\"}, {\"year\": 2022, \"month\": 10, \"type\": \"comment\", \"count\": 327727, \"date\": \"2022-10-01T00:00:00\"}, {\"year\": 2022, \"month\": 11, \"type\": \"comment\", \"count\": 348975, \"date\": \"2022-11-01T00:00:00\"}, {\"year\": 2022, \"month\": 11, \"type\": \"story\", \"count\": 32173, \"date\": \"2022-11-01T00:00:00\"}, {\"year\": 2022, \"month\": 12, \"type\": \"comment\", \"count\": 350905, \"date\": \"2022-12-01T00:00:00\"}, {\"year\": 2022, \"month\": 12, \"type\": \"story\", \"count\": 32331, \"date\": \"2022-12-01T00:00:00\"}, {\"year\": 2023, \"month\": 1, \"type\": \"story\", \"count\": 33332, \"date\": \"2023-01-01T00:00:00\"}, {\"year\": 2023, \"month\": 1, \"type\": \"comment\", \"count\": 350850, \"date\": \"2023-01-01T00:00:00\"}, {\"year\": 2023, \"month\": 2, \"type\": \"comment\", \"count\": 330908, \"date\": \"2023-02-01T00:00:00\"}, {\"year\": 2023, \"month\": 2, \"type\": \"story\", \"count\": 30105, \"date\": \"2023-02-01T00:00:00\"}, {\"year\": 2023, \"month\": 3, \"type\": \"story\", \"count\": 35391, \"date\": \"2023-03-01T00:00:00\"}, {\"year\": 2023, \"month\": 3, \"type\": \"comment\", \"count\": 368577, \"date\": \"2023-03-01T00:00:00\"}, {\"year\": 2023, \"month\": 4, \"type\": \"story\", \"count\": 32605, \"date\": \"2023-04-01T00:00:00\"}, {\"year\": 2023, \"month\": 4, \"type\": \"comment\", \"count\": 332043, \"date\": \"2023-04-01T00:00:00\"}, {\"year\": 2023, \"month\": 5, \"type\": \"comment\", \"count\": 329746, \"date\": \"2023-05-01T00:00:00\"}, {\"year\": 2023, \"month\": 5, \"type\": \"story\", \"count\": 33019, \"date\": \"2023-05-01T00:00:00\"}, {\"year\": 2023, \"month\": 6, \"type\": \"comment\", \"count\": 350127, \"date\": \"2023-06-01T00:00:00\"}, {\"year\": 2023, \"month\": 6, \"type\": \"story\", \"count\": 32426, \"date\": \"2023-06-01T00:00:00\"}, {\"year\": 2023, \"month\": 7, \"type\": \"story\", \"count\": 32522, \"date\": \"2023-07-01T00:00:00\"}, {\"year\": 2023, \"month\": 7, \"type\": \"comment\", \"count\": 362260, \"date\": \"2023-07-01T00:00:00\"}, {\"year\": 2023, \"month\": 8, \"type\": \"comment\", \"count\": 346467, \"date\": \"2023-08-01T00:00:00\"}, {\"year\": 2023, \"month\": 8, \"type\": \"story\", \"count\": 32994, \"date\": \"2023-08-01T00:00:00\"}, {\"year\": 2023, \"month\": 9, \"type\": \"story\", \"count\": 32526, \"date\": \"2023-09-01T00:00:00\"}, {\"year\": 2023, \"month\": 9, \"type\": \"comment\", \"count\": 334078, \"date\": \"2023-09-01T00:00:00\"}, {\"year\": 2023, \"month\": 10, \"type\": \"comment\", \"count\": 322628, \"date\": \"2023-10-01T00:00:00\"}, {\"year\": 2023, \"month\": 10, \"type\": \"story\", \"count\": 32057, \"date\": \"2023-10-01T00:00:00\"}, {\"year\": 2023, \"month\": 11, \"type\": \"comment\", \"count\": 342128, \"date\": \"2023-11-01T00:00:00\"}, {\"year\": 2023, \"month\": 11, \"type\": \"story\", \"count\": 31735, \"date\": \"2023-11-01T00:00:00\"}, {\"year\": 2023, \"month\": 12, \"type\": \"comment\", \"count\": 310251, \"date\": \"2023-12-01T00:00:00\"}, {\"year\": 2023, \"month\": 12, \"type\": \"story\", \"count\": 30469, \"date\": \"2023-12-01T00:00:00\"}, {\"year\": 2024, \"month\": 1, \"type\": \"comment\", \"count\": 332537, \"date\": \"2024-01-01T00:00:00\"}, {\"year\": 2024, \"month\": 1, \"type\": \"story\", \"count\": 34443, \"date\": \"2024-01-01T00:00:00\"}, {\"year\": 2024, \"month\": 2, \"type\": \"story\", \"count\": 32294, \"date\": \"2024-02-01T00:00:00\"}, {\"year\": 2024, \"month\": 2, \"type\": \"comment\", \"count\": 302173, \"date\": \"2024-02-01T00:00:00\"}, {\"year\": 2024, \"month\": 3, \"type\": \"comment\", \"count\": 290989, \"date\": \"2024-03-01T00:00:00\"}, {\"year\": 2024, \"month\": 3, \"type\": \"story\", \"count\": 33159, \"date\": \"2024-03-01T00:00:00\"}, {\"year\": 2024, \"month\": 4, \"type\": \"comment\", \"count\": 283043, \"date\": \"2024-04-01T00:00:00\"}, {\"year\": 2024, \"month\": 4, \"type\": \"story\", \"count\": 31946, \"date\": \"2024-04-01T00:00:00\"}, {\"year\": 2024, \"month\": 5, \"type\": \"comment\", \"count\": 279439, \"date\": \"2024-05-01T00:00:00\"}, {\"year\": 2024, \"month\": 5, \"type\": \"story\", \"count\": 33160, \"date\": \"2024-05-01T00:00:00\"}, {\"year\": 2024, \"month\": 6, \"type\": \"story\", \"count\": 30487, \"date\": \"2024-06-01T00:00:00\"}, {\"year\": 2024, \"month\": 6, \"type\": \"comment\", \"count\": 263748, \"date\": \"2024-06-01T00:00:00\"}, {\"year\": 2024, \"month\": 7, \"type\": \"story\", \"count\": 29917, \"date\": \"2024-07-01T00:00:00\"}, {\"year\": 2024, \"month\": 7, \"type\": \"comment\", \"count\": 242398, \"date\": \"2024-07-01T00:00:00\"}, {\"year\": 2024, \"month\": 8, \"type\": \"comment\", \"count\": 249562, \"date\": \"2024-08-01T00:00:00\"}, {\"year\": 2024, \"month\": 8, \"type\": \"story\", \"count\": 31954, \"date\": \"2024-08-01T00:00:00\"}, {\"year\": 2024, \"month\": 9, \"type\": \"story\", \"count\": 30911, \"date\": \"2024-09-01T00:00:00\"}, {\"year\": 2024, \"month\": 9, \"type\": \"comment\", \"count\": 247452, \"date\": \"2024-09-01T00:00:00\"}, {\"year\": 2024, \"month\": 10, \"type\": \"comment\", \"count\": 263691, \"date\": \"2024-10-01T00:00:00\"}, {\"year\": 2024, \"month\": 10, \"type\": \"story\", \"count\": 34291, \"date\": \"2024-10-01T00:00:00\"}, {\"year\": 2024, \"month\": 11, \"type\": \"comment\", \"count\": 237367, \"date\": \"2024-11-01T00:00:00\"}, {\"year\": 2024, \"month\": 11, \"type\": \"story\", \"count\": 28986, \"date\": \"2024-11-01T00:00:00\"}, {\"year\": 2024, \"month\": 12, \"type\": \"story\", \"count\": 28482, \"date\": \"2024-12-01T00:00:00\"}, {\"year\": 2024, \"month\": 12, \"type\": \"comment\", \"count\": 239977, \"date\": \"2024-12-01T00:00:00\"}, {\"year\": 2025, \"month\": 1, \"type\": \"comment\", \"count\": 288371, \"date\": \"2025-01-01T00:00:00\"}, {\"year\": 2025, \"month\": 1, \"type\": \"story\", \"count\": 31234, \"date\": \"2025-01-01T00:00:00\"}, {\"year\": 2025, \"month\": 2, \"type\": \"comment\", \"count\": 151768, \"date\": \"2025-02-01T00:00:00\"}, {\"year\": 2025, \"month\": 2, \"type\": \"story\", \"count\": 15642, \"date\": \"2025-02-01T00:00:00\"}]}}, {\"mode\": \"vega-lite\"});\n",
       "</script>"
      ],
      "text/plain": [
       "alt.Chart(...)"
      ]
     },
     "execution_count": 4,
     "metadata": {},
     "output_type": "execute_result"
    }
   ],
   "source": [
    "import pandas as pd\n",
    "\n",
    "df = conn.sql(\"\"\"\n",
    "    SELECT \n",
    "        year,\n",
    "        month,\n",
    "        type,\n",
    "        COUNT(*) as count\n",
    "    FROM hn \n",
    "    WHERE type IN ('comment', 'story')\n",
    "    GROUP BY year, month, type\n",
    "    ORDER BY year, month\n",
    "\"\"\").df()\n",
    "\n",
    "df['date'] = pd.to_datetime(df['year'].astype(str) + '-' + df['month'].astype(str) + '-01')\n",
    "chart = alt.Chart(df).mark_line().encode(\n",
    "    x=alt.X('date:T', title='Date'),\n",
    "    y=alt.Y('count:Q', title='Count'),\n",
    "    color=alt.Color('type:N', title='Content Type'),\n",
    "    tooltip=[\n",
    "        alt.Tooltip('date:T', title='Date', format='%B %Y'),\n",
    "        alt.Tooltip('type:N', title='Type'),\n",
    "        alt.Tooltip('count:Q', title='Count', format=',')\n",
    "    ]\n",
    ").properties(\n",
    "    title='Monthly Comments and Stories on Hacker News',\n",
    "    width=800,\n",
    "    height=400\n",
    ")\n",
    "chart\n"
   ]
  },
  {
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "Interesting that the number of comments increase over time, while the number of stories seems to stop growing since 2012.\n",
    "\n",
    "There is also drop of comments in starting on end of 2023."
   ]
  },
  {
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "### Activity heatmap"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 5,
   "metadata": {},
   "outputs": [
    {
     "data": {
      "text/html": [
       "\n",
       "<style>\n",
       "  #altair-viz-f9b18583ea954dbd93b100569d5737c6.vega-embed {\n",
       "    width: 100%;\n",
       "    display: flex;\n",
       "  }\n",
       "\n",
       "  #altair-viz-f9b18583ea954dbd93b100569d5737c6.vega-embed details,\n",
       "  #altair-viz-f9b18583ea954dbd93b100569d5737c6.vega-embed details summary {\n",
       "    position: relative;\n",
       "  }\n",
       "</style>\n",
       "<div id=\"altair-viz-f9b18583ea954dbd93b100569d5737c6\"></div>\n",
       "<script type=\"text/javascript\">\n",
       "  var VEGA_DEBUG = (typeof VEGA_DEBUG == \"undefined\") ? {} : VEGA_DEBUG;\n",
       "  (function(spec, embedOpt){\n",
       "    let outputDiv = document.currentScript.previousElementSibling;\n",
       "    if (outputDiv.id !== \"altair-viz-f9b18583ea954dbd93b100569d5737c6\") {\n",
       "      outputDiv = document.getElementById(\"altair-viz-f9b18583ea954dbd93b100569d5737c6\");\n",
       "    }\n",
       "\n",
       "    const paths = {\n",
       "      \"vega\": \"https://cdn.jsdelivr.net/npm/vega@5?noext\",\n",
       "      \"vega-lib\": \"https://cdn.jsdelivr.net/npm/vega-lib?noext\",\n",
       "      \"vega-lite\": \"https://cdn.jsdelivr.net/npm/vega-lite@5.20.1?noext\",\n",
       "      \"vega-embed\": \"https://cdn.jsdelivr.net/npm/vega-embed@6?noext\",\n",
       "    };\n",
       "\n",
       "    function maybeLoadScript(lib, version) {\n",
       "      var key = `${lib.replace(\"-\", \"\")}_version`;\n",
       "      return (VEGA_DEBUG[key] == version) ?\n",
       "        Promise.resolve(paths[lib]) :\n",
       "        new Promise(function(resolve, reject) {\n",
       "          var s = document.createElement('script');\n",
       "          document.getElementsByTagName(\"head\")[0].appendChild(s);\n",
       "          s.async = true;\n",
       "          s.onload = () => {\n",
       "            VEGA_DEBUG[key] = version;\n",
       "            return resolve(paths[lib]);\n",
       "          };\n",
       "          s.onerror = () => reject(`Error loading script: ${paths[lib]}`);\n",
       "          s.src = paths[lib];\n",
       "        });\n",
       "    }\n",
       "\n",
       "    function showError(err) {\n",
       "      outputDiv.innerHTML = `<div class=\"error\" style=\"color:red;\">${err}</div>`;\n",
       "      throw err;\n",
       "    }\n",
       "\n",
       "    function displayChart(vegaEmbed) {\n",
       "      vegaEmbed(outputDiv, spec, embedOpt)\n",
       "        .catch(err => showError(`Javascript Error: ${err.message}<br>This usually means there's a typo in your chart specification. See the javascript console for the full traceback.`));\n",
       "    }\n",
       "\n",
       "    if(typeof define === \"function\" && define.amd) {\n",
       "      requirejs.config({paths});\n",
       "      let deps = [\"vega-embed\"];\n",
       "      require(deps, displayChart, err => showError(`Error loading script: ${err.message}`));\n",
       "    } else {\n",
       "      maybeLoadScript(\"vega\", \"5\")\n",
       "        .then(() => maybeLoadScript(\"vega-lite\", \"5.20.1\"))\n",
       "        .then(() => maybeLoadScript(\"vega-embed\", \"6\"))\n",
       "        .catch(showError)\n",
       "        .then(() => displayChart(vegaEmbed));\n",
       "    }\n",
       "  })({\"config\": {\"view\": {\"continuousWidth\": 300, \"continuousHeight\": 300}}, \"data\": {\"name\": \"data-70b9d093db46eb1738457755302b9c21\"}, \"mark\": {\"type\": \"rect\"}, \"encoding\": {\"color\": {\"field\": \"count\", \"scale\": {\"scheme\": \"greens\"}, \"title\": \"Number of Posts\", \"type\": \"quantitative\"}, \"tooltip\": [{\"field\": \"hour\", \"title\": \"Hour\", \"type\": \"ordinal\"}, {\"field\": \"day_name\", \"title\": \"Day\", \"type\": \"ordinal\"}, {\"field\": \"count\", \"format\": \",\", \"title\": \"Count\", \"type\": \"quantitative\"}], \"x\": {\"field\": \"hour\", \"title\": \"Hour of Day (CST)\", \"type\": \"ordinal\"}, \"y\": {\"field\": \"day_name\", \"sort\": [\"Sun\", \"Mon\", \"Tue\", \"Wed\", \"Thu\", \"Fri\", \"Sat\"], \"title\": \"Day of Week\", \"type\": \"ordinal\"}}, \"height\": 300, \"title\": \"Activity Heatmap by Hour and Day of Week\", \"width\": 800, \"$schema\": \"https://vega.github.io/schema/vega-lite/v5.20.1.json\", \"datasets\": {\"data-70b9d093db46eb1738457755302b9c21\": [{\"hour\": 0, \"day_of_week\": 0, \"count\": 192740, \"day_name\": \"Sun\"}, {\"hour\": 0, \"day_of_week\": 1, \"count\": 208048, \"day_name\": \"Mon\"}, {\"hour\": 0, \"day_of_week\": 2, \"count\": 270556, \"day_name\": \"Tue\"}, {\"hour\": 0, \"day_of_week\": 3, \"count\": 275833, \"day_name\": \"Wed\"}, {\"hour\": 0, \"day_of_week\": 4, \"count\": 275660, \"day_name\": \"Thu\"}, {\"hour\": 0, \"day_of_week\": 5, \"count\": 268278, \"day_name\": \"Fri\"}, {\"hour\": 0, \"day_of_week\": 6, \"count\": 239691, \"day_name\": \"Sat\"}, {\"hour\": 1, \"day_of_week\": 0, \"count\": 174306, \"day_name\": \"Sun\"}, {\"hour\": 1, \"day_of_week\": 1, \"count\": 192534, \"day_name\": \"Mon\"}, {\"hour\": 1, \"day_of_week\": 2, \"count\": 235243, \"day_name\": \"Tue\"}, {\"hour\": 1, \"day_of_week\": 3, \"count\": 240056, \"day_name\": \"Wed\"}, {\"hour\": 1, \"day_of_week\": 4, \"count\": 238987, \"day_name\": \"Thu\"}, {\"hour\": 1, \"day_of_week\": 5, \"count\": 232188, \"day_name\": \"Fri\"}, {\"hour\": 1, \"day_of_week\": 6, \"count\": 204117, \"day_name\": \"Sat\"}, {\"hour\": 2, \"day_of_week\": 0, \"count\": 162531, \"day_name\": \"Sun\"}, {\"hour\": 2, \"day_of_week\": 1, \"count\": 183895, \"day_name\": \"Mon\"}, {\"hour\": 2, \"day_of_week\": 2, \"count\": 213300, \"day_name\": \"Tue\"}, {\"hour\": 2, \"day_of_week\": 3, \"count\": 214732, \"day_name\": \"Wed\"}, {\"hour\": 2, \"day_of_week\": 4, \"count\": 213790, \"day_name\": \"Thu\"}, {\"hour\": 2, \"day_of_week\": 5, \"count\": 206891, \"day_name\": \"Fri\"}, {\"hour\": 2, \"day_of_week\": 6, \"count\": 177646, \"day_name\": \"Sat\"}, {\"hour\": 3, \"day_of_week\": 0, \"count\": 152050, \"day_name\": \"Sun\"}, {\"hour\": 3, \"day_of_week\": 1, \"count\": 176474, \"day_name\": \"Mon\"}, {\"hour\": 3, \"day_of_week\": 2, \"count\": 197372, \"day_name\": \"Tue\"}, {\"hour\": 3, \"day_of_week\": 3, \"count\": 198419, \"day_name\": \"Wed\"}, {\"hour\": 3, \"day_of_week\": 4, \"count\": 197216, \"day_name\": \"Thu\"}, {\"hour\": 3, \"day_of_week\": 5, \"count\": 192274, \"day_name\": \"Fri\"}, {\"hour\": 3, \"day_of_week\": 6, \"count\": 161113, \"day_name\": \"Sat\"}, {\"hour\": 4, \"day_of_week\": 0, \"count\": 142933, \"day_name\": \"Sun\"}, {\"hour\": 4, \"day_of_week\": 1, \"count\": 170459, \"day_name\": \"Mon\"}, {\"hour\": 4, \"day_of_week\": 2, \"count\": 186721, \"day_name\": \"Tue\"}, {\"hour\": 4, \"day_of_week\": 3, \"count\": 187099, \"day_name\": \"Wed\"}, {\"hour\": 4, \"day_of_week\": 4, \"count\": 185726, \"day_name\": \"Thu\"}, {\"hour\": 4, \"day_of_week\": 5, \"count\": 180119, \"day_name\": \"Fri\"}, {\"hour\": 4, \"day_of_week\": 6, \"count\": 149773, \"day_name\": \"Sat\"}, {\"hour\": 5, \"day_of_week\": 0, \"count\": 133167, \"day_name\": \"Sun\"}, {\"hour\": 5, \"day_of_week\": 1, \"count\": 162915, \"day_name\": \"Mon\"}, {\"hour\": 5, \"day_of_week\": 2, \"count\": 175561, \"day_name\": \"Tue\"}, {\"hour\": 5, \"day_of_week\": 3, \"count\": 176525, \"day_name\": \"Wed\"}, {\"hour\": 5, \"day_of_week\": 4, \"count\": 174956, \"day_name\": \"Thu\"}, {\"hour\": 5, \"day_of_week\": 5, \"count\": 170867, \"day_name\": \"Fri\"}, {\"hour\": 5, \"day_of_week\": 6, \"count\": 138019, \"day_name\": \"Sat\"}, {\"hour\": 6, \"day_of_week\": 0, \"count\": 125617, \"day_name\": \"Sun\"}, {\"hour\": 6, \"day_of_week\": 1, \"count\": 157376, \"day_name\": \"Mon\"}, {\"hour\": 6, \"day_of_week\": 2, \"count\": 168157, \"day_name\": \"Tue\"}, {\"hour\": 6, \"day_of_week\": 3, \"count\": 170505, \"day_name\": \"Wed\"}, {\"hour\": 6, \"day_of_week\": 4, \"count\": 169174, \"day_name\": \"Thu\"}, {\"hour\": 6, \"day_of_week\": 5, \"count\": 162988, \"day_name\": \"Fri\"}, {\"hour\": 6, \"day_of_week\": 6, \"count\": 127716, \"day_name\": \"Sat\"}, {\"hour\": 7, \"day_of_week\": 0, \"count\": 122283, \"day_name\": \"Sun\"}, {\"hour\": 7, \"day_of_week\": 1, \"count\": 160628, \"day_name\": \"Mon\"}, {\"hour\": 7, \"day_of_week\": 2, \"count\": 170069, \"day_name\": \"Tue\"}, {\"hour\": 7, \"day_of_week\": 3, \"count\": 171720, \"day_name\": \"Wed\"}, {\"hour\": 7, \"day_of_week\": 4, \"count\": 170817, \"day_name\": \"Thu\"}, {\"hour\": 7, \"day_of_week\": 5, \"count\": 164410, \"day_name\": \"Fri\"}, {\"hour\": 7, \"day_of_week\": 6, \"count\": 123515, \"day_name\": \"Sat\"}, {\"hour\": 8, \"day_of_week\": 0, \"count\": 121894, \"day_name\": \"Sun\"}, {\"hour\": 8, \"day_of_week\": 1, \"count\": 165708, \"day_name\": \"Mon\"}, {\"hour\": 8, \"day_of_week\": 2, \"count\": 173583, \"day_name\": \"Tue\"}, {\"hour\": 8, \"day_of_week\": 3, \"count\": 177881, \"day_name\": \"Wed\"}, {\"hour\": 8, \"day_of_week\": 4, \"count\": 175129, \"day_name\": \"Thu\"}, {\"hour\": 8, \"day_of_week\": 5, \"count\": 169896, \"day_name\": \"Fri\"}, {\"hour\": 8, \"day_of_week\": 6, \"count\": 123820, \"day_name\": \"Sat\"}, {\"hour\": 9, \"day_of_week\": 0, \"count\": 123222, \"day_name\": \"Sun\"}, {\"hour\": 9, \"day_of_week\": 1, \"count\": 171434, \"day_name\": \"Mon\"}, {\"hour\": 9, \"day_of_week\": 2, \"count\": 178164, \"day_name\": \"Tue\"}, {\"hour\": 9, \"day_of_week\": 3, \"count\": 182347, \"day_name\": \"Wed\"}, {\"hour\": 9, \"day_of_week\": 4, \"count\": 176841, \"day_name\": \"Thu\"}, {\"hour\": 9, \"day_of_week\": 5, \"count\": 174635, \"day_name\": \"Fri\"}, {\"hour\": 9, \"day_of_week\": 6, \"count\": 123029, \"day_name\": \"Sat\"}, {\"hour\": 10, \"day_of_week\": 0, \"count\": 124776, \"day_name\": \"Sun\"}, {\"hour\": 10, \"day_of_week\": 1, \"count\": 176316, \"day_name\": \"Mon\"}, {\"hour\": 10, \"day_of_week\": 2, \"count\": 180835, \"day_name\": \"Tue\"}, {\"hour\": 10, \"day_of_week\": 3, \"count\": 182804, \"day_name\": \"Wed\"}, {\"hour\": 10, \"day_of_week\": 4, \"count\": 179103, \"day_name\": \"Thu\"}, {\"hour\": 10, \"day_of_week\": 5, \"count\": 175388, \"day_name\": \"Fri\"}, {\"hour\": 10, \"day_of_week\": 6, \"count\": 121256, \"day_name\": \"Sat\"}, {\"hour\": 11, \"day_of_week\": 0, \"count\": 126881, \"day_name\": \"Sun\"}, {\"hour\": 11, \"day_of_week\": 1, \"count\": 180596, \"day_name\": \"Mon\"}, {\"hour\": 11, \"day_of_week\": 2, \"count\": 185654, \"day_name\": \"Tue\"}, {\"hour\": 11, \"day_of_week\": 3, \"count\": 185071, \"day_name\": \"Wed\"}, {\"hour\": 11, \"day_of_week\": 4, \"count\": 181905, \"day_name\": \"Thu\"}, {\"hour\": 11, \"day_of_week\": 5, \"count\": 177123, \"day_name\": \"Fri\"}, {\"hour\": 11, \"day_of_week\": 6, \"count\": 122798, \"day_name\": \"Sat\"}, {\"hour\": 12, \"day_of_week\": 0, \"count\": 135583, \"day_name\": \"Sun\"}, {\"hour\": 12, \"day_of_week\": 1, \"count\": 201257, \"day_name\": \"Mon\"}, {\"hour\": 12, \"day_of_week\": 2, \"count\": 208595, \"day_name\": \"Tue\"}, {\"hour\": 12, \"day_of_week\": 3, \"count\": 205269, \"day_name\": \"Wed\"}, {\"hour\": 12, \"day_of_week\": 4, \"count\": 202881, \"day_name\": \"Thu\"}, {\"hour\": 12, \"day_of_week\": 5, \"count\": 195951, \"day_name\": \"Fri\"}, {\"hour\": 12, \"day_of_week\": 6, \"count\": 133165, \"day_name\": \"Sat\"}, {\"hour\": 13, \"day_of_week\": 0, \"count\": 153006, \"day_name\": \"Sun\"}, {\"hour\": 13, \"day_of_week\": 1, \"count\": 241472, \"day_name\": \"Mon\"}, {\"hour\": 13, \"day_of_week\": 2, \"count\": 252842, \"day_name\": \"Tue\"}, {\"hour\": 13, \"day_of_week\": 3, \"count\": 256311, \"day_name\": \"Wed\"}, {\"hour\": 13, \"day_of_week\": 4, \"count\": 252120, \"day_name\": \"Thu\"}, {\"hour\": 13, \"day_of_week\": 5, \"count\": 240672, \"day_name\": \"Fri\"}, {\"hour\": 13, \"day_of_week\": 6, \"count\": 151457, \"day_name\": \"Sat\"}, {\"hour\": 14, \"day_of_week\": 0, \"count\": 177918, \"day_name\": \"Sun\"}, {\"hour\": 14, \"day_of_week\": 1, \"count\": 301561, \"day_name\": \"Mon\"}, {\"hour\": 14, \"day_of_week\": 2, \"count\": 322748, \"day_name\": \"Tue\"}, {\"hour\": 14, \"day_of_week\": 3, \"count\": 324903, \"day_name\": \"Wed\"}, {\"hour\": 14, \"day_of_week\": 4, \"count\": 318808, \"day_name\": \"Thu\"}, {\"hour\": 14, \"day_of_week\": 5, \"count\": 304246, \"day_name\": \"Fri\"}, {\"hour\": 14, \"day_of_week\": 6, \"count\": 177014, \"day_name\": \"Sat\"}, {\"hour\": 15, \"day_of_week\": 0, \"count\": 210984, \"day_name\": \"Sun\"}, {\"hour\": 15, \"day_of_week\": 1, \"count\": 359363, \"day_name\": \"Mon\"}, {\"hour\": 15, \"day_of_week\": 2, \"count\": 386624, \"day_name\": \"Tue\"}, {\"hour\": 15, \"day_of_week\": 3, \"count\": 391503, \"day_name\": \"Wed\"}, {\"hour\": 15, \"day_of_week\": 4, \"count\": 381516, \"day_name\": \"Thu\"}, {\"hour\": 15, \"day_of_week\": 5, \"count\": 362275, \"day_name\": \"Fri\"}, {\"hour\": 15, \"day_of_week\": 6, \"count\": 208021, \"day_name\": \"Sat\"}, {\"hour\": 16, \"day_of_week\": 0, \"count\": 237670, \"day_name\": \"Sun\"}, {\"hour\": 16, \"day_of_week\": 1, \"count\": 401942, \"day_name\": \"Mon\"}, {\"hour\": 16, \"day_of_week\": 2, \"count\": 431613, \"day_name\": \"Tue\"}, {\"hour\": 16, \"day_of_week\": 3, \"count\": 433376, \"day_name\": \"Wed\"}, {\"hour\": 16, \"day_of_week\": 4, \"count\": 423274, \"day_name\": \"Thu\"}, {\"hour\": 16, \"day_of_week\": 5, \"count\": 398864, \"day_name\": \"Fri\"}, {\"hour\": 16, \"day_of_week\": 6, \"count\": 234685, \"day_name\": \"Sat\"}, {\"hour\": 17, \"day_of_week\": 0, \"count\": 257225, \"day_name\": \"Sun\"}, {\"hour\": 17, \"day_of_week\": 1, \"count\": 421268, \"day_name\": \"Mon\"}, {\"hour\": 17, \"day_of_week\": 2, \"count\": 450091, \"day_name\": \"Tue\"}, {\"hour\": 17, \"day_of_week\": 3, \"count\": 448503, \"day_name\": \"Wed\"}, {\"hour\": 17, \"day_of_week\": 4, \"count\": 439310, \"day_name\": \"Thu\"}, {\"hour\": 17, \"day_of_week\": 5, \"count\": 414239, \"day_name\": \"Fri\"}, {\"hour\": 17, \"day_of_week\": 6, \"count\": 252038, \"day_name\": \"Sat\"}, {\"hour\": 18, \"day_of_week\": 0, \"count\": 266911, \"day_name\": \"Sun\"}, {\"hour\": 18, \"day_of_week\": 1, \"count\": 421737, \"day_name\": \"Mon\"}, {\"hour\": 18, \"day_of_week\": 2, \"count\": 450418, \"day_name\": \"Tue\"}, {\"hour\": 18, \"day_of_week\": 3, \"count\": 443674, \"day_name\": \"Wed\"}, {\"hour\": 18, \"day_of_week\": 4, \"count\": 435435, \"day_name\": \"Thu\"}, {\"hour\": 18, \"day_of_week\": 5, \"count\": 409856, \"day_name\": \"Fri\"}, {\"hour\": 18, \"day_of_week\": 6, \"count\": 257915, \"day_name\": \"Sat\"}, {\"hour\": 19, \"day_of_week\": 0, \"count\": 267788, \"day_name\": \"Sun\"}, {\"hour\": 19, \"day_of_week\": 1, \"count\": 411286, \"day_name\": \"Mon\"}, {\"hour\": 19, \"day_of_week\": 2, \"count\": 436943, \"day_name\": \"Tue\"}, {\"hour\": 19, \"day_of_week\": 3, \"count\": 427905, \"day_name\": \"Wed\"}, {\"hour\": 19, \"day_of_week\": 4, \"count\": 420722, \"day_name\": \"Thu\"}, {\"hour\": 19, \"day_of_week\": 5, \"count\": 394538, \"day_name\": \"Fri\"}, {\"hour\": 19, \"day_of_week\": 6, \"count\": 255107, \"day_name\": \"Sat\"}, {\"hour\": 20, \"day_of_week\": 0, \"count\": 264218, \"day_name\": \"Sun\"}, {\"hour\": 20, \"day_of_week\": 1, \"count\": 395243, \"day_name\": \"Mon\"}, {\"hour\": 20, \"day_of_week\": 2, \"count\": 413961, \"day_name\": \"Tue\"}, {\"hour\": 20, \"day_of_week\": 3, \"count\": 405495, \"day_name\": \"Wed\"}, {\"hour\": 20, \"day_of_week\": 4, \"count\": 399142, \"day_name\": \"Thu\"}, {\"hour\": 20, \"day_of_week\": 5, \"count\": 369576, \"day_name\": \"Fri\"}, {\"hour\": 20, \"day_of_week\": 6, \"count\": 246285, \"day_name\": \"Sat\"}, {\"hour\": 21, \"day_of_week\": 0, \"count\": 257024, \"day_name\": \"Sun\"}, {\"hour\": 21, \"day_of_week\": 1, \"count\": 380448, \"day_name\": \"Mon\"}, {\"hour\": 21, \"day_of_week\": 2, \"count\": 392725, \"day_name\": \"Tue\"}, {\"hour\": 21, \"day_of_week\": 3, \"count\": 391408, \"day_name\": \"Wed\"}, {\"hour\": 21, \"day_of_week\": 4, \"count\": 382380, \"day_name\": \"Thu\"}, {\"hour\": 21, \"day_of_week\": 5, \"count\": 350460, \"day_name\": \"Fri\"}, {\"hour\": 21, \"day_of_week\": 6, \"count\": 237214, \"day_name\": \"Sat\"}, {\"hour\": 22, \"day_of_week\": 0, \"count\": 246103, \"day_name\": \"Sun\"}, {\"hour\": 22, \"day_of_week\": 1, \"count\": 355311, \"day_name\": \"Mon\"}, {\"hour\": 22, \"day_of_week\": 2, \"count\": 366469, \"day_name\": \"Tue\"}, {\"hour\": 22, \"day_of_week\": 3, \"count\": 363033, \"day_name\": \"Wed\"}, {\"hour\": 22, \"day_of_week\": 4, \"count\": 356155, \"day_name\": \"Thu\"}, {\"hour\": 22, \"day_of_week\": 5, \"count\": 320877, \"day_name\": \"Fri\"}, {\"hour\": 22, \"day_of_week\": 6, \"count\": 226207, \"day_name\": \"Sat\"}, {\"hour\": 23, \"day_of_week\": 0, \"count\": 229624, \"day_name\": \"Sun\"}, {\"hour\": 23, \"day_of_week\": 1, \"count\": 312353, \"day_name\": \"Mon\"}, {\"hour\": 23, \"day_of_week\": 2, \"count\": 320095, \"day_name\": \"Tue\"}, {\"hour\": 23, \"day_of_week\": 3, \"count\": 317627, \"day_name\": \"Wed\"}, {\"hour\": 23, \"day_of_week\": 4, \"count\": 311407, \"day_name\": \"Thu\"}, {\"hour\": 23, \"day_of_week\": 5, \"count\": 279258, \"day_name\": \"Fri\"}, {\"hour\": 23, \"day_of_week\": 6, \"count\": 209838, \"day_name\": \"Sat\"}]}}, {\"mode\": \"vega-lite\"});\n",
       "</script>"
      ],
      "text/plain": [
       "alt.Chart(...)"
      ]
     },
     "execution_count": 5,
     "metadata": {},
     "output_type": "execute_result"
    }
   ],
   "source": [
    "df = conn.sql(\"\"\"\n",
    "    SELECT \n",
    "        EXTRACT(HOUR FROM (time - INTERVAL '6' HOUR)) as hour,\n",
    "        EXTRACT(DOW FROM (time - INTERVAL '6' HOUR)) as day_of_week,\n",
    "        COUNT(*) as count\n",
    "    FROM hn\n",
    "    GROUP BY hour, day_of_week\n",
    "    ORDER BY hour, day_of_week\n",
    "\"\"\").df()\n",
    "\n",
    "days = ['Sun', 'Mon', 'Tue', 'Wed', 'Thu', 'Fri', 'Sat']\n",
    "df['day_name'] = df['day_of_week'].map(lambda x: days[int(x)])\n",
    "\n",
    "chart = alt.Chart(df).mark_rect().encode(\n",
    "    x=alt.X('hour:O', title='Hour of Day (CST)'),\n",
    "    y=alt.Y('day_name:O', title='Day of Week', sort=days),\n",
    "    color=alt.Color('count:Q', title='Number of Posts',\n",
    "                   scale=alt.Scale(scheme='greens')),\n",
    "    tooltip=[\n",
    "        alt.Tooltip('hour:O', title='Hour'),\n",
    "        alt.Tooltip('day_name:O', title='Day'),\n",
    "        alt.Tooltip('count:Q', title='Count', format=',')\n",
    "    ]\n",
    ").properties(\n",
    "    title='Activity Heatmap by Hour and Day of Week',\n",
    "    width=800,\n",
    "    height=300\n",
    ")\n",
    "chart\n"
   ]
  },
  {
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "Hour is shifted to USA Central Time (-6 hours) since [a lot of Hacker News users are in USA](https://news.ycombinator.com/item?id=30210378).\n",
    "\n",
    "Looks like the most active time is afternoon on work days. Are people using Hacker News to procrastinate after lunch break?"
   ]
  },
  {
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "### Hiring trends\n",
    "\n",
    "Every month, there are two threads on Hacker News that are dedicated to hiring. One is about who is hiring, and the other is about who wants to be hired."
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 6,
   "metadata": {},
   "outputs": [
    {
     "data": {
      "text/html": [
       "\n",
       "<style>\n",
       "  #altair-viz-9af7f784c2534176bc8742dff8d0169a.vega-embed {\n",
       "    width: 100%;\n",
       "    display: flex;\n",
       "  }\n",
       "\n",
       "  #altair-viz-9af7f784c2534176bc8742dff8d0169a.vega-embed details,\n",
       "  #altair-viz-9af7f784c2534176bc8742dff8d0169a.vega-embed details summary {\n",
       "    position: relative;\n",
       "  }\n",
       "</style>\n",
       "<div id=\"altair-viz-9af7f784c2534176bc8742dff8d0169a\"></div>\n",
       "<script type=\"text/javascript\">\n",
       "  var VEGA_DEBUG = (typeof VEGA_DEBUG == \"undefined\") ? {} : VEGA_DEBUG;\n",
       "  (function(spec, embedOpt){\n",
       "    let outputDiv = document.currentScript.previousElementSibling;\n",
       "    if (outputDiv.id !== \"altair-viz-9af7f784c2534176bc8742dff8d0169a\") {\n",
       "      outputDiv = document.getElementById(\"altair-viz-9af7f784c2534176bc8742dff8d0169a\");\n",
       "    }\n",
       "\n",
       "    const paths = {\n",
       "      \"vega\": \"https://cdn.jsdelivr.net/npm/vega@5?noext\",\n",
       "      \"vega-lib\": \"https://cdn.jsdelivr.net/npm/vega-lib?noext\",\n",
       "      \"vega-lite\": \"https://cdn.jsdelivr.net/npm/vega-lite@5.20.1?noext\",\n",
       "      \"vega-embed\": \"https://cdn.jsdelivr.net/npm/vega-embed@6?noext\",\n",
       "    };\n",
       "\n",
       "    function maybeLoadScript(lib, version) {\n",
       "      var key = `${lib.replace(\"-\", \"\")}_version`;\n",
       "      return (VEGA_DEBUG[key] == version) ?\n",
       "        Promise.resolve(paths[lib]) :\n",
       "        new Promise(function(resolve, reject) {\n",
       "          var s = document.createElement('script');\n",
       "          document.getElementsByTagName(\"head\")[0].appendChild(s);\n",
       "          s.async = true;\n",
       "          s.onload = () => {\n",
       "            VEGA_DEBUG[key] = version;\n",
       "            return resolve(paths[lib]);\n",
       "          };\n",
       "          s.onerror = () => reject(`Error loading script: ${paths[lib]}`);\n",
       "          s.src = paths[lib];\n",
       "        });\n",
       "    }\n",
       "\n",
       "    function showError(err) {\n",
       "      outputDiv.innerHTML = `<div class=\"error\" style=\"color:red;\">${err}</div>`;\n",
       "      throw err;\n",
       "    }\n",
       "\n",
       "    function displayChart(vegaEmbed) {\n",
       "      vegaEmbed(outputDiv, spec, embedOpt)\n",
       "        .catch(err => showError(`Javascript Error: ${err.message}<br>This usually means there's a typo in your chart specification. See the javascript console for the full traceback.`));\n",
       "    }\n",
       "\n",
       "    if(typeof define === \"function\" && define.amd) {\n",
       "      requirejs.config({paths});\n",
       "      let deps = [\"vega-embed\"];\n",
       "      require(deps, displayChart, err => showError(`Error loading script: ${err.message}`));\n",
       "    } else {\n",
       "      maybeLoadScript(\"vega\", \"5\")\n",
       "        .then(() => maybeLoadScript(\"vega-lite\", \"5.20.1\"))\n",
       "        .then(() => maybeLoadScript(\"vega-embed\", \"6\"))\n",
       "        .catch(showError)\n",
       "        .then(() => displayChart(vegaEmbed));\n",
       "    }\n",
       "  })({\"config\": {\"view\": {\"continuousWidth\": 300, \"continuousHeight\": 300}}, \"data\": {\"name\": \"data-4408db3b97911895efd9e0689dbfe269\"}, \"mark\": {\"type\": \"line\", \"point\": false}, \"encoding\": {\"color\": {\"field\": \"type\", \"title\": \"Thread Type\", \"type\": \"nominal\"}, \"tooltip\": [{\"field\": \"year\", \"title\": \"Year\", \"type\": \"quantitative\"}, {\"field\": \"month\", \"title\": \"Month\", \"type\": \"quantitative\"}, {\"field\": \"type\", \"title\": \"Type\", \"type\": \"nominal\"}, {\"field\": \"comments\", \"format\": \".1f\", \"title\": \"Comments\", \"type\": \"quantitative\"}], \"x\": {\"axis\": {\"format\": \"d\"}, \"field\": \"timeline\", \"title\": \"Year\", \"type\": \"quantitative\"}, \"y\": {\"field\": \"comments\", \"title\": \"Comments per Thread\", \"type\": \"quantitative\"}}, \"height\": 400, \"title\": \"Monthly Hiring vs Seeking Work Thread Activity\", \"width\": 800, \"$schema\": \"https://vega.github.io/schema/vega-lite/v5.20.1.json\", \"datasets\": {\"data-4408db3b97911895efd9e0689dbfe269\": [{\"year\": 2013, \"month\": 1, \"type\": \"Who is Hiring\", \"comments\": 253.0, \"timeline\": 2013.0833333333333}, {\"year\": 2013, \"month\": 2, \"type\": \"Who is Hiring\", \"comments\": 267.0, \"timeline\": 2013.1666666666667}, {\"year\": 2013, \"month\": 3, \"type\": \"Who is Hiring\", \"comments\": 365.0, \"timeline\": 2013.25}, {\"year\": 2013, \"month\": 4, \"type\": \"Who is Hiring\", \"comments\": 421.0, \"timeline\": 2013.3333333333333}, {\"year\": 2013, \"month\": 5, \"type\": \"Who is Hiring\", \"comments\": 437.0, \"timeline\": 2013.4166666666667}, {\"year\": 2013, \"month\": 6, \"type\": \"Who is Hiring\", \"comments\": 359.0, \"timeline\": 2013.5}, {\"year\": 2013, \"month\": 7, \"type\": \"Who is Hiring\", \"comments\": 372.0, \"timeline\": 2013.5833333333333}, {\"year\": 2013, \"month\": 8, \"type\": \"Who is Hiring\", \"comments\": 397.0, \"timeline\": 2013.6666666666667}, {\"year\": 2013, \"month\": 9, \"type\": \"Who is Hiring\", \"comments\": 389.0, \"timeline\": 2013.75}, {\"year\": 2013, \"month\": 10, \"type\": \"Who is Hiring\", \"comments\": 632.0, \"timeline\": 2013.8333333333333}, {\"year\": 2013, \"month\": 11, \"type\": \"Who is Hiring\", \"comments\": 389.0, \"timeline\": 2013.9166666666667}, {\"year\": 2013, \"month\": 12, \"type\": \"Who is Hiring\", \"comments\": 321.0, \"timeline\": 2014.0}, {\"year\": 2014, \"month\": 1, \"type\": \"Who is Hiring\", \"comments\": 288.0, \"timeline\": 2014.0833333333333}, {\"year\": 2014, \"month\": 2, \"type\": \"Who is Hiring\", \"comments\": 381.0, \"timeline\": 2014.1666666666667}, {\"year\": 2014, \"month\": 3, \"type\": \"Who is Hiring\", \"comments\": 431.0, \"timeline\": 2014.25}, {\"year\": 2014, \"month\": 4, \"type\": \"Who is Hiring\", \"comments\": 530.0, \"timeline\": 2014.3333333333333}, {\"year\": 2014, \"month\": 5, \"type\": \"Who is Hiring\", \"comments\": 493.0, \"timeline\": 2014.4166666666667}, {\"year\": 2014, \"month\": 5, \"type\": \"Who wants to be hired\", \"comments\": 381.0, \"timeline\": 2014.4166666666667}, {\"year\": 2014, \"month\": 6, \"type\": \"Who is Hiring\", \"comments\": 435.0, \"timeline\": 2014.5}, {\"year\": 2014, \"month\": 6, \"type\": \"Who wants to be hired\", \"comments\": 267.0, \"timeline\": 2014.5}, {\"year\": 2014, \"month\": 7, \"type\": \"Who is Hiring\", \"comments\": 493.0, \"timeline\": 2014.5833333333333}, {\"year\": 2014, \"month\": 7, \"type\": \"Who wants to be hired\", \"comments\": 207.0, \"timeline\": 2014.5833333333333}, {\"year\": 2014, \"month\": 8, \"type\": \"Who is Hiring\", \"comments\": 587.0, \"timeline\": 2014.6666666666667}, {\"year\": 2014, \"month\": 8, \"type\": \"Who wants to be hired\", \"comments\": 333.0, \"timeline\": 2014.6666666666667}, {\"year\": 2014, \"month\": 9, \"type\": \"Who is Hiring\", \"comments\": 324.0, \"timeline\": 2014.75}, {\"year\": 2014, \"month\": 9, \"type\": \"Who wants to be hired\", \"comments\": 215.0, \"timeline\": 2014.75}, {\"year\": 2014, \"month\": 10, \"type\": \"Who is Hiring\", \"comments\": 425.0, \"timeline\": 2014.8333333333333}, {\"year\": 2014, \"month\": 10, \"type\": \"Who wants to be hired\", \"comments\": 135.0, \"timeline\": 2014.8333333333333}, {\"year\": 2014, \"month\": 11, \"type\": \"Who is Hiring\", \"comments\": 425.0, \"timeline\": 2014.9166666666667}, {\"year\": 2014, \"month\": 11, \"type\": \"Who wants to be hired\", \"comments\": 140.0, \"timeline\": 2014.9166666666667}, {\"year\": 2014, \"month\": 12, \"type\": \"Who is Hiring\", \"comments\": 565.0, \"timeline\": 2015.0}, {\"year\": 2014, \"month\": 12, \"type\": \"Who wants to be hired\", \"comments\": 199.0, \"timeline\": 2015.0}, {\"year\": 2015, \"month\": 1, \"type\": \"Who is Hiring\", \"comments\": 331.0, \"timeline\": 2015.0833333333333}, {\"year\": 2015, \"month\": 1, \"type\": \"Who wants to be hired\", \"comments\": 95.0, \"timeline\": 2015.0833333333333}, {\"year\": 2015, \"month\": 2, \"type\": \"Who is Hiring\", \"comments\": 646.0, \"timeline\": 2015.1666666666667}, {\"year\": 2015, \"month\": 2, \"type\": \"Who wants to be hired\", \"comments\": 162.0, \"timeline\": 2015.1666666666667}, {\"year\": 2015, \"month\": 3, \"type\": \"Who is Hiring\", \"comments\": 678.0, \"timeline\": 2015.25}, {\"year\": 2015, \"month\": 3, \"type\": \"Who wants to be hired\", \"comments\": 172.0, \"timeline\": 2015.25}, {\"year\": 2015, \"month\": 4, \"type\": \"Who is Hiring\", \"comments\": 1066.0, \"timeline\": 2015.3333333333333}, {\"year\": 2015, \"month\": 4, \"type\": \"Who wants to be hired\", \"comments\": 190.0, \"timeline\": 2015.3333333333333}, {\"year\": 2015, \"month\": 5, \"type\": \"Who is Hiring\", \"comments\": 893.0, \"timeline\": 2015.4166666666667}, {\"year\": 2015, \"month\": 5, \"type\": \"Who wants to be hired\", \"comments\": 119.0, \"timeline\": 2015.4166666666667}, {\"year\": 2015, \"month\": 6, \"type\": \"Who is Hiring\", \"comments\": 777.0, \"timeline\": 2015.5}, {\"year\": 2015, \"month\": 6, \"type\": \"Who wants to be hired\", \"comments\": 271.0, \"timeline\": 2015.5}, {\"year\": 2015, \"month\": 7, \"type\": \"Who is Hiring\", \"comments\": 1224.0, \"timeline\": 2015.5833333333333}, {\"year\": 2015, \"month\": 7, \"type\": \"Who wants to be hired\", \"comments\": 140.0, \"timeline\": 2015.5833333333333}, {\"year\": 2015, \"month\": 8, \"type\": \"Who is Hiring\", \"comments\": 970.0, \"timeline\": 2015.6666666666667}, {\"year\": 2015, \"month\": 8, \"type\": \"Who wants to be hired\", \"comments\": 170.0, \"timeline\": 2015.6666666666667}, {\"year\": 2015, \"month\": 9, \"type\": \"Who is Hiring\", \"comments\": 866.0, \"timeline\": 2015.75}, {\"year\": 2015, \"month\": 9, \"type\": \"Who wants to be hired\", \"comments\": 185.0, \"timeline\": 2015.75}, {\"year\": 2015, \"month\": 10, \"type\": \"Who is Hiring\", \"comments\": 709.0, \"timeline\": 2015.8333333333333}, {\"year\": 2015, \"month\": 10, \"type\": \"Who wants to be hired\", \"comments\": 116.0, \"timeline\": 2015.8333333333333}, {\"year\": 2015, \"month\": 11, \"type\": \"Who is Hiring\", \"comments\": 882.0, \"timeline\": 2015.9166666666667}, {\"year\": 2015, \"month\": 11, \"type\": \"Who wants to be hired\", \"comments\": 95.0, \"timeline\": 2015.9166666666667}, {\"year\": 2015, \"month\": 12, \"type\": \"Who is Hiring\", \"comments\": 438.0, \"timeline\": 2016.0}, {\"year\": 2015, \"month\": 12, \"type\": \"Who wants to be hired\", \"comments\": 92.0, \"timeline\": 2016.0}, {\"year\": 2016, \"month\": 1, \"type\": \"Who is Hiring\", \"comments\": 469.0, \"timeline\": 2016.0833333333333}, {\"year\": 2016, \"month\": 1, \"type\": \"Who wants to be hired\", \"comments\": 118.0, \"timeline\": 2016.0833333333333}, {\"year\": 2016, \"month\": 2, \"type\": \"Who is Hiring\", \"comments\": 788.0, \"timeline\": 2016.1666666666667}, {\"year\": 2016, \"month\": 2, \"type\": \"Who wants to be hired\", \"comments\": 119.0, \"timeline\": 2016.1666666666667}, {\"year\": 2016, \"month\": 3, \"type\": \"Who is Hiring\", \"comments\": 823.0, \"timeline\": 2016.25}, {\"year\": 2016, \"month\": 3, \"type\": \"Who wants to be hired\", \"comments\": 197.0, \"timeline\": 2016.25}, {\"year\": 2016, \"month\": 4, \"type\": \"Who is Hiring\", \"comments\": 714.0, \"timeline\": 2016.3333333333333}, {\"year\": 2016, \"month\": 4, \"type\": \"Who wants to be hired\", \"comments\": 268.0, \"timeline\": 2016.3333333333333}, {\"year\": 2016, \"month\": 5, \"type\": \"Who is Hiring\", \"comments\": 940.0, \"timeline\": 2016.4166666666667}, {\"year\": 2016, \"month\": 5, \"type\": \"Who wants to be hired\", \"comments\": 161.0, \"timeline\": 2016.4166666666667}, {\"year\": 2016, \"month\": 6, \"type\": \"Who is Hiring\", \"comments\": 1007.0, \"timeline\": 2016.5}, {\"year\": 2016, \"month\": 6, \"type\": \"Who wants to be hired\", \"comments\": 237.0, \"timeline\": 2016.5}, {\"year\": 2016, \"month\": 7, \"type\": \"Who is Hiring\", \"comments\": 894.0, \"timeline\": 2016.5833333333333}, {\"year\": 2016, \"month\": 7, \"type\": \"Who wants to be hired\", \"comments\": 199.0, \"timeline\": 2016.5833333333333}, {\"year\": 2016, \"month\": 8, \"type\": \"Who is Hiring\", \"comments\": 941.0, \"timeline\": 2016.6666666666667}, {\"year\": 2016, \"month\": 8, \"type\": \"Who wants to be hired\", \"comments\": 115.0, \"timeline\": 2016.6666666666667}, {\"year\": 2016, \"month\": 9, \"type\": \"Who is Hiring\", \"comments\": 906.0, \"timeline\": 2016.75}, {\"year\": 2016, \"month\": 9, \"type\": \"Who wants to be hired\", \"comments\": 154.0, \"timeline\": 2016.75}, {\"year\": 2016, \"month\": 10, \"type\": \"Who is Hiring\", \"comments\": 982.0, \"timeline\": 2016.8333333333333}, {\"year\": 2016, \"month\": 10, \"type\": \"Who wants to be hired\", \"comments\": 250.0, \"timeline\": 2016.8333333333333}, {\"year\": 2016, \"month\": 11, \"type\": \"Who is Hiring\", \"comments\": 1132.0, \"timeline\": 2016.9166666666667}, {\"year\": 2016, \"month\": 11, \"type\": \"Who wants to be hired\", \"comments\": 237.0, \"timeline\": 2016.9166666666667}, {\"year\": 2016, \"month\": 12, \"type\": \"Who is Hiring\", \"comments\": 830.0, \"timeline\": 2017.0}, {\"year\": 2016, \"month\": 12, \"type\": \"Who wants to be hired\", \"comments\": 116.0, \"timeline\": 2017.0}, {\"year\": 2017, \"month\": 1, \"type\": \"Who is Hiring\", \"comments\": 938.0, \"timeline\": 2017.0833333333333}, {\"year\": 2017, \"month\": 1, \"type\": \"Who wants to be hired\", \"comments\": 261.0, \"timeline\": 2017.0833333333333}, {\"year\": 2017, \"month\": 2, \"type\": \"Who is Hiring\", \"comments\": 1222.0, \"timeline\": 2017.1666666666667}, {\"year\": 2017, \"month\": 2, \"type\": \"Who wants to be hired\", \"comments\": 102.0, \"timeline\": 2017.1666666666667}, {\"year\": 2017, \"month\": 3, \"type\": \"Who is Hiring\", \"comments\": 1054.0, \"timeline\": 2017.25}, {\"year\": 2017, \"month\": 3, \"type\": \"Who wants to be hired\", \"comments\": 113.0, \"timeline\": 2017.25}, {\"year\": 2017, \"month\": 4, \"type\": \"Who is Hiring\", \"comments\": 942.0, \"timeline\": 2017.3333333333333}, {\"year\": 2017, \"month\": 4, \"type\": \"Who wants to be hired\", \"comments\": 159.0, \"timeline\": 2017.3333333333333}, {\"year\": 2017, \"month\": 5, \"type\": \"Who is Hiring\", \"comments\": 1083.0, \"timeline\": 2017.4166666666667}, {\"year\": 2017, \"month\": 5, \"type\": \"Who wants to be hired\", \"comments\": 224.0, \"timeline\": 2017.4166666666667}, {\"year\": 2017, \"month\": 6, \"type\": \"Who is Hiring\", \"comments\": 1016.0, \"timeline\": 2017.5}, {\"year\": 2017, \"month\": 6, \"type\": \"Who wants to be hired\", \"comments\": 98.0, \"timeline\": 2017.5}, {\"year\": 2017, \"month\": 7, \"type\": \"Who is Hiring\", \"comments\": 970.0, \"timeline\": 2017.5833333333333}, {\"year\": 2017, \"month\": 7, \"type\": \"Who wants to be hired\", \"comments\": 98.0, \"timeline\": 2017.5833333333333}, {\"year\": 2017, \"month\": 8, \"type\": \"Who is Hiring\", \"comments\": 944.0, \"timeline\": 2017.6666666666667}, {\"year\": 2017, \"month\": 8, \"type\": \"Who wants to be hired\", \"comments\": 217.0, \"timeline\": 2017.6666666666667}, {\"year\": 2017, \"month\": 9, \"type\": \"Who is Hiring\", \"comments\": 967.0, \"timeline\": 2017.75}, {\"year\": 2017, \"month\": 9, \"type\": \"Who wants to be hired\", \"comments\": 107.0, \"timeline\": 2017.75}, {\"year\": 2017, \"month\": 10, \"type\": \"Who is Hiring\", \"comments\": 1127.0, \"timeline\": 2017.8333333333333}, {\"year\": 2017, \"month\": 10, \"type\": \"Who wants to be hired\", \"comments\": 119.0, \"timeline\": 2017.8333333333333}, {\"year\": 2017, \"month\": 11, \"type\": \"Who is Hiring\", \"comments\": 1186.0, \"timeline\": 2017.9166666666667}, {\"year\": 2017, \"month\": 11, \"type\": \"Who wants to be hired\", \"comments\": 136.0, \"timeline\": 2017.9166666666667}, {\"year\": 2017, \"month\": 12, \"type\": \"Who is Hiring\", \"comments\": 985.0, \"timeline\": 2018.0}, {\"year\": 2017, \"month\": 12, \"type\": \"Who wants to be hired\", \"comments\": 64.0, \"timeline\": 2018.0}, {\"year\": 2018, \"month\": 1, \"type\": \"Who is Hiring\", \"comments\": 894.0, \"timeline\": 2018.0833333333333}, {\"year\": 2018, \"month\": 1, \"type\": \"Who wants to be hired\", \"comments\": 78.0, \"timeline\": 2018.0833333333333}, {\"year\": 2018, \"month\": 2, \"type\": \"Who is Hiring\", \"comments\": 1054.0, \"timeline\": 2018.1666666666667}, {\"year\": 2018, \"month\": 2, \"type\": \"Who wants to be hired\", \"comments\": 72.0, \"timeline\": 2018.1666666666667}, {\"year\": 2018, \"month\": 3, \"type\": \"Who is Hiring\", \"comments\": 1176.0, \"timeline\": 2018.25}, {\"year\": 2018, \"month\": 3, \"type\": \"Who wants to be hired\", \"comments\": 77.0, \"timeline\": 2018.25}, {\"year\": 2018, \"month\": 4, \"type\": \"Who is Hiring\", \"comments\": 1116.0, \"timeline\": 2018.3333333333333}, {\"year\": 2018, \"month\": 4, \"type\": \"Who wants to be hired\", \"comments\": 134.0, \"timeline\": 2018.3333333333333}, {\"year\": 2018, \"month\": 5, \"type\": \"Who is Hiring\", \"comments\": 1182.0, \"timeline\": 2018.4166666666667}, {\"year\": 2018, \"month\": 5, \"type\": \"Who wants to be hired\", \"comments\": 64.0, \"timeline\": 2018.4166666666667}, {\"year\": 2018, \"month\": 6, \"type\": \"Who is Hiring\", \"comments\": 900.0, \"timeline\": 2018.5}, {\"year\": 2018, \"month\": 6, \"type\": \"Who wants to be hired\", \"comments\": 82.0, \"timeline\": 2018.5}, {\"year\": 2018, \"month\": 7, \"type\": \"Who is Hiring\", \"comments\": 860.0, \"timeline\": 2018.5833333333333}, {\"year\": 2018, \"month\": 7, \"type\": \"Who wants to be hired\", \"comments\": 112.0, \"timeline\": 2018.5833333333333}, {\"year\": 2018, \"month\": 8, \"type\": \"Who is Hiring\", \"comments\": 1083.0, \"timeline\": 2018.6666666666667}, {\"year\": 2018, \"month\": 8, \"type\": \"Who wants to be hired\", \"comments\": 79.0, \"timeline\": 2018.6666666666667}, {\"year\": 2018, \"month\": 9, \"type\": \"Who is Hiring\", \"comments\": 915.0, \"timeline\": 2018.75}, {\"year\": 2018, \"month\": 9, \"type\": \"Who wants to be hired\", \"comments\": 67.0, \"timeline\": 2018.75}, {\"year\": 2018, \"month\": 10, \"type\": \"Who is Hiring\", \"comments\": 1091.0, \"timeline\": 2018.8333333333333}, {\"year\": 2018, \"month\": 10, \"type\": \"Who wants to be hired\", \"comments\": 103.0, \"timeline\": 2018.8333333333333}, {\"year\": 2018, \"month\": 11, \"type\": \"Who is Hiring\", \"comments\": 1037.0, \"timeline\": 2018.9166666666667}, {\"year\": 2018, \"month\": 11, \"type\": \"Who wants to be hired\", \"comments\": 147.0, \"timeline\": 2018.9166666666667}, {\"year\": 2018, \"month\": 12, \"type\": \"Who is Hiring\", \"comments\": 953.0, \"timeline\": 2019.0}, {\"year\": 2018, \"month\": 12, \"type\": \"Who wants to be hired\", \"comments\": 170.0, \"timeline\": 2019.0}, {\"year\": 2019, \"month\": 1, \"type\": \"Who is Hiring\", \"comments\": 955.0, \"timeline\": 2019.0833333333333}, {\"year\": 2019, \"month\": 1, \"type\": \"Who wants to be hired\", \"comments\": 242.0, \"timeline\": 2019.0833333333333}, {\"year\": 2019, \"month\": 2, \"type\": \"Who is Hiring\", \"comments\": 833.0, \"timeline\": 2019.1666666666667}, {\"year\": 2019, \"month\": 2, \"type\": \"Who wants to be hired\", \"comments\": 185.0, \"timeline\": 2019.1666666666667}, {\"year\": 2019, \"month\": 3, \"type\": \"Who is Hiring\", \"comments\": 867.0, \"timeline\": 2019.25}, {\"year\": 2019, \"month\": 3, \"type\": \"Who wants to be hired\", \"comments\": 232.0, \"timeline\": 2019.25}, {\"year\": 2019, \"month\": 4, \"type\": \"Who is Hiring\", \"comments\": 862.0, \"timeline\": 2019.3333333333333}, {\"year\": 2019, \"month\": 4, \"type\": \"Who wants to be hired\", \"comments\": 306.0, \"timeline\": 2019.3333333333333}, {\"year\": 2019, \"month\": 5, \"type\": \"Who is Hiring\", \"comments\": 1010.0, \"timeline\": 2019.4166666666667}, {\"year\": 2019, \"month\": 5, \"type\": \"Who wants to be hired\", \"comments\": 270.0, \"timeline\": 2019.4166666666667}, {\"year\": 2019, \"month\": 6, \"type\": \"Who is Hiring\", \"comments\": 791.0, \"timeline\": 2019.5}, {\"year\": 2019, \"month\": 6, \"type\": \"Who wants to be hired\", \"comments\": 212.0, \"timeline\": 2019.5}, {\"year\": 2019, \"month\": 7, \"type\": \"Who is Hiring\", \"comments\": 895.0, \"timeline\": 2019.5833333333333}, {\"year\": 2019, \"month\": 7, \"type\": \"Who wants to be hired\", \"comments\": 707.0, \"timeline\": 2019.5833333333333}, {\"year\": 2019, \"month\": 8, \"type\": \"Who is Hiring\", \"comments\": 854.0, \"timeline\": 2019.6666666666667}, {\"year\": 2019, \"month\": 8, \"type\": \"Who wants to be hired\", \"comments\": 257.0, \"timeline\": 2019.6666666666667}, {\"year\": 2019, \"month\": 9, \"type\": \"Who is Hiring\", \"comments\": 707.0, \"timeline\": 2019.75}, {\"year\": 2019, \"month\": 9, \"type\": \"Who wants to be hired\", \"comments\": 160.0, \"timeline\": 2019.75}, {\"year\": 2019, \"month\": 10, \"type\": \"Who is Hiring\", \"comments\": 893.0, \"timeline\": 2019.8333333333333}, {\"year\": 2019, \"month\": 10, \"type\": \"Who wants to be hired\", \"comments\": 367.0, \"timeline\": 2019.8333333333333}, {\"year\": 2019, \"month\": 11, \"type\": \"Who is Hiring\", \"comments\": 743.0, \"timeline\": 2019.9166666666667}, {\"year\": 2019, \"month\": 11, \"type\": \"Who wants to be hired\", \"comments\": 251.0, \"timeline\": 2019.9166666666667}, {\"year\": 2019, \"month\": 12, \"type\": \"Who is Hiring\", \"comments\": 670.0, \"timeline\": 2020.0}, {\"year\": 2019, \"month\": 12, \"type\": \"Who wants to be hired\", \"comments\": 247.0, \"timeline\": 2020.0}, {\"year\": 2020, \"month\": 1, \"type\": \"Who is Hiring\", \"comments\": 695.0, \"timeline\": 2020.0833333333333}, {\"year\": 2020, \"month\": 1, \"type\": \"Who wants to be hired\", \"comments\": 240.0, \"timeline\": 2020.0833333333333}, {\"year\": 2020, \"month\": 2, \"type\": \"Who is Hiring\", \"comments\": 675.0, \"timeline\": 2020.1666666666667}, {\"year\": 2020, \"month\": 2, \"type\": \"Who wants to be hired\", \"comments\": 223.0, \"timeline\": 2020.1666666666667}, {\"year\": 2020, \"month\": 3, \"type\": \"Who is Hiring\", \"comments\": 1562.0, \"timeline\": 2020.25}, {\"year\": 2020, \"month\": 3, \"type\": \"Who wants to be hired\", \"comments\": 731.0, \"timeline\": 2020.25}, {\"year\": 2020, \"month\": 4, \"type\": \"Who is Hiring\", \"comments\": 599.0, \"timeline\": 2020.3333333333333}, {\"year\": 2020, \"month\": 4, \"type\": \"Who wants to be hired\", \"comments\": 516.0, \"timeline\": 2020.3333333333333}, {\"year\": 2020, \"month\": 5, \"type\": \"Who is Hiring\", \"comments\": 685.0, \"timeline\": 2020.4166666666667}, {\"year\": 2020, \"month\": 5, \"type\": \"Who wants to be hired\", \"comments\": 328.0, \"timeline\": 2020.4166666666667}, {\"year\": 2020, \"month\": 6, \"type\": \"Who is Hiring\", \"comments\": 657.0, \"timeline\": 2020.5}, {\"year\": 2020, \"month\": 6, \"type\": \"Who wants to be hired\", \"comments\": 302.0, \"timeline\": 2020.5}, {\"year\": 2020, \"month\": 7, \"type\": \"Who is Hiring\", \"comments\": 679.0, \"timeline\": 2020.5833333333333}, {\"year\": 2020, \"month\": 7, \"type\": \"Who wants to be hired\", \"comments\": 283.0, \"timeline\": 2020.5833333333333}, {\"year\": 2020, \"month\": 8, \"type\": \"Who is Hiring\", \"comments\": 827.0, \"timeline\": 2020.6666666666667}, {\"year\": 2020, \"month\": 8, \"type\": \"Who wants to be hired\", \"comments\": 408.0, \"timeline\": 2020.6666666666667}, {\"year\": 2020, \"month\": 9, \"type\": \"Who is Hiring\", \"comments\": 697.0, \"timeline\": 2020.75}, {\"year\": 2020, \"month\": 9, \"type\": \"Who wants to be hired\", \"comments\": 345.0, \"timeline\": 2020.75}, {\"year\": 2020, \"month\": 10, \"type\": \"Who is Hiring\", \"comments\": 809.0, \"timeline\": 2020.8333333333333}, {\"year\": 2020, \"month\": 10, \"type\": \"Who wants to be hired\", \"comments\": 289.0, \"timeline\": 2020.8333333333333}, {\"year\": 2020, \"month\": 11, \"type\": \"Who is Hiring\", \"comments\": 804.0, \"timeline\": 2020.9166666666667}, {\"year\": 2020, \"month\": 11, \"type\": \"Who wants to be hired\", \"comments\": 254.0, \"timeline\": 2020.9166666666667}, {\"year\": 2020, \"month\": 12, \"type\": \"Who is Hiring\", \"comments\": 762.0, \"timeline\": 2021.0}, {\"year\": 2020, \"month\": 12, \"type\": \"Who wants to be hired\", \"comments\": 273.0, \"timeline\": 2021.0}, {\"year\": 2021, \"month\": 1, \"type\": \"Who is Hiring\", \"comments\": 851.0, \"timeline\": 2021.0833333333333}, {\"year\": 2021, \"month\": 1, \"type\": \"Who wants to be hired\", \"comments\": 279.0, \"timeline\": 2021.0833333333333}, {\"year\": 2021, \"month\": 2, \"type\": \"Who is Hiring\", \"comments\": 1047.0, \"timeline\": 2021.1666666666667}, {\"year\": 2021, \"month\": 2, \"type\": \"Who wants to be hired\", \"comments\": 231.0, \"timeline\": 2021.1666666666667}, {\"year\": 2021, \"month\": 3, \"type\": \"Who is Hiring\", \"comments\": 961.0, \"timeline\": 2021.25}, {\"year\": 2021, \"month\": 3, \"type\": \"Who wants to be hired\", \"comments\": 224.0, \"timeline\": 2021.25}, {\"year\": 2021, \"month\": 4, \"type\": \"Who is Hiring\", \"comments\": 967.0, \"timeline\": 2021.3333333333333}, {\"year\": 2021, \"month\": 4, \"type\": \"Who wants to be hired\", \"comments\": 223.0, \"timeline\": 2021.3333333333333}, {\"year\": 2021, \"month\": 5, \"type\": \"Who is Hiring\", \"comments\": 922.0, \"timeline\": 2021.4166666666667}, {\"year\": 2021, \"month\": 5, \"type\": \"Who wants to be hired\", \"comments\": 238.0, \"timeline\": 2021.4166666666667}, {\"year\": 2021, \"month\": 6, \"type\": \"Who is Hiring\", \"comments\": 1035.0, \"timeline\": 2021.5}, {\"year\": 2021, \"month\": 6, \"type\": \"Who wants to be hired\", \"comments\": 214.0, \"timeline\": 2021.5}, {\"year\": 2021, \"month\": 7, \"type\": \"Who is Hiring\", \"comments\": 981.0, \"timeline\": 2021.5833333333333}, {\"year\": 2021, \"month\": 7, \"type\": \"Who wants to be hired\", \"comments\": 163.0, \"timeline\": 2021.5833333333333}, {\"year\": 2021, \"month\": 8, \"type\": \"Who is Hiring\", \"comments\": 900.0, \"timeline\": 2021.6666666666667}, {\"year\": 2021, \"month\": 8, \"type\": \"Who wants to be hired\", \"comments\": 216.0, \"timeline\": 2021.6666666666667}, {\"year\": 2021, \"month\": 9, \"type\": \"Who is Hiring\", \"comments\": 962.0, \"timeline\": 2021.75}, {\"year\": 2021, \"month\": 9, \"type\": \"Who wants to be hired\", \"comments\": 210.0, \"timeline\": 2021.75}, {\"year\": 2021, \"month\": 10, \"type\": \"Who is Hiring\", \"comments\": 845.0, \"timeline\": 2021.8333333333333}, {\"year\": 2021, \"month\": 10, \"type\": \"Who wants to be hired\", \"comments\": 193.0, \"timeline\": 2021.8333333333333}, {\"year\": 2021, \"month\": 11, \"type\": \"Who is Hiring\", \"comments\": 1011.0, \"timeline\": 2021.9166666666667}, {\"year\": 2021, \"month\": 11, \"type\": \"Who wants to be hired\", \"comments\": 239.0, \"timeline\": 2021.9166666666667}, {\"year\": 2021, \"month\": 12, \"type\": \"Who is Hiring\", \"comments\": 822.0, \"timeline\": 2022.0}, {\"year\": 2021, \"month\": 12, \"type\": \"Who wants to be hired\", \"comments\": 199.0, \"timeline\": 2022.0}, {\"year\": 2022, \"month\": 1, \"type\": \"Who is Hiring\", \"comments\": 761.0, \"timeline\": 2022.0833333333333}, {\"year\": 2022, \"month\": 1, \"type\": \"Who wants to be hired\", \"comments\": 228.0, \"timeline\": 2022.0833333333333}, {\"year\": 2022, \"month\": 2, \"type\": \"Who is Hiring\", \"comments\": 890.0, \"timeline\": 2022.1666666666667}, {\"year\": 2022, \"month\": 2, \"type\": \"Who wants to be hired\", \"comments\": 255.0, \"timeline\": 2022.1666666666667}, {\"year\": 2022, \"month\": 3, \"type\": \"Who is Hiring\", \"comments\": 860.0, \"timeline\": 2022.25}, {\"year\": 2022, \"month\": 3, \"type\": \"Who wants to be hired\", \"comments\": 280.0, \"timeline\": 2022.25}, {\"year\": 2022, \"month\": 4, \"type\": \"Who is Hiring\", \"comments\": 797.0, \"timeline\": 2022.3333333333333}, {\"year\": 2022, \"month\": 4, \"type\": \"Who wants to be hired\", \"comments\": 180.0, \"timeline\": 2022.3333333333333}, {\"year\": 2022, \"month\": 5, \"type\": \"Who is Hiring\", \"comments\": 879.0, \"timeline\": 2022.4166666666667}, {\"year\": 2022, \"month\": 5, \"type\": \"Who wants to be hired\", \"comments\": 272.0, \"timeline\": 2022.4166666666667}, {\"year\": 2022, \"month\": 6, \"type\": \"Who is Hiring\", \"comments\": 845.0, \"timeline\": 2022.5}, {\"year\": 2022, \"month\": 6, \"type\": \"Who wants to be hired\", \"comments\": 233.0, \"timeline\": 2022.5}, {\"year\": 2022, \"month\": 7, \"type\": \"Who is Hiring\", \"comments\": 612.0, \"timeline\": 2022.5833333333333}, {\"year\": 2022, \"month\": 7, \"type\": \"Who wants to be hired\", \"comments\": 240.0, \"timeline\": 2022.5833333333333}, {\"year\": 2022, \"month\": 8, \"type\": \"Who is Hiring\", \"comments\": 750.0, \"timeline\": 2022.6666666666667}, {\"year\": 2022, \"month\": 8, \"type\": \"Who wants to be hired\", \"comments\": 316.0, \"timeline\": 2022.6666666666667}, {\"year\": 2022, \"month\": 9, \"type\": \"Who is Hiring\", \"comments\": 611.0, \"timeline\": 2022.75}, {\"year\": 2022, \"month\": 9, \"type\": \"Who wants to be hired\", \"comments\": 298.0, \"timeline\": 2022.75}, {\"year\": 2022, \"month\": 10, \"type\": \"Who is Hiring\", \"comments\": 870.0, \"timeline\": 2022.8333333333333}, {\"year\": 2022, \"month\": 10, \"type\": \"Who wants to be hired\", \"comments\": 272.0, \"timeline\": 2022.8333333333333}, {\"year\": 2022, \"month\": 11, \"type\": \"Who is Hiring\", \"comments\": 712.0, \"timeline\": 2022.9166666666667}, {\"year\": 2022, \"month\": 11, \"type\": \"Who wants to be hired\", \"comments\": 304.0, \"timeline\": 2022.9166666666667}, {\"year\": 2022, \"month\": 12, \"type\": \"Who is Hiring\", \"comments\": 566.0, \"timeline\": 2023.0}, {\"year\": 2022, \"month\": 12, \"type\": \"Who wants to be hired\", \"comments\": 200.0, \"timeline\": 2023.0}, {\"year\": 2023, \"month\": 1, \"type\": \"Who is Hiring\", \"comments\": 526.0, \"timeline\": 2023.0833333333333}, {\"year\": 2023, \"month\": 1, \"type\": \"Who wants to be hired\", \"comments\": 396.0, \"timeline\": 2023.0833333333333}, {\"year\": 2023, \"month\": 2, \"type\": \"Who is Hiring\", \"comments\": 587.0, \"timeline\": 2023.1666666666667}, {\"year\": 2023, \"month\": 2, \"type\": \"Who wants to be hired\", \"comments\": 396.0, \"timeline\": 2023.1666666666667}, {\"year\": 2023, \"month\": 3, \"type\": \"Who is Hiring\", \"comments\": 663.0, \"timeline\": 2023.25}, {\"year\": 2023, \"month\": 3, \"type\": \"Who wants to be hired\", \"comments\": 397.0, \"timeline\": 2023.25}, {\"year\": 2023, \"month\": 4, \"type\": \"Who is Hiring\", \"comments\": 580.0, \"timeline\": 2023.3333333333333}, {\"year\": 2023, \"month\": 4, \"type\": \"Who wants to be hired\", \"comments\": 477.0, \"timeline\": 2023.3333333333333}, {\"year\": 2023, \"month\": 5, \"type\": \"Who is Hiring\", \"comments\": 502.0, \"timeline\": 2023.4166666666667}, {\"year\": 2023, \"month\": 5, \"type\": \"Who wants to be hired\", \"comments\": 386.0, \"timeline\": 2023.4166666666667}, {\"year\": 2023, \"month\": 6, \"type\": \"Who is Hiring\", \"comments\": 532.0, \"timeline\": 2023.5}, {\"year\": 2023, \"month\": 6, \"type\": \"Who wants to be hired\", \"comments\": 455.0, \"timeline\": 2023.5}, {\"year\": 2023, \"month\": 7, \"type\": \"Who is Hiring\", \"comments\": 863.0, \"timeline\": 2023.5833333333333}, {\"year\": 2023, \"month\": 7, \"type\": \"Who wants to be hired\", \"comments\": 359.0, \"timeline\": 2023.5833333333333}, {\"year\": 2023, \"month\": 8, \"type\": \"Who is Hiring\", \"comments\": 537.0, \"timeline\": 2023.6666666666667}, {\"year\": 2023, \"month\": 8, \"type\": \"Who wants to be hired\", \"comments\": 465.0, \"timeline\": 2023.6666666666667}, {\"year\": 2023, \"month\": 9, \"type\": \"Who is Hiring\", \"comments\": 494.0, \"timeline\": 2023.75}, {\"year\": 2023, \"month\": 9, \"type\": \"Who wants to be hired\", \"comments\": 404.0, \"timeline\": 2023.75}, {\"year\": 2023, \"month\": 10, \"type\": \"Who is Hiring\", \"comments\": 642.0, \"timeline\": 2023.8333333333333}, {\"year\": 2023, \"month\": 10, \"type\": \"Who wants to be hired\", \"comments\": 516.0, \"timeline\": 2023.8333333333333}, {\"year\": 2023, \"month\": 11, \"type\": \"Who is Hiring\", \"comments\": 632.0, \"timeline\": 2023.9166666666667}, {\"year\": 2023, \"month\": 11, \"type\": \"Who wants to be hired\", \"comments\": 392.0, \"timeline\": 2023.9166666666667}, {\"year\": 2023, \"month\": 12, \"type\": \"Who is Hiring\", \"comments\": 512.0, \"timeline\": 2024.0}, {\"year\": 2023, \"month\": 12, \"type\": \"Who wants to be hired\", \"comments\": 424.0, \"timeline\": 2024.0}, {\"year\": 2024, \"month\": 1, \"type\": \"Who is Hiring\", \"comments\": 485.0, \"timeline\": 2024.0833333333333}, {\"year\": 2024, \"month\": 1, \"type\": \"Who wants to be hired\", \"comments\": 501.0, \"timeline\": 2024.0833333333333}, {\"year\": 2024, \"month\": 2, \"type\": \"Who is Hiring\", \"comments\": 555.0, \"timeline\": 2024.1666666666667}, {\"year\": 2024, \"month\": 2, \"type\": \"Who wants to be hired\", \"comments\": 500.0, \"timeline\": 2024.1666666666667}, {\"year\": 2024, \"month\": 3, \"type\": \"Who is Hiring\", \"comments\": 487.0, \"timeline\": 2024.25}, {\"year\": 2024, \"month\": 3, \"type\": \"Who wants to be hired\", \"comments\": 453.0, \"timeline\": 2024.25}, {\"year\": 2024, \"month\": 4, \"type\": \"Who is Hiring\", \"comments\": 574.0, \"timeline\": 2024.3333333333333}, {\"year\": 2024, \"month\": 4, \"type\": \"Who wants to be hired\", \"comments\": 486.0, \"timeline\": 2024.3333333333333}, {\"year\": 2024, \"month\": 5, \"type\": \"Who is Hiring\", \"comments\": 738.0, \"timeline\": 2024.4166666666667}, {\"year\": 2024, \"month\": 5, \"type\": \"Who wants to be hired\", \"comments\": 493.0, \"timeline\": 2024.4166666666667}, {\"year\": 2024, \"month\": 6, \"type\": \"Who is Hiring\", \"comments\": 560.0, \"timeline\": 2024.5}, {\"year\": 2024, \"month\": 6, \"type\": \"Who wants to be hired\", \"comments\": 509.0, \"timeline\": 2024.5}, {\"year\": 2024, \"month\": 7, \"type\": \"Who is Hiring\", \"comments\": 610.0, \"timeline\": 2024.5833333333333}, {\"year\": 2024, \"month\": 7, \"type\": \"Who wants to be hired\", \"comments\": 568.0, \"timeline\": 2024.5833333333333}, {\"year\": 2024, \"month\": 8, \"type\": \"Who is Hiring\", \"comments\": 469.0, \"timeline\": 2024.6666666666667}, {\"year\": 2024, \"month\": 8, \"type\": \"Who wants to be hired\", \"comments\": 416.0, \"timeline\": 2024.6666666666667}, {\"year\": 2024, \"month\": 9, \"type\": \"Who is Hiring\", \"comments\": 491.0, \"timeline\": 2024.75}, {\"year\": 2024, \"month\": 9, \"type\": \"Who wants to be hired\", \"comments\": 471.0, \"timeline\": 2024.75}, {\"year\": 2024, \"month\": 10, \"type\": \"Who is Hiring\", \"comments\": 431.0, \"timeline\": 2024.8333333333333}, {\"year\": 2024, \"month\": 10, \"type\": \"Who wants to be hired\", \"comments\": 441.0, \"timeline\": 2024.8333333333333}, {\"year\": 2024, \"month\": 11, \"type\": \"Who is Hiring\", \"comments\": 458.0, \"timeline\": 2024.9166666666667}, {\"year\": 2024, \"month\": 11, \"type\": \"Who wants to be hired\", \"comments\": 396.0, \"timeline\": 2024.9166666666667}, {\"year\": 2024, \"month\": 12, \"type\": \"Who is Hiring\", \"comments\": 463.0, \"timeline\": 2025.0}, {\"year\": 2024, \"month\": 12, \"type\": \"Who wants to be hired\", \"comments\": 475.0, \"timeline\": 2025.0}, {\"year\": 2025, \"month\": 1, \"type\": \"Who is Hiring\", \"comments\": 487.0, \"timeline\": 2025.0833333333333}, {\"year\": 2025, \"month\": 1, \"type\": \"Who wants to be hired\", \"comments\": 400.0, \"timeline\": 2025.0833333333333}, {\"year\": 2025, \"month\": 2, \"type\": \"Who is Hiring\", \"comments\": 624.0, \"timeline\": 2025.1666666666667}, {\"year\": 2025, \"month\": 2, \"type\": \"Who wants to be hired\", \"comments\": 552.0, \"timeline\": 2025.1666666666667}]}}, {\"mode\": \"vega-lite\"});\n",
       "</script>"
      ],
      "text/plain": [
       "alt.Chart(...)"
      ]
     },
     "execution_count": 6,
     "metadata": {},
     "output_type": "execute_result"
    }
   ],
   "source": [
    "df = conn.sql(\"\"\"\n",
    "    WITH hiring_posts AS (\n",
    "        SELECT \n",
    "            year,\n",
    "            month,\n",
    "            'Who is Hiring' as type,\n",
    "            SUM(descendants) as comments\n",
    "        FROM hn \n",
    "        WHERE title LIKE '%Who is hiring%'\n",
    "            AND type = 'story'\n",
    "            AND year >= 2013\n",
    "        GROUP BY year, month\n",
    "        \n",
    "        UNION ALL\n",
    "        \n",
    "        SELECT \n",
    "            year,\n",
    "            month,\n",
    "            'Who wants to be hired' as type,\n",
    "            SUM(descendants) as comments\n",
    "        FROM hn \n",
    "        WHERE title LIKE '%Who wants to be hired%'\n",
    "            AND type = 'story'\n",
    "            AND year >= 2013\n",
    "        GROUP BY year, month\n",
    "    )\n",
    "    SELECT \n",
    "        year,\n",
    "        month,\n",
    "        type,\n",
    "        comments,\n",
    "        year + month/12.0 as timeline\n",
    "    FROM hiring_posts\n",
    "    ORDER BY year, month, type\n",
    "\"\"\").df()\n",
    "\n",
    "chart = alt.Chart(df).mark_line(point=False).encode(\n",
    "    x=alt.X('timeline:Q', \n",
    "            title='Year',\n",
    "            axis=alt.Axis(format='d')),  # Use 'd' format to show integers without decimals\n",
    "    y=alt.Y('comments:Q', title='Comments per Thread'),\n",
    "    color=alt.Color('type:N', title='Thread Type'),\n",
    "    tooltip=[\n",
    "        alt.Tooltip('year:Q', title='Year'),\n",
    "        alt.Tooltip('month:Q', title='Month'),\n",
    "        alt.Tooltip('type:N', title='Type'),\n",
    "        alt.Tooltip('comments:Q', title='Comments', format='.1f')\n",
    "    ]\n",
    ").properties(\n",
    "    title='Monthly Hiring vs Seeking Work Thread Activity',\n",
    "    width=800,\n",
    "    height=400\n",
    ")\n",
    "chart\n"
   ]
  },
  {
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "Looking at the graph, we can see several interesting patterns:\n",
    "\n",
    "1. Overall trend: The \"Who is hiring\" threads consistently get more comments than \n",
    "   \"Who wants to be hired\" threads throughout the entire period.\n",
    "\n",
    "2. Peak activity: There was a notable spike in activity around 2020-2021, \n",
    "   particularly in the \"Who is hiring\" threads, possibly related to the tech \n",
    "   industry changes during the COVID-19 pandemic.\n",
    "\n",
    "3. Recent trends: After 2021, both thread types show a decline in activity, \n",
    "   with comment counts returning to levels similar to 2018-2019. The gap between\n",
    "   hiring and seeking work threads has also narrowed in recent years.\n",
    "\n",
    "4. Seasonality: There appears to be some seasonal variation in both thread types,\n",
    "   with regular ups and downs throughout each year, though the pattern is more\n",
    "   pronounced in the hiring threads."
   ]
  },
  {
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "## Top story of each year"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 7,
   "metadata": {},
   "outputs": [
    {
     "data": {
      "text/html": [
       "<table border=\"1\" class=\"dataframe\">\n",
       "  <thead>\n",
       "    <tr style=\"text-align: right;\">\n",
       "      <th>year</th>\n",
       "      <th>title</th>\n",
       "      <th>score</th>\n",
       "      <th>by</th>\n",
       "    </tr>\n",
       "  </thead>\n",
       "  <tbody>\n",
       "    <tr>\n",
       "      <td>2025</td>\n",
       "      <td><a href=\"https://news.ycombinator.com/item?id=42797260\" target=\"_blank\">Show HN: I made an open-source laptop from scratch</a></td>\n",
       "      <td>3237.0</td>\n",
       "      <td>Hello9999901</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <td>2024</td>\n",
       "      <td><a href=\"https://news.ycombinator.com/item?id=39865810\" target=\"_blank\">Backdoor in upstream xz/liblzma leading to SSH server compromise</a></td>\n",
       "      <td>4549.0</td>\n",
       "      <td>rkta</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <td>2023</td>\n",
       "      <td><a href=\"https://news.ycombinator.com/item?id=38309611\" target=\"_blank\">OpenAI's board has fired Sam Altman</a></td>\n",
       "      <td>5710.0</td>\n",
       "      <td>davidbarker</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <td>2022</td>\n",
       "      <td><a href=\"https://news.ycombinator.com/item?id=31261533\" target=\"_blank\">Mechanical Watch</a></td>\n",
       "      <td>4298.0</td>\n",
       "      <td>todsacerdoti</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <td>2021</td>\n",
       "      <td><a href=\"https://news.ycombinator.com/item?id=26699106\" target=\"_blank\">Google’s copying of the Java SE API was fair use [pdf]</a></td>\n",
       "      <td>4103.0</td>\n",
       "      <td>pdoconnell</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <td>2020</td>\n",
       "      <td><a href=\"https://news.ycombinator.com/item?id=24872911\" target=\"_blank\">YouTube-dl has received a DMCA takedown from RIAA</a></td>\n",
       "      <td>4240.0</td>\n",
       "      <td>phantop</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <td>2019</td>\n",
       "      <td><a href=\"https://news.ycombinator.com/item?id=20052623\" target=\"_blank\">Switch from Chrome to Firefox</a></td>\n",
       "      <td>3287.0</td>\n",
       "      <td>WisNorCan</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <td>2018</td>\n",
       "      <td><a href=\"https://news.ycombinator.com/item?id=16582136\" target=\"_blank\">Stephen Hawking has died</a></td>\n",
       "      <td>6015.0</td>\n",
       "      <td>Cogito</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <td>2017</td>\n",
       "      <td><a href=\"https://news.ycombinator.com/item?id=13682022\" target=\"_blank\">Reflecting on one very, very strange year at Uber</a></td>\n",
       "      <td>4107.0</td>\n",
       "      <td>grey-area</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <td>2016</td>\n",
       "      <td><a href=\"https://news.ycombinator.com/item?id=11116274\" target=\"_blank\">A Message to Our Customers</a></td>\n",
       "      <td>5771.0</td>\n",
       "      <td>epaga</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <td>2015</td>\n",
       "      <td><a href=\"https://news.ycombinator.com/item?id=10037157\" target=\"_blank\">G is for Google</a></td>\n",
       "      <td>2228.0</td>\n",
       "      <td>dkasper</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <td>2014</td>\n",
       "      <td><a href=\"https://news.ycombinator.com/item?id=8532261\" target=\"_blank\">Tim Cook Speaks Up</a></td>\n",
       "      <td>3086.0</td>\n",
       "      <td>replicatorblog</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <td>2013</td>\n",
       "      <td><a href=\"https://news.ycombinator.com/item?id=6258422\" target=\"_blank\">Don't Fly During Ramadan</a></td>\n",
       "      <td>2744.0</td>\n",
       "      <td>chimeracoder</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <td>2012</td>\n",
       "      <td><a href=\"https://news.ycombinator.com/item?id=3742902\" target=\"_blank\">Show HN: This up votes itself</a></td>\n",
       "      <td>3531.0</td>\n",
       "      <td>olalonde</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <td>2011</td>\n",
       "      <td><a href=\"https://news.ycombinator.com/item?id=3078128\" target=\"_blank\">Steve Jobs has passed away.</a></td>\n",
       "      <td>4338.0</td>\n",
       "      <td>patricktomas</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <td>2010</td>\n",
       "      <td><a href=\"https://news.ycombinator.com/item?id=2014555\" target=\"_blank\">Introducing Word Lens</a></td>\n",
       "      <td>1297.0</td>\n",
       "      <td>jf</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <td>2009</td>\n",
       "      <td><a href=\"https://news.ycombinator.com/item?id=639976\" target=\"_blank\">How I Hacked Hacker News (with arc security advisory)</a></td>\n",
       "      <td>928.0</td>\n",
       "      <td>dfranke</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <td>2008</td>\n",
       "      <td><a href=\"https://news.ycombinator.com/item?id=250704\" target=\"_blank\">Startup Ideas We'd Like to Fund</a></td>\n",
       "      <td>532.0</td>\n",
       "      <td>pg</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <td>2007</td>\n",
       "      <td><a href=\"https://news.ycombinator.com/item?id=363\" target=\"_blank\">Please tell us what features you'd like in news.ycombinator</a></td>\n",
       "      <td>262.0</td>\n",
       "      <td>pg</td>\n",
       "    </tr>\n",
       "  </tbody>\n",
       "</table>"
      ],
      "text/plain": [
       "<IPython.core.display.HTML object>"
      ]
     },
     "metadata": {},
     "output_type": "display_data"
    }
   ],
   "source": [
    "# Get top story by score for each year\n",
    "from IPython.display import HTML\n",
    "df = conn.sql(\"\"\"\n",
    "    SELECT \n",
    "        year,\n",
    "        CONCAT('<a href=\"https://news.ycombinator.com/item?id=', id, '\" target=\"_blank\">', title, '</a>') as title,\n",
    "        score,\n",
    "        by,\n",
    "    FROM hn\n",
    "    WHERE type = 'story'\n",
    "    AND score = (\n",
    "        SELECT MAX(score)\n",
    "        FROM hn AS h2 \n",
    "        WHERE h2.year = hn.year\n",
    "        AND h2.type = 'story'\n",
    "    )\n",
    "    ORDER BY year DESC\n",
    "\"\"\").df()\n",
    "\n",
    "pd.set_option('display.max_colwidth', None)\n",
    "display(HTML(df.to_html(escape=False, index=False)))"
   ]
  },
  {
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "## Top story of all time"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 8,
   "metadata": {
    "hide": true
   },
   "outputs": [
    {
     "data": {
      "text/html": [
       "<table border=\"1\" class=\"dataframe\">\n",
       "  <thead>\n",
       "    <tr style=\"text-align: right;\">\n",
       "      <th>title</th>\n",
       "      <th>score</th>\n",
       "      <th>by</th>\n",
       "    </tr>\n",
       "  </thead>\n",
       "  <tbody>\n",
       "    <tr>\n",
       "      <td><a href=\"https://news.ycombinator.com/item?id=16582136\" target=\"_blank\">Stephen Hawking has died</a></td>\n",
       "      <td>6015.0</td>\n",
       "      <td>Cogito</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <td><a href=\"https://news.ycombinator.com/item?id=11116274\" target=\"_blank\">A Message to Our Customers</a></td>\n",
       "      <td>5771.0</td>\n",
       "      <td>epaga</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <td><a href=\"https://news.ycombinator.com/item?id=38309611\" target=\"_blank\">OpenAI's board has fired Sam Altman</a></td>\n",
       "      <td>5710.0</td>\n",
       "      <td>davidbarker</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <td><a href=\"https://news.ycombinator.com/item?id=39865810\" target=\"_blank\">Backdoor in upstream xz/liblzma leading to SSH server compromise</a></td>\n",
       "      <td>4549.0</td>\n",
       "      <td>rkta</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <td><a href=\"https://news.ycombinator.com/item?id=41002195\" target=\"_blank\">CrowdStrike Update: Windows Bluescreen and Boot Loops</a></td>\n",
       "      <td>4405.0</td>\n",
       "      <td>BLKNSLVR</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <td><a href=\"https://news.ycombinator.com/item?id=3078128\" target=\"_blank\">Steve Jobs has passed away.</a></td>\n",
       "      <td>4338.0</td>\n",
       "      <td>patricktomas</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <td><a href=\"https://news.ycombinator.com/item?id=37011324\" target=\"_blank\">Bram Moolenaar has died</a></td>\n",
       "      <td>4310.0</td>\n",
       "      <td>wufocaculura</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <td><a href=\"https://news.ycombinator.com/item?id=31261533\" target=\"_blank\">Mechanical Watch</a></td>\n",
       "      <td>4298.0</td>\n",
       "      <td>todsacerdoti</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <td><a href=\"https://news.ycombinator.com/item?id=24872911\" target=\"_blank\">YouTube-dl has received a DMCA takedown from RIAA</a></td>\n",
       "      <td>4240.0</td>\n",
       "      <td>phantop</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <td><a href=\"https://news.ycombinator.com/item?id=13682022\" target=\"_blank\">Reflecting on one very, very strange year at Uber</a></td>\n",
       "      <td>4107.0</td>\n",
       "      <td>grey-area</td>\n",
       "    </tr>\n",
       "  </tbody>\n",
       "</table>"
      ],
      "text/plain": [
       "<IPython.core.display.HTML object>"
      ]
     },
     "metadata": {},
     "output_type": "display_data"
    }
   ],
   "source": [
    "df = conn.sql(\"\"\"\n",
    "    SELECT \n",
    "        CONCAT('<a href=\"https://news.ycombinator.com/item?id=', id, '\" target=\"_blank\">', title, '</a>') as title,\n",
    "        score,\n",
    "        by\n",
    "    FROM hn\n",
    "    WHERE type = 'story'\n",
    "    ORDER BY score DESC\n",
    "    LIMIT 10\n",
    "\"\"\").df()\n",
    "\n",
    "pd.set_option('display.max_colwidth', None)\n",
    "display(HTML(df.to_html(escape=False, index=False)))\n"
   ]
  },
  {
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "## Popular Show HN"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 9,
   "metadata": {
    "hide": true
   },
   "outputs": [
    {
     "data": {
      "text/html": [
       "<table border=\"1\" class=\"dataframe\">\n",
       "  <thead>\n",
       "    <tr style=\"text-align: right;\">\n",
       "      <th>title</th>\n",
       "      <th>score</th>\n",
       "      <th>by</th>\n",
       "    </tr>\n",
       "  </thead>\n",
       "  <tbody>\n",
       "    <tr>\n",
       "      <td><a href=\"https://news.ycombinator.com/item?id=3742902\" target=\"_blank\">Show HN: This up votes itself</a></td>\n",
       "      <td>3531.0</td>\n",
       "      <td>olalonde</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <td><a href=\"https://news.ycombinator.com/item?id=42797260\" target=\"_blank\">Show HN: I made an open-source laptop from scratch</a></td>\n",
       "      <td>3237.0</td>\n",
       "      <td>Hello9999901</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <td><a href=\"https://news.ycombinator.com/item?id=41247023\" target=\"_blank\">Show HN: If YouTube had actual channels</a></td>\n",
       "      <td>2741.0</td>\n",
       "      <td>hadisafa</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <td><a href=\"https://news.ycombinator.com/item?id=19393279\" target=\"_blank\">Show HN: A retro video game console I've been working on in my free time</a></td>\n",
       "      <td>2690.0</td>\n",
       "      <td>pkiller</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <td><a href=\"https://news.ycombinator.com/item?id=26271117\" target=\"_blank\">Show HN: Redbean – Single-file distributable web server</a></td>\n",
       "      <td>1998.0</td>\n",
       "      <td>jart</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <td><a href=\"https://news.ycombinator.com/item?id=36296695\" target=\"_blank\">Show HN: Non.io, a Reddit-like platform Ive been working on for the last 4 years</a></td>\n",
       "      <td>1943.0</td>\n",
       "      <td>jjcm</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <td><a href=\"https://news.ycombinator.com/item?id=32884467\" target=\"_blank\">Show HN: I may have created a new type of puzzle</a></td>\n",
       "      <td>1859.0</td>\n",
       "      <td>drcode</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <td><a href=\"https://news.ycombinator.com/item?id=33357197\" target=\"_blank\">Show HN: I 3D scanned the interior of the Great Pyramid at Giza</a></td>\n",
       "      <td>1752.0</td>\n",
       "      <td>lukehollis</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <td><a href=\"https://news.ycombinator.com/item?id=12166687\" target=\"_blank\">Show HN: Web Design in 4 minutes</a></td>\n",
       "      <td>1624.0</td>\n",
       "      <td>bbx</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <td><a href=\"https://news.ycombinator.com/item?id=24696658\" target=\"_blank\">Show HN: I made a site where you practice typing by retyping entire novels</a></td>\n",
       "      <td>1589.0</td>\n",
       "      <td>Octouroboros</td>\n",
       "    </tr>\n",
       "  </tbody>\n",
       "</table>"
      ],
      "text/plain": [
       "<IPython.core.display.HTML object>"
      ]
     },
     "metadata": {},
     "output_type": "display_data"
    }
   ],
   "source": [
    "df = conn.sql(\"\"\"\n",
    "    SELECT \n",
    "        CONCAT('<a href=\"https://news.ycombinator.com/item?id=', id, '\" target=\"_blank\">', title, '</a>') as title,\n",
    "        score,\n",
    "        by\n",
    "    FROM hn\n",
    "    WHERE type = 'story'\n",
    "    AND title LIKE 'Show HN:%'\n",
    "    ORDER BY score DESC\n",
    "    LIMIT 10\n",
    "\"\"\").df()\n",
    "\n",
    "pd.set_option('display.max_colwidth', None)\n",
    "display(HTML(df.to_html(escape=False, index=False)))"
   ]
  },
  {
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "## Popular Ask HN"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 10,
   "metadata": {
    "hide": true
   },
   "outputs": [
    {
     "data": {
      "text/html": [
       "<table border=\"1\" class=\"dataframe\">\n",
       "  <thead>\n",
       "    <tr style=\"text-align: right;\">\n",
       "      <th>title</th>\n",
       "      <th>score</th>\n",
       "      <th>by</th>\n",
       "    </tr>\n",
       "  </thead>\n",
       "  <tbody>\n",
       "    <tr>\n",
       "      <td><a href=\"https://news.ycombinator.com/item?id=37392676\" target=\"_blank\">Ask HN: I’m an FCC Commissioner proposing regulation of IoT security updates</a></td>\n",
       "      <td>3387.0</td>\n",
       "      <td>SimingtonFCC</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <td><a href=\"https://news.ycombinator.com/item?id=22918980\" target=\"_blank\">Ask HN: I'm a software engineer going blind, how should I prepare?</a></td>\n",
       "      <td>3270.0</td>\n",
       "      <td>zachrip</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <td><a href=\"https://news.ycombinator.com/item?id=13755673\" target=\"_blank\">Ask HN: Is S3 down?</a></td>\n",
       "      <td>2589.0</td>\n",
       "      <td>iamdeedubs</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <td><a href=\"https://news.ycombinator.com/item?id=32186203\" target=\"_blank\">Ask HN: What are some cool but obscure data structures you know about?</a></td>\n",
       "      <td>2051.0</td>\n",
       "      <td>Uptrenda</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <td><a href=\"https://news.ycombinator.com/item?id=26721951\" target=\"_blank\">Ask HN: What tech job would let me get away with the least real work possible?</a></td>\n",
       "      <td>2022.0</td>\n",
       "      <td>lmueongoqx</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <td><a href=\"https://news.ycombinator.com/item?id=19087418\" target=\"_blank\">Ask HN: What books changed the way you think about almost everything?</a></td>\n",
       "      <td>2009.0</td>\n",
       "      <td>apitman</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <td><a href=\"https://news.ycombinator.com/item?id=35729232\" target=\"_blank\">Ask HN: Most interesting tech you built for just yourself?</a></td>\n",
       "      <td>1901.0</td>\n",
       "      <td>l2silver</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <td><a href=\"https://news.ycombinator.com/item?id=20264911\" target=\"_blank\">Ask HN: What do you do with your Raspberry Pi?</a></td>\n",
       "      <td>1741.0</td>\n",
       "      <td>xylo</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <td><a href=\"https://news.ycombinator.com/item?id=16775744\" target=\"_blank\">Ask HN: How to self-learn electronics?</a></td>\n",
       "      <td>1681.0</td>\n",
       "      <td>sidyapa</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <td><a href=\"https://news.ycombinator.com/item?id=16745042\" target=\"_blank\">Ask HN: What are the best MOOCs you've taken?</a></td>\n",
       "      <td>1665.0</td>\n",
       "      <td>csdrane</td>\n",
       "    </tr>\n",
       "  </tbody>\n",
       "</table>"
      ],
      "text/plain": [
       "<IPython.core.display.HTML object>"
      ]
     },
     "metadata": {},
     "output_type": "display_data"
    }
   ],
   "source": [
    "df = conn.sql(\"\"\"\n",
    "    SELECT \n",
    "        CONCAT('<a href=\"https://news.ycombinator.com/item?id=', id, '\" target=\"_blank\">', title, '</a>') as title,\n",
    "        score,\n",
    "        by\n",
    "    FROM hn\n",
    "    WHERE type = 'story'\n",
    "    AND title LIKE 'Ask HN:%'\n",
    "    ORDER BY score DESC\n",
    "    LIMIT 10\n",
    "\"\"\").df()\n",
    "\n",
    "pd.set_option('display.max_colwidth', None)\n",
    "display(HTML(df.to_html(escape=False, index=False)))\n"
   ]
  },
  {
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "### Most submitted domain"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 11,
   "metadata": {},
   "outputs": [
    {
     "data": {
      "text/html": [
       "\n",
       "<style>\n",
       "  #altair-viz-af3a99030f8d41fb9903a29778417770.vega-embed {\n",
       "    width: 100%;\n",
       "    display: flex;\n",
       "  }\n",
       "\n",
       "  #altair-viz-af3a99030f8d41fb9903a29778417770.vega-embed details,\n",
       "  #altair-viz-af3a99030f8d41fb9903a29778417770.vega-embed details summary {\n",
       "    position: relative;\n",
       "  }\n",
       "</style>\n",
       "<div id=\"altair-viz-af3a99030f8d41fb9903a29778417770\"></div>\n",
       "<script type=\"text/javascript\">\n",
       "  var VEGA_DEBUG = (typeof VEGA_DEBUG == \"undefined\") ? {} : VEGA_DEBUG;\n",
       "  (function(spec, embedOpt){\n",
       "    let outputDiv = document.currentScript.previousElementSibling;\n",
       "    if (outputDiv.id !== \"altair-viz-af3a99030f8d41fb9903a29778417770\") {\n",
       "      outputDiv = document.getElementById(\"altair-viz-af3a99030f8d41fb9903a29778417770\");\n",
       "    }\n",
       "\n",
       "    const paths = {\n",
       "      \"vega\": \"https://cdn.jsdelivr.net/npm/vega@5?noext\",\n",
       "      \"vega-lib\": \"https://cdn.jsdelivr.net/npm/vega-lib?noext\",\n",
       "      \"vega-lite\": \"https://cdn.jsdelivr.net/npm/vega-lite@5.20.1?noext\",\n",
       "      \"vega-embed\": \"https://cdn.jsdelivr.net/npm/vega-embed@6?noext\",\n",
       "    };\n",
       "\n",
       "    function maybeLoadScript(lib, version) {\n",
       "      var key = `${lib.replace(\"-\", \"\")}_version`;\n",
       "      return (VEGA_DEBUG[key] == version) ?\n",
       "        Promise.resolve(paths[lib]) :\n",
       "        new Promise(function(resolve, reject) {\n",
       "          var s = document.createElement('script');\n",
       "          document.getElementsByTagName(\"head\")[0].appendChild(s);\n",
       "          s.async = true;\n",
       "          s.onload = () => {\n",
       "            VEGA_DEBUG[key] = version;\n",
       "            return resolve(paths[lib]);\n",
       "          };\n",
       "          s.onerror = () => reject(`Error loading script: ${paths[lib]}`);\n",
       "          s.src = paths[lib];\n",
       "        });\n",
       "    }\n",
       "\n",
       "    function showError(err) {\n",
       "      outputDiv.innerHTML = `<div class=\"error\" style=\"color:red;\">${err}</div>`;\n",
       "      throw err;\n",
       "    }\n",
       "\n",
       "    function displayChart(vegaEmbed) {\n",
       "      vegaEmbed(outputDiv, spec, embedOpt)\n",
       "        .catch(err => showError(`Javascript Error: ${err.message}<br>This usually means there's a typo in your chart specification. See the javascript console for the full traceback.`));\n",
       "    }\n",
       "\n",
       "    if(typeof define === \"function\" && define.amd) {\n",
       "      requirejs.config({paths});\n",
       "      let deps = [\"vega-embed\"];\n",
       "      require(deps, displayChart, err => showError(`Error loading script: ${err.message}`));\n",
       "    } else {\n",
       "      maybeLoadScript(\"vega\", \"5\")\n",
       "        .then(() => maybeLoadScript(\"vega-lite\", \"5.20.1\"))\n",
       "        .then(() => maybeLoadScript(\"vega-embed\", \"6\"))\n",
       "        .catch(showError)\n",
       "        .then(() => displayChart(vegaEmbed));\n",
       "    }\n",
       "  })({\"config\": {\"view\": {\"continuousWidth\": 300, \"continuousHeight\": 300}}, \"data\": {\"name\": \"data-56e92a7431d40126efb3eee1cedd11e4\"}, \"mark\": {\"type\": \"bar\"}, \"encoding\": {\"tooltip\": [{\"field\": \"domain\", \"type\": \"nominal\"}, {\"field\": \"count\", \"type\": \"quantitative\"}], \"x\": {\"field\": \"count\", \"title\": \"Number of submissions\", \"type\": \"quantitative\"}, \"y\": {\"field\": \"domain\", \"sort\": \"-x\", \"title\": \"Domain\", \"type\": \"nominal\"}}, \"height\": 400, \"title\": \"Most Submitted Domains on HN\", \"width\": 800, \"$schema\": \"https://vega.github.io/schema/vega-lite/v5.20.1.json\", \"datasets\": {\"data-56e92a7431d40126efb3eee1cedd11e4\": [{\"domain\": \"https://github.com\", \"count\": 160992}, {\"domain\": \"https://medium.com\", \"count\": 117772}, {\"domain\": \"https://www.youtube.com\", \"count\": 102706}, {\"domain\": \"https://twitter.com\", \"count\": 43899}, {\"domain\": \"https://www.nytimes.com\", \"count\": 42888}, {\"domain\": \"https://en.wikipedia.org\", \"count\": 42651}, {\"domain\": \"https://www.theguardian.com\", \"count\": 31930}, {\"domain\": \"http://www.nytimes.com\", \"count\": 27391}, {\"domain\": \"https://www.bloomberg.com\", \"count\": 26964}, {\"domain\": \"http://techcrunch.com\", \"count\": 26242}, {\"domain\": \"https://arstechnica.com\", \"count\": 25716}, {\"domain\": \"https://techcrunch.com\", \"count\": 24375}, {\"domain\": \"https://www.theverge.com\", \"count\": 21747}, {\"domain\": \"https://www.wsj.com\", \"count\": 20942}, {\"domain\": \"http://www.youtube.com\", \"count\": 17097}, {\"domain\": \"http://arstechnica.com\", \"count\": 17003}, {\"domain\": \"https://arxiv.org\", \"count\": 15622}, {\"domain\": \"https://www.washingtonpost.com\", \"count\": 14942}, {\"domain\": \"https://www.bbc.com\", \"count\": 14876}, {\"domain\": \"https://www.reuters.com\", \"count\": 14546}]}}, {\"mode\": \"vega-lite\"});\n",
       "</script>"
      ],
      "text/plain": [
       "alt.Chart(...)"
      ]
     },
     "execution_count": 11,
     "metadata": {},
     "output_type": "execute_result"
    }
   ],
   "source": [
    "df = conn.sql(\"\"\"\n",
    "    SELECT \n",
    "        REGEXP_EXTRACT(url, 'https?://([^/]+)') as domain,\n",
    "        COUNT(*) as count\n",
    "    FROM hn\n",
    "    WHERE type = 'story'\n",
    "    AND url IS NOT NULL \n",
    "    AND url != ''\n",
    "    GROUP BY domain\n",
    "    ORDER BY count DESC\n",
    "    LIMIT 20\n",
    "\"\"\").df()\n",
    "\n",
    "chart = alt.Chart(df).mark_bar().encode(\n",
    "    x=alt.X('count:Q', title='Number of submissions'),\n",
    "    y=alt.Y('domain:N', sort='-x', title='Domain'),\n",
    "    tooltip=['domain', 'count']\n",
    ").properties(\n",
    "    title='Most Submitted Domains on HN',\n",
    "    width=800,\n",
    "    height=400\n",
    ")\n",
    "\n",
    "chart\n"
   ]
  },
  {
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "### Most upvoted domain"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 12,
   "metadata": {},
   "outputs": [
    {
     "data": {
      "text/html": [
       "\n",
       "<style>\n",
       "  #altair-viz-73698a4fc8b04582b67cd4da13ca758d.vega-embed {\n",
       "    width: 100%;\n",
       "    display: flex;\n",
       "  }\n",
       "\n",
       "  #altair-viz-73698a4fc8b04582b67cd4da13ca758d.vega-embed details,\n",
       "  #altair-viz-73698a4fc8b04582b67cd4da13ca758d.vega-embed details summary {\n",
       "    position: relative;\n",
       "  }\n",
       "</style>\n",
       "<div id=\"altair-viz-73698a4fc8b04582b67cd4da13ca758d\"></div>\n",
       "<script type=\"text/javascript\">\n",
       "  var VEGA_DEBUG = (typeof VEGA_DEBUG == \"undefined\") ? {} : VEGA_DEBUG;\n",
       "  (function(spec, embedOpt){\n",
       "    let outputDiv = document.currentScript.previousElementSibling;\n",
       "    if (outputDiv.id !== \"altair-viz-73698a4fc8b04582b67cd4da13ca758d\") {\n",
       "      outputDiv = document.getElementById(\"altair-viz-73698a4fc8b04582b67cd4da13ca758d\");\n",
       "    }\n",
       "\n",
       "    const paths = {\n",
       "      \"vega\": \"https://cdn.jsdelivr.net/npm/vega@5?noext\",\n",
       "      \"vega-lib\": \"https://cdn.jsdelivr.net/npm/vega-lib?noext\",\n",
       "      \"vega-lite\": \"https://cdn.jsdelivr.net/npm/vega-lite@5.20.1?noext\",\n",
       "      \"vega-embed\": \"https://cdn.jsdelivr.net/npm/vega-embed@6?noext\",\n",
       "    };\n",
       "\n",
       "    function maybeLoadScript(lib, version) {\n",
       "      var key = `${lib.replace(\"-\", \"\")}_version`;\n",
       "      return (VEGA_DEBUG[key] == version) ?\n",
       "        Promise.resolve(paths[lib]) :\n",
       "        new Promise(function(resolve, reject) {\n",
       "          var s = document.createElement('script');\n",
       "          document.getElementsByTagName(\"head\")[0].appendChild(s);\n",
       "          s.async = true;\n",
       "          s.onload = () => {\n",
       "            VEGA_DEBUG[key] = version;\n",
       "            return resolve(paths[lib]);\n",
       "          };\n",
       "          s.onerror = () => reject(`Error loading script: ${paths[lib]}`);\n",
       "          s.src = paths[lib];\n",
       "        });\n",
       "    }\n",
       "\n",
       "    function showError(err) {\n",
       "      outputDiv.innerHTML = `<div class=\"error\" style=\"color:red;\">${err}</div>`;\n",
       "      throw err;\n",
       "    }\n",
       "\n",
       "    function displayChart(vegaEmbed) {\n",
       "      vegaEmbed(outputDiv, spec, embedOpt)\n",
       "        .catch(err => showError(`Javascript Error: ${err.message}<br>This usually means there's a typo in your chart specification. See the javascript console for the full traceback.`));\n",
       "    }\n",
       "\n",
       "    if(typeof define === \"function\" && define.amd) {\n",
       "      requirejs.config({paths});\n",
       "      let deps = [\"vega-embed\"];\n",
       "      require(deps, displayChart, err => showError(`Error loading script: ${err.message}`));\n",
       "    } else {\n",
       "      maybeLoadScript(\"vega\", \"5\")\n",
       "        .then(() => maybeLoadScript(\"vega-lite\", \"5.20.1\"))\n",
       "        .then(() => maybeLoadScript(\"vega-embed\", \"6\"))\n",
       "        .catch(showError)\n",
       "        .then(() => displayChart(vegaEmbed));\n",
       "    }\n",
       "  })({\"config\": {\"view\": {\"continuousWidth\": 300, \"continuousHeight\": 300}}, \"data\": {\"name\": \"data-d037c1b185a03a54fb0576b9629aea55\"}, \"mark\": {\"type\": \"bar\"}, \"encoding\": {\"tooltip\": [{\"field\": \"domain\", \"type\": \"nominal\"}, {\"field\": \"total_score\", \"type\": \"quantitative\"}, {\"field\": \"count\", \"type\": \"quantitative\"}, {\"field\": \"avg_score\", \"type\": \"quantitative\"}], \"x\": {\"field\": \"total_score\", \"title\": \"Total score\", \"type\": \"quantitative\"}, \"y\": {\"field\": \"domain\", \"sort\": \"-x\", \"title\": \"Domain\", \"type\": \"nominal\"}}, \"height\": 400, \"title\": \"Most Upvoted Domains on HN\", \"width\": 800, \"$schema\": \"https://vega.github.io/schema/vega-lite/v5.20.1.json\", \"datasets\": {\"data-d037c1b185a03a54fb0576b9629aea55\": [{\"domain\": \"https://github.com\", \"count\": 160992, \"total_score\": 3200771.0, \"avg_score\": 19.881553120651958}, {\"domain\": \"https://twitter.com\", \"count\": 43899, \"total_score\": 983532.0, \"avg_score\": 22.404428346887173}, {\"domain\": \"https://medium.com\", \"count\": 117772, \"total_score\": 859858.0, \"avg_score\": 7.301039296267364}, {\"domain\": \"https://www.nytimes.com\", \"count\": 42888, \"total_score\": 772937.0, \"avg_score\": 18.02222066778586}, {\"domain\": \"https://www.youtube.com\", \"count\": 102706, \"total_score\": 598459.0, \"avg_score\": 5.82691371487547}, {\"domain\": \"https://www.bloomberg.com\", \"count\": 26964, \"total_score\": 584030.0, \"avg_score\": 21.65962023438659}, {\"domain\": \"https://www.theguardian.com\", \"count\": 31930, \"total_score\": 522731.0, \"avg_score\": 16.371155652990918}, {\"domain\": \"https://techcrunch.com\", \"count\": 24375, \"total_score\": 474123.0, \"avg_score\": 19.4512}, {\"domain\": \"https://arstechnica.com\", \"count\": 25716, \"total_score\": 468622.0, \"avg_score\": 18.22297402395396}, {\"domain\": \"http://www.nytimes.com\", \"count\": 27391, \"total_score\": 465839.0, \"avg_score\": 17.007009601693987}, {\"domain\": \"https://www.reuters.com\", \"count\": 14546, \"total_score\": 391612.0, \"avg_score\": 26.922315413172004}, {\"domain\": \"http://techcrunch.com\", \"count\": 26242, \"total_score\": 379619.0, \"avg_score\": 14.466084902065392}, {\"domain\": \"https://en.wikipedia.org\", \"count\": 42651, \"total_score\": 356323.0, \"avg_score\": 8.35438793932147}, {\"domain\": \"https://www.theverge.com\", \"count\": 21747, \"total_score\": 346848.0, \"avg_score\": 15.94923437715547}, {\"domain\": \"https://www.wsj.com\", \"count\": 20942, \"total_score\": 324078.0, \"avg_score\": 15.47502626301213}, {\"domain\": \"https://www.bbc.com\", \"count\": 14876, \"total_score\": 303264.0, \"avg_score\": 20.386125302500673}, {\"domain\": \"https://www.washingtonpost.com\", \"count\": 14942, \"total_score\": 280432.0, \"avg_score\": 18.76803640744211}, {\"domain\": \"https://arxiv.org\", \"count\": 15622, \"total_score\": 250808.0, \"avg_score\": 16.054794520547944}, {\"domain\": \"http://arstechnica.com\", \"count\": 17003, \"total_score\": 242908.0, \"avg_score\": 14.286184790919249}, {\"domain\": \"https://www.nature.com\", \"count\": 11744, \"total_score\": 209446.0, \"avg_score\": 17.834298365122617}]}}, {\"mode\": \"vega-lite\"});\n",
       "</script>"
      ],
      "text/plain": [
       "alt.Chart(...)"
      ]
     },
     "execution_count": 12,
     "metadata": {},
     "output_type": "execute_result"
    }
   ],
   "source": [
    "df = conn.sql(\"\"\"\n",
    "    SELECT \n",
    "        REGEXP_EXTRACT(url, 'https?://([^/]+)') as domain,\n",
    "        COUNT(*) as count,\n",
    "        SUM(score) as total_score,\n",
    "        AVG(score) as avg_score\n",
    "    FROM hn\n",
    "    WHERE type = 'story'\n",
    "    AND url IS NOT NULL \n",
    "    AND url != ''\n",
    "    GROUP BY domain\n",
    "    ORDER BY total_score DESC\n",
    "    LIMIT 20\n",
    "\"\"\").df()\n",
    "\n",
    "chart = alt.Chart(df).mark_bar().encode(\n",
    "    x=alt.X('total_score:Q', title='Total score'),\n",
    "    y=alt.Y('domain:N', sort='-x', title='Domain'),\n",
    "    tooltip=['domain', 'total_score', 'count', 'avg_score']\n",
    ").properties(\n",
    "    title='Most Upvoted Domains on HN',\n",
    "    width=800,\n",
    "    height=400\n",
    ")\n",
    "\n",
    "chart\n"
   ]
  },
  {
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "### High quality submission\n",
    "\n",
    "But then, what is usually sources that get most upvoted often vs people just spamming it?"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 13,
   "metadata": {},
   "outputs": [
    {
     "data": {
      "text/html": [
       "\n",
       "<style>\n",
       "  #altair-viz-c9f922caf32b4969be40d05b6a2870a3.vega-embed {\n",
       "    width: 100%;\n",
       "    display: flex;\n",
       "  }\n",
       "\n",
       "  #altair-viz-c9f922caf32b4969be40d05b6a2870a3.vega-embed details,\n",
       "  #altair-viz-c9f922caf32b4969be40d05b6a2870a3.vega-embed details summary {\n",
       "    position: relative;\n",
       "  }\n",
       "</style>\n",
       "<div id=\"altair-viz-c9f922caf32b4969be40d05b6a2870a3\"></div>\n",
       "<script type=\"text/javascript\">\n",
       "  var VEGA_DEBUG = (typeof VEGA_DEBUG == \"undefined\") ? {} : VEGA_DEBUG;\n",
       "  (function(spec, embedOpt){\n",
       "    let outputDiv = document.currentScript.previousElementSibling;\n",
       "    if (outputDiv.id !== \"altair-viz-c9f922caf32b4969be40d05b6a2870a3\") {\n",
       "      outputDiv = document.getElementById(\"altair-viz-c9f922caf32b4969be40d05b6a2870a3\");\n",
       "    }\n",
       "\n",
       "    const paths = {\n",
       "      \"vega\": \"https://cdn.jsdelivr.net/npm/vega@5?noext\",\n",
       "      \"vega-lib\": \"https://cdn.jsdelivr.net/npm/vega-lib?noext\",\n",
       "      \"vega-lite\": \"https://cdn.jsdelivr.net/npm/vega-lite@5.20.1?noext\",\n",
       "      \"vega-embed\": \"https://cdn.jsdelivr.net/npm/vega-embed@6?noext\",\n",
       "    };\n",
       "\n",
       "    function maybeLoadScript(lib, version) {\n",
       "      var key = `${lib.replace(\"-\", \"\")}_version`;\n",
       "      return (VEGA_DEBUG[key] == version) ?\n",
       "        Promise.resolve(paths[lib]) :\n",
       "        new Promise(function(resolve, reject) {\n",
       "          var s = document.createElement('script');\n",
       "          document.getElementsByTagName(\"head\")[0].appendChild(s);\n",
       "          s.async = true;\n",
       "          s.onload = () => {\n",
       "            VEGA_DEBUG[key] = version;\n",
       "            return resolve(paths[lib]);\n",
       "          };\n",
       "          s.onerror = () => reject(`Error loading script: ${paths[lib]}`);\n",
       "          s.src = paths[lib];\n",
       "        });\n",
       "    }\n",
       "\n",
       "    function showError(err) {\n",
       "      outputDiv.innerHTML = `<div class=\"error\" style=\"color:red;\">${err}</div>`;\n",
       "      throw err;\n",
       "    }\n",
       "\n",
       "    function displayChart(vegaEmbed) {\n",
       "      vegaEmbed(outputDiv, spec, embedOpt)\n",
       "        .catch(err => showError(`Javascript Error: ${err.message}<br>This usually means there's a typo in your chart specification. See the javascript console for the full traceback.`));\n",
       "    }\n",
       "\n",
       "    if(typeof define === \"function\" && define.amd) {\n",
       "      requirejs.config({paths});\n",
       "      let deps = [\"vega-embed\"];\n",
       "      require(deps, displayChart, err => showError(`Error loading script: ${err.message}`));\n",
       "    } else {\n",
       "      maybeLoadScript(\"vega\", \"5\")\n",
       "        .then(() => maybeLoadScript(\"vega-lite\", \"5.20.1\"))\n",
       "        .then(() => maybeLoadScript(\"vega-embed\", \"6\"))\n",
       "        .catch(showError)\n",
       "        .then(() => displayChart(vegaEmbed));\n",
       "    }\n",
       "  })({\"config\": {\"view\": {\"continuousWidth\": 300, \"continuousHeight\": 300}}, \"data\": {\"name\": \"data-9e0c737ac45982668ceb319f1488a88e\"}, \"mark\": {\"type\": \"bar\"}, \"encoding\": {\"tooltip\": [{\"field\": \"domain\", \"type\": \"nominal\"}, {\"field\": \"total_score\", \"type\": \"quantitative\"}, {\"field\": \"count\", \"type\": \"quantitative\"}, {\"field\": \"avg_score\", \"type\": \"quantitative\"}, {\"field\": \"score_per_submission\", \"type\": \"quantitative\"}], \"x\": {\"field\": \"score_per_submission\", \"title\": \"Average Score per Submission\", \"type\": \"quantitative\"}, \"y\": {\"field\": \"domain\", \"sort\": \"-x\", \"title\": \"Domain\", \"type\": \"nominal\"}}, \"height\": 400, \"title\": \"Domains with Highest Average Score per Submission (min 10 submissions)\", \"width\": 800, \"$schema\": \"https://vega.github.io/schema/vega-lite/v5.20.1.json\", \"datasets\": {\"data-9e0c737ac45982668ceb319f1488a88e\": [{\"domain\": \"https://www.susanjfowler.com\", \"count\": 10, \"total_score\": 4781.0, \"avg_score\": 478.1, \"score_per_submission\": 478.1}, {\"domain\": \"https://ciechanow.ski\", \"count\": 68, \"total_score\": 28649.0, \"avg_score\": 421.30882352941177, \"score_per_submission\": 421.30882352941177}, {\"domain\": \"https://intuitiveexplanations.com\", \"count\": 12, \"total_score\": 4957.0, \"avg_score\": 413.0833333333333, \"score_per_submission\": 413.0833333333333}, {\"domain\": \"https://dkb.io\", \"count\": 12, \"total_score\": 4532.0, \"avg_score\": 377.6666666666667, \"score_per_submission\": 377.6666666666667}, {\"domain\": \"https://www.phind.com\", \"count\": 13, \"total_score\": 4832.0, \"avg_score\": 371.6923076923077, \"score_per_submission\": 371.6923076923077}, {\"domain\": \"https://learningmusic.ableton.com\", \"count\": 12, \"total_score\": 4422.0, \"avg_score\": 368.5, \"score_per_submission\": 368.5}, {\"domain\": \"https://bugs.xdavidhu.me\", \"count\": 11, \"total_score\": 3901.0, \"avg_score\": 354.6363636363636, \"score_per_submission\": 354.6363636363636}, {\"domain\": \"http://blog.rust-lang.org\", \"count\": 34, \"total_score\": 11646.0, \"avg_score\": 342.52941176470586, \"score_per_submission\": 342.52941176470586}, {\"domain\": \"http://blog.chriszacharias.com\", \"count\": 11, \"total_score\": 3727.0, \"avg_score\": 338.8181818181818, \"score_per_submission\": 338.8181818181818}, {\"domain\": \"https://oimo.io\", \"count\": 25, \"total_score\": 7867.0, \"avg_score\": 314.68, \"score_per_submission\": 314.68}, {\"domain\": \"http://immersivemath.com\", \"count\": 12, \"total_score\": 3630.0, \"avg_score\": 302.5, \"score_per_submission\": 302.5}, {\"domain\": \"https://www.moore.org\", \"count\": 11, \"total_score\": 3019.0, \"avg_score\": 274.45454545454544, \"score_per_submission\": 274.45454545454544}, {\"domain\": \"https://tinyprojects.dev\", \"count\": 20, \"total_score\": 5307.0, \"avg_score\": 265.35, \"score_per_submission\": 265.35}, {\"domain\": \"https://tauri.studio\", \"count\": 10, \"total_score\": 2642.0, \"avg_score\": 264.2, \"score_per_submission\": 264.2}, {\"domain\": \"https://casparwre.de\", \"count\": 11, \"total_score\": 2862.0, \"avg_score\": 260.1818181818182, \"score_per_submission\": 260.1818181818182}, {\"domain\": \"https://mattferraro.dev\", \"count\": 11, \"total_score\": 2859.0, \"avg_score\": 259.90909090909093, \"score_per_submission\": 259.90909090909093}, {\"domain\": \"https://www.sofuckingagile.com\", \"count\": 10, \"total_score\": 2591.0, \"avg_score\": 259.1, \"score_per_submission\": 259.1}, {\"domain\": \"https://asahilinux.org\", \"count\": 28, \"total_score\": 7010.0, \"avg_score\": 250.35714285714286, \"score_per_submission\": 250.35714285714286}, {\"domain\": \"https://omar.website\", \"count\": 10, \"total_score\": 2493.0, \"avg_score\": 249.3, \"score_per_submission\": 249.3}, {\"domain\": \"https://www.tablix.org\", \"count\": 10, \"total_score\": 2492.0, \"avg_score\": 249.2, \"score_per_submission\": 249.2}]}}, {\"mode\": \"vega-lite\"});\n",
       "</script>"
      ],
      "text/plain": [
       "alt.Chart(...)"
      ]
     },
     "execution_count": 13,
     "metadata": {},
     "output_type": "execute_result"
    }
   ],
   "source": [
    "df = conn.sql(\"\"\"\n",
    "    SELECT \n",
    "        REGEXP_EXTRACT(url, 'https?://([^/]+)') as domain,\n",
    "        COUNT(*) as count,\n",
    "        SUM(score) as total_score,\n",
    "        AVG(score) as avg_score,\n",
    "        total_score / COUNT(*) as score_per_submission\n",
    "    FROM hn\n",
    "    WHERE type = 'story'\n",
    "    AND url IS NOT NULL \n",
    "    AND url != ''\n",
    "    AND score > 0\n",
    "    GROUP BY domain\n",
    "    HAVING count >= 10\n",
    "    ORDER BY score_per_submission DESC\n",
    "    LIMIT 20\n",
    "\"\"\").df()\n",
    "\n",
    "chart = alt.Chart(df).mark_bar().encode(\n",
    "    x=alt.X('score_per_submission:Q', title='Average Score per Submission'),\n",
    "    y=alt.Y('domain:N', sort='-x', title='Domain'),\n",
    "    tooltip=['domain', 'total_score', 'count', 'avg_score', 'score_per_submission']\n",
    ").properties(\n",
    "    title='Domains with Highest Average Score per Submission (min 10 submissions)',\n",
    "    width=800,\n",
    "    height=400\n",
    ")\n",
    "\n",
    "chart\n"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 14,
   "metadata": {},
   "outputs": [
    {
     "data": {
      "text/html": [
       "<table border=\"1\" class=\"dataframe\">\n",
       "  <thead>\n",
       "    <tr style=\"text-align: right;\">\n",
       "      <th>domain</th>\n",
       "      <th>count</th>\n",
       "      <th>total_score</th>\n",
       "      <th>avg_score</th>\n",
       "      <th>score_per_submission</th>\n",
       "    </tr>\n",
       "  </thead>\n",
       "  <tbody>\n",
       "    <tr>\n",
       "      <td><a href=\"https://www.susanjfowler.com\" target=\"_blank\">https://www.susanjfowler.com</a></td>\n",
       "      <td>10</td>\n",
       "      <td>4781.0</td>\n",
       "      <td>478.100000</td>\n",
       "      <td>478.100000</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <td><a href=\"https://ciechanow.ski\" target=\"_blank\">https://ciechanow.ski</a></td>\n",
       "      <td>68</td>\n",
       "      <td>28649.0</td>\n",
       "      <td>421.308824</td>\n",
       "      <td>421.308824</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <td><a href=\"https://intuitiveexplanations.com\" target=\"_blank\">https://intuitiveexplanations.com</a></td>\n",
       "      <td>12</td>\n",
       "      <td>4957.0</td>\n",
       "      <td>413.083333</td>\n",
       "      <td>413.083333</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <td><a href=\"https://dkb.io\" target=\"_blank\">https://dkb.io</a></td>\n",
       "      <td>12</td>\n",
       "      <td>4532.0</td>\n",
       "      <td>377.666667</td>\n",
       "      <td>377.666667</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <td><a href=\"https://www.phind.com\" target=\"_blank\">https://www.phind.com</a></td>\n",
       "      <td>13</td>\n",
       "      <td>4832.0</td>\n",
       "      <td>371.692308</td>\n",
       "      <td>371.692308</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <td><a href=\"https://learningmusic.ableton.com\" target=\"_blank\">https://learningmusic.ableton.com</a></td>\n",
       "      <td>12</td>\n",
       "      <td>4422.0</td>\n",
       "      <td>368.500000</td>\n",
       "      <td>368.500000</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <td><a href=\"https://bugs.xdavidhu.me\" target=\"_blank\">https://bugs.xdavidhu.me</a></td>\n",
       "      <td>11</td>\n",
       "      <td>3901.0</td>\n",
       "      <td>354.636364</td>\n",
       "      <td>354.636364</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <td><a href=\"http://blog.rust-lang.org\" target=\"_blank\">http://blog.rust-lang.org</a></td>\n",
       "      <td>34</td>\n",
       "      <td>11646.0</td>\n",
       "      <td>342.529412</td>\n",
       "      <td>342.529412</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <td><a href=\"http://blog.chriszacharias.com\" target=\"_blank\">http://blog.chriszacharias.com</a></td>\n",
       "      <td>11</td>\n",
       "      <td>3727.0</td>\n",
       "      <td>338.818182</td>\n",
       "      <td>338.818182</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <td><a href=\"https://oimo.io\" target=\"_blank\">https://oimo.io</a></td>\n",
       "      <td>25</td>\n",
       "      <td>7867.0</td>\n",
       "      <td>314.680000</td>\n",
       "      <td>314.680000</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <td><a href=\"http://immersivemath.com\" target=\"_blank\">http://immersivemath.com</a></td>\n",
       "      <td>12</td>\n",
       "      <td>3630.0</td>\n",
       "      <td>302.500000</td>\n",
       "      <td>302.500000</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <td><a href=\"https://www.moore.org\" target=\"_blank\">https://www.moore.org</a></td>\n",
       "      <td>11</td>\n",
       "      <td>3019.0</td>\n",
       "      <td>274.454545</td>\n",
       "      <td>274.454545</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <td><a href=\"https://tinyprojects.dev\" target=\"_blank\">https://tinyprojects.dev</a></td>\n",
       "      <td>20</td>\n",
       "      <td>5307.0</td>\n",
       "      <td>265.350000</td>\n",
       "      <td>265.350000</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <td><a href=\"https://tauri.studio\" target=\"_blank\">https://tauri.studio</a></td>\n",
       "      <td>10</td>\n",
       "      <td>2642.0</td>\n",
       "      <td>264.200000</td>\n",
       "      <td>264.200000</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <td><a href=\"https://casparwre.de\" target=\"_blank\">https://casparwre.de</a></td>\n",
       "      <td>11</td>\n",
       "      <td>2862.0</td>\n",
       "      <td>260.181818</td>\n",
       "      <td>260.181818</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <td><a href=\"https://mattferraro.dev\" target=\"_blank\">https://mattferraro.dev</a></td>\n",
       "      <td>11</td>\n",
       "      <td>2859.0</td>\n",
       "      <td>259.909091</td>\n",
       "      <td>259.909091</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <td><a href=\"https://www.sofuckingagile.com\" target=\"_blank\">https://www.sofuckingagile.com</a></td>\n",
       "      <td>10</td>\n",
       "      <td>2591.0</td>\n",
       "      <td>259.100000</td>\n",
       "      <td>259.100000</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <td><a href=\"https://asahilinux.org\" target=\"_blank\">https://asahilinux.org</a></td>\n",
       "      <td>28</td>\n",
       "      <td>7010.0</td>\n",
       "      <td>250.357143</td>\n",
       "      <td>250.357143</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <td><a href=\"https://omar.website\" target=\"_blank\">https://omar.website</a></td>\n",
       "      <td>10</td>\n",
       "      <td>2493.0</td>\n",
       "      <td>249.300000</td>\n",
       "      <td>249.300000</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <td><a href=\"https://www.tablix.org\" target=\"_blank\">https://www.tablix.org</a></td>\n",
       "      <td>10</td>\n",
       "      <td>2492.0</td>\n",
       "      <td>249.200000</td>\n",
       "      <td>249.200000</td>\n",
       "    </tr>\n",
       "  </tbody>\n",
       "</table>"
      ],
      "text/plain": [
       "<IPython.core.display.HTML object>"
      ]
     },
     "metadata": {},
     "output_type": "display_data"
    }
   ],
   "source": [
    "df['domain'] = df['domain'].apply(lambda x: f'<a href=\"{x}\" target=\"_blank\">{x}</a>')\n",
    "display(HTML(df.to_html(escape=False, index=False)))"
   ]
  }
 ],
 "metadata": {
  "kernelspec": {
   "display_name": "Python 3",
   "language": "python",
   "name": "python3"
  },
  "language_info": {
   "codemirror_mode": {
    "name": "ipython",
    "version": 3
   },
   "file_extension": ".py",
   "mimetype": "text/x-python",
   "name": "python",
   "nbconvert_exporter": "python",
   "pygments_lexer": "ipython3",
   "version": "3.12.3"
  }
 },
 "nbformat": 4,
 "nbformat_minor": 2
}
